@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: ddev.c,v 1.6 98/03/06 08:04:22 abe Exp $";
+static char *rcsid = "$Id: ddev.c,v 1.7 99/01/25 06:52:39 abe Exp $";
 #endif
 
 
@@ -40,17 +40,8 @@ static char *rcsid = "$Id: ddev.c,v 1.6 98/03/06 08:04:22 abe Exp $";
 
 
 _PROTOTYPE(static int rmdupdev,(struct l_dev ***dp, int n, char *nm));
-_PROTOTYPE(static void stkdir,(char *p));
 
 
-/*
- * Local static values
- */
-
-static int Dn = 0;			/* directory stack entries allocated */
-static char **Dstk = (char **)NULL;	/* directory stack */
-int Dx = 0;				/* directory stack index */
-
 #if	defined(HASDCACHE) && AIXV>=4140
 
 
@@ -187,34 +178,39 @@ readdev(skip)
 	}
 #endif	/* AIXV >=4140 */
 
+	Dstk = (char **)NULL;
+	Dstkn = Dstkx = 0;
 	(void) stkdir("/dev");
 /*
  * Unstack the next /dev or /dev/<subdirectory> directory.
  */
-	while (--Dx >= 0) {
-	    if (!(dfp = opendir(Dstk[Dx]))) {
+	while (--Dstkx >= 0) {
+	    if (!(dfp = opendir(Dstk[Dstkx]))) {
 
 #if	defined(WARNDEVACCESS)
 		if (!Fwarn) {
 		    (void) fprintf(stderr, "%s: WARNING: can't open: ", Pn);
-		    safestrprt(Dstk[Dx], stderr, 1);
+		    safestrprt(Dstk[Dstkx], stderr, 1);
 		}
 #endif	/* defined(WARNDEVACCESS) */
 
+		(void) free((FREE_P *)Dstk[Dstkx]);
+		Dstk[Dstkx] = (char *)NULL;
 		continue;
 	    }
 	    if (path) {
 		(void) free((FREE_P *)path);
 		path = (char *)NULL;
 	    }
-	    if (!(path = mkstrcat(Dstk[Dx], -1, "/", 1, (char *)NULL, -1, &pl)))
+	    if (!(path = mkstrcat(Dstk[Dstkx], -1, "/", 1, (char *)NULL,
+				  -1, &pl)))
 	    {
 		(void) fprintf(stderr, "%s: no space for: ", Pn);
-		safestrprt(Dstk[Dx], stderr, 1);
+		safestrprt(Dstk[Dstkx], stderr, 1);
 		Exit(1);
 	    }
-	    (void) free((FREE_P *)Dstk[Dx]);
-	    Dstk[Dx] = (char *)NULL;
+	    (void) free((FREE_P *)Dstk[Dstkx]);
+	    Dstk[Dstkx] = (char *)NULL;
 	/*
 	 * Scan the directory.
 	 */
@@ -360,8 +356,11 @@ readdev(skip)
 /*
  * Free any allocated space.
  */
-	if (Dstk)
+	if (Dstk) {
 	    (void) free((FREE_P *)Dstk);
+	    Dstk = (char **)NULL;
+	    Dstkn = Dstkx = 0;
+	}
 	if (fp)
 	    (void) free((FREE_P *)fp);
 	if (path)
@@ -656,44 +655,6 @@ bad_cached_clone:
 #endif	/* defined(HASDCACHE) && AIXV>=4140 */
 
 
-/*
- * stkdir() - stack directory name
- */
-
-static void
-stkdir(p)
-	char *p;		/* directory path */
-{
-	MALLOC_S len;
-/*
- * Provide space for new directory stack entry.
- */
-	if (Dx >= Dn) {
-	    Dn += 10;
-	    len = (MALLOC_S)(Dn * sizeof(char *));
-	    if (!Dstk)
-		Dstk = (char **)malloc(len);
-	    else
-		Dstk = (char **)realloc((MALLOC_P *)Dstk, len);
-	    if (!Dstk) {
-		(void) fprintf(stderr,
-		    "%s: no space for directory stack at ", Pn);
-		safestrprt(p, stderr, 1);
-		Exit(1);
-	    }
-	}
-/*
- * Allocate space for the name, copy it there and put its pointer on the stack.
- */
-	if (!(Dstk[Dx] = mkstrcpy(p, (MALLOC_S *)NULL))) {
-	    (void) fprintf(stderr, "%s: no space for: ", Pn);
-	    safestrprt(p, stderr, 1);
-	    Exit(1);
-	}
-	Dx++;
-}
-
-
 #if	defined(HASDCACHE)
 /*
  * vfy_dev() - verify a device table entry (usually when DCunsafe == 1)
