@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.15 98/06/22 07:44:11 abe Exp $
+ * $Id: dlsof.h,v 1.17 10/.0/.1 .0:.0:.4 abe Exp $
  */
 
 
@@ -136,7 +136,13 @@ struct rnode {
 	caddr_t	r_d1[11];		/* dummies; links? */
 	struct vnode r_vnode;		/* vnode for remote file */
 	struct gnode r_gnode;		/* gnode for remote file */
+
+#  if	AIXV<4330
 	caddr_t r_d2[29];		/* dummies; rnode elements? */
+#  else	/* AIXV>=4330 */
+	caddr_t r_d2[26];		/* dummies; rnode elements? */
+#  endif	/* AIXV<4330 */
+
 	off_t r_size;			/* client's view of file size (long)*/
 	struct vattr r_attr;		/* cached vnode attributes */
 };
@@ -207,6 +213,21 @@ struct la {			/* loader anchor */
 # endif	/* AIXV>=4300 */
 
 # if	defined(HAS_AFS)
+
+/*
+ *  Avoid typdef conflicts in <afs/stds.h>.
+ */
+
+#  if	defined(HASINT16TYPE)
+#define	int16	AFS_int16
+#  endif	/* defined(HASINT16TYPE */
+#  if	defined(HASUINT16TYPE)
+#define	u_int16	AFS_u_int16
+#  endif	/* defined(HASUINT16TYPE */
+#  if	defined(HASINT32TYPE)
+#define int32	AFS_int32
+#  endif	/* defined(HASINT32TYPE) */
+
 #include <afs/stds.h>
 #include <afs/param.h>
 #include <afs/afsint.h>
@@ -271,7 +292,7 @@ typedef	off_t		KA_T;
 
 
 /*
- * Define for ckfa.c
+ * Definition for ckfa.c
  */
 
 #define	CKFA_MPXCHAN	1
@@ -281,12 +302,20 @@ typedef	off_t		KA_T;
  * Definitions for dvch.c
  */
 
-#if	AIXV>=4140
+# if	AIXV>=4140
 #define	DCACHE_CLONE	rw_clone_sect	/* clone function for read_dcache */
 #define	DCACHE_CLR	clr_sect	/* function to clear clone and
 					 * pseudo caches when reading the
 					 * device cache file fails */
-#endif	/* AIXV>=4140 */
+# endif	/* AIXV>=4140 */
+
+
+/*
+ * Definitions for enter_dir()
+ */
+
+#define	DIRTYPE	dirent
+#define	HASDNAMLEN	1
 
 
 # if	defined(HAS_AFS)
@@ -382,7 +411,7 @@ extern struct nlist AFSnl[];	/* AFS kernel symbol name list table */
 extern char *AFSApath;		/* alternate AFS name list path (from -a) */
 #  endif	/* defined(HASAOPT) */
 
-KA_T AFSVfsp;			/* AFS struct vfs kernel pointer */
+extern KA_T AFSVfsp;		/* AFS struct vfs kernel pointer */
 # endif	/* defined(HAS_AFS) */
 
 extern int Kd;
