@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.5 98/03/06 08:04:32 abe Exp Locker: abe $";
+static char *rcsid = "$Id: dnode.c,v 1.9 10/.0/.1 .0:.0:.4 abe Exp $";
 #endif
 
 
@@ -179,10 +179,16 @@ process_node(va)
 	    enter_nm(Namech);
 	    return;
 	}
+
+#if	defined(HASFSTRUCT)
+	Lf->fsv |= FSV_NI;
+	Lf->fna = va;
+#endif	/* defined(HASFSTRUCT) */
+
 /*
  * Read the gnode.
  */
-	if (!v->v_gnode || readgnode((caddr_t)v->v_gnode, &g)) {
+	if (!v->v_gnode || readgnode((KA_T)v->v_gnode, &g)) {
 	    (void) sprintf(Namech, "vnode at %s has no gnode\n",
 		print_kptr(va, (char *)NULL));
 	    enter_nm(Namech);
@@ -351,8 +357,14 @@ process_clone_again:
  */
 
 #if	defined(HAS_NFS)
-	if (vfs && vfs->vmt_flags & MNT_REMOTE)
-	    Ntype = N_NFS;
+	if (vfs && vfs->vmt_flags & MNT_REMOTE) {
+
+# if	defined(HAS_AFS)
+	    if (!AFSVfsp || (KA_T)v->v_vfsp != AFSVfsp)
+# endif	/* defined(HAS_AFS) */
+
+		Ntype = N_NFS;
+	}
 #endif	/* defined(HAS_NFS) */
 
 	Lf->lock = isglocked(&g);
@@ -569,6 +581,44 @@ process_clone_again:
 		break;
 	    }
 	}
+/*
+ * Record link count.
+ */
+	if (Fnlink) {
+	    switch(Ntype) {
+
+# if	defined(HAS_AFS)
+	    case N_AFS:
+		Lf->nlink = an.nlink;
+		Lf->nlink_def = an.nlink_st;
+		break;
+# endif	/* defined(HAS_AFS) */
+
+# if	defined(HAS_NFS)
+	    case N_NFS:
+		Lf->nlink = (long)r.r_attr.va_nlink;
+		Lf->nlink_def = 1;
+		break;
+# endif	/* defined(HAS_NFS) */
+
+# if	AIXV>=3200
+	    case N_BLK:				/* no link count? */
+		break;
+	    case N_CHR:
+	    case N_FIFO:
+	    case N_MPC:
+# endif	/* AIXV>=3200 */
+
+	    case N_REGLR:
+		if (ins) {
+		    Lf->nlink = (long)i.i_nlink;
+		    Lf->nlink_def = 1;
+		}
+		break;
+	    }
+	    if (Nlink && Lf->nlink_def && (Lf->nlink < Nlink))
+		Lf->sf |= SELNLINK;
+	}
 /*
  * Record an NFS file selection.
  */
