@@ -33,7 +33,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1996 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode1.c,v 1.3 98/03/10 16:24:07 abe Exp $";
+static char *rcsid = "$Id: dnode1.c,v 1.6 10/.0/.1 .0:.0:.5 abe Exp $";
 #endif
 
 
@@ -41,6 +41,31 @@ static char *rcsid = "$Id: dnode1.c,v 1.3 98/03/10 16:24:07 abe Exp $";
 #include "lsof.h"
 
 
+/*
+ * This is an emulation of the afs_rwlock_t definition that appears in
+ * the AFS sources in afs/lock.h.
+ */
+
+struct afs_lock {
+
+#  if	HAS_AFS<304
+	unsigned long d1[4];
+#  else	/* HAS_AFS>=304 */
+	unsigned char d1[2];
+	unsigned short d2[3];
+	struct timeval d3;
+	unsigned int d4[3];
+#  endif	/* HAS_AFS<304 */
+
+};
+typedef struct afs_lock afs_lock_t;
+typedef struct afs_lock afs_rwlock_t;
+
+#define	KERNEL
+#include <afs/afs.h>
+#undef	KERNEL
+
+
 /*
  * Local function prototypes
  */
@@ -244,13 +269,16 @@ readafsnode(va, v, an)
 	vc = (struct vcache *)v;
 	an->dev = AFSDEV;
 	an->size = (unsigned long)vc->m.Length;
+	an->nlink = (long)vc->m.LinkCount;
+	an->nlink_st  = 1;
 /*
  * Manufacture the "inode" number.
  */
 	if (vc->mvstat == 2) {
 	    if ((vp = getvolume(&vc->fid, &vols))) {
-		an->inode = (unsigned long)(vp->mtpoint.Fid.Vnode +
-					    (vp->mtpoint.Fid.Volume << 16));
+		an->inode = (unsigned long)((vp->mtpoint.Fid.Vnode
+			  +		     (vp->mtpoint.Fid.Volume << 16))
+			  & 0x7fffffff);
 		if (an->inode == (unsigned long)0) {
 		    if (is_rootFid(vc, &rfid))
 			an->ino_st = 1;
