@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.8 98/06/29 15:54:15 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.13 99/10/22 08:21:43 abe Exp $";
 #endif
 
 
@@ -186,7 +186,7 @@ gather_proc_info()
 	static struct fdsinfo *fds = (struct fdsinfo *)NULL;
 	MALLOC_S msz;
 	pid_t pid;
-#endif	/* AIXV4300 */
+#endif	/* AIXV>=4300 */
 
 /*
  * Read the process table.
@@ -235,9 +235,9 @@ gather_proc_info()
 			"%s: no more space for proc storage\n", Pn);
 		    Exit(1);
 		}
+		Np += PROCINFO_INCR;
 	    }
-	    p = (struct PROCINFO *)((char *)P + (Np * PROCSIZE));
-	    Np += PROCINFO_INCR;
+	    p = (struct PROCINFO *)((char *)P + (np * PROCSIZE));
 	}
 	if (i > 0)
 	    np += i;
@@ -395,8 +395,20 @@ gather_proc_info()
 		if (fp) {
 		    alloc_lfile((char *)NULL, i);
 		    process_file(fp);
-		    if (Lf->sf)
+		    if (Lf->sf) {
+
+#if	defined(HASFSTRUCT)
+			if (Fsv & FSV_FG)
+
+# if	AIXV<4300
+			    Lf->pof = (long)(Up->u_ufd[i].flags & 0x7f);
+#else	/* AIXV>=4300 */
+			    Lf->pof = (long)(fds->pi_ufd[i].flags & 0x7f);
+#endif	/* AIXV<4300 */
+#endif	/* defined(HASFSTRUCT) */
+
 			link_lfile();
+		    }
 		}
 	    }
 	/*
@@ -481,7 +493,13 @@ get_kernel_access()
 
 	    struct nlist ll[] = {
 		{ "library_anchor"	},
+
+# if	AIXV>=4330
+		{ "library_le_handle"	},
+# else	/* AIXV<4330 */
 		{ "library_data_handle"	},
+# endif	/* AIXV>=4330 */
+
 		{ (char *)NULL		}
 	    };
 	    struct ublock *ub;
@@ -659,10 +677,10 @@ process_text(sid)
 		    process_file((KA_T)xf);
 		    if (Lf->sf) {
 
-#if	AIXV>=4110
+#if	AIXV>=4110 && AIXV<4300
 			if (!Lf->nm || !Lf->nm[0])
 			    getlenm(le);
-#endif	/* AIXV>=4110 */
+#endif	/* AIXV>=4110 && AIXV<4300 */
 
 			link_lfile();
 		    }
