@@ -33,17 +33,26 @@
 
 
 /*
- * $Id: dproto.h,v 1.7 96/05/13 08:04:54 abe Exp $
+ * $Id: dproto.h,v 1.2 98/03/06 08:04:37 abe Exp $
  */
 
 
+#if     defined(HAS_AFS)
+_PROTOTYPE(extern struct vnode *alloc_vcache,(void));
+_PROTOTYPE(extern int hasAFS,(struct vnode *vp));
+_PROTOTYPE(extern int readafsnode,(KA_T va, struct vnode *v, struct afsnode *an));
+#endif  /* defined(HAS_AFS) */
+
 _PROTOTYPE(extern int getchan,(char *p));
 _PROTOTYPE(extern int is_file_named,(char *p, enum vtype ty, chan_t ch, int ic));
 _PROTOTYPE(extern char isglocked,(struct gnode *ga));
 _PROTOTYPE(extern struct l_vfs *readvfs,(struct vnode *vn));
 
-#if     defined(HAS_AFS)
-_PROTOTYPE(extern struct vnode *alloc_vcache,(void));
-_PROTOTYPE(extern int hasAFS,(struct vnode *vp));
-_PROTOTYPE(extern int readafsnode,(caddr_t va, struct vnode *v, struct afsnode *an));
-#endif  /* defined(HAS_AFS) */
+#if	AIXV>=4200
+_PROTOTYPE(extern void process_shmt,(KA_T sa));
+#endif	/* AIV>=4200 */
+
+#if	defined(HASDCACHE) && AIXV>=4140
+_PROTOTYPE(extern void clr_sect,(void));
+_PROTOTYPE(extern int rw_clone_sect,(int m));
+#endif	/* defined(HASDCACHE) && AIXV>=4140 */
