@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.7 98/08/03 07:40:29 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.10 99/11/28 06:39:04 abe Exp $";
 #endif
 
 
@@ -122,10 +122,21 @@ process_socket(sa)
  * Process an Internet domain socket.
  */
 	case AF_INET:
+
+#if	defined(HASIPv6)
+	case AF_INET6:
+#endif	/* defined(HASIPv6) */
+
 	    if (Fnet)
 		Lf->sf |= SELNET;
-	    (void) strcpy(Lf->type, "inet");
 	    printiproto(p.pr_protocol);
+
+#if	defined(HASIPv6)
+	    (void) strcpy(Lf->type, (fam == AF_INET) ? "IPv4" : "IPv6");
+#else	/* !defined(HASIPv6) */
+	    (void) strcpy(Lf->type, "inet");
+#endif	/* defined(HASIPv6) */
+
 	/*
 	 * Read protocol control block.
 	 */
@@ -142,14 +153,53 @@ process_socket(sa)
 	    enter_dev_ch(print_kptr((KA_T)(inp.inp_ppcb ? inp.inp_ppcb
 							: s.so_pcb),
 				    (char *)NULL));
-	    la = (unsigned char *)&inp.inp_laddr;
+	/*
+	 * Select local address.
+	 */
+	    if (fam == AF_INET)
+		la = (unsigned char *)&inp.inp_laddr;
+
+#if	defined(HASIPv6)
+	    else if (fam == AF_INET6)
+		la = (unsigned char *)&inp.inp_laddr6;
+#endif	/* defined(HASIPv6) */
+
 	    lp = (int)ntohs(inp.inp_lport);
-	    if (inp.inp_faddr.s_addr != INADDR_ANY || inp.inp_fport != 0) {
-		fa = (unsigned char *)&inp.inp_faddr;
-		fp = (int)ntohs(inp.inp_fport);
+	/*
+	 * Select foreign (remote) address.
+	 */
+	    if (fam == AF_INET) {
+		if (inp.inp_faddr.s_addr != INADDR_ANY || inp.inp_fport) {
+		    fa = (unsigned char *)&inp.inp_faddr;
+		    fp = (int)ntohs(inp.inp_fport);
+		}
 	    }
+
+#if	defined(HASIPv6)
+	    else if (fam == AF_INET6) {
+		if (!IN6_IS_ADDR_UNSPECIFIED(&inp.inp_faddr6)
+		||   inp.inp_fport)
+		{
+		    fa = (unsigned char *)&inp.inp_faddr6;
+		    fp = (int)ntohs(inp.inp_fport);
+		}
+		if ((la && IN6_IS_ADDR_V4MAPPED((struct in6_addr *)la))
+		||  (fa && IN6_IS_ADDR_V4MAPPED((struct in6_addr *)fa))) {
+		    Lf->type[3] = '4';
+		    fam = AF_INET;
+		    if (la)
+			la = (unsigned char *)&inp.inp_laddr;
+		    if (fa)
+			fa = (unsigned char *)&inp.inp_faddr;
+		}
+	    }
+#endif	/* defined(HASIPv6) */
+
+	/*
+	 * Save addresses.
+	 */
 	    if (fa || la)
-		(void) ent_inaddr(la, lp, fa, fp, AF_INET);
+		(void) ent_inaddr(la, lp, fa, fp, fam);
 	    if (p.pr_protocol == IPPROTO_TCP && inp.inp_ppcb
 	    &&  !kread((KA_T)inp.inp_ppcb, (char *)&t, sizeof(t))) {
 		Lf->lts.type = 0;
@@ -230,15 +280,15 @@ process_socket(sa)
 		if (mb.m_len >= sizeof(struct sockaddr_un))
 		    mb.m_len = sizeof(struct sockaddr_un) - 1;
 		*((char *)ua + mb.m_len) = '\0';
-		if (Sfile && is_file_named(ua->sun_path, VSOCK))
+		if (Sfile && is_file_named(ua->sun_path, 0))
 		    Lf->sf |= SELNM;
-		else
+		if (!Namech[0])
 		    (void) strcpy(Namech, ua->sun_path);
 	    } else
 		(void) strcpy(Namech, "no address");
 	    break;
 	default:
-	    printunkaf(fam);
+	    printunkaf(fam, 1);
 	}
 	if (Namech[0])
 	    enter_nm(Namech);
