@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dstore.c,v 1.4 98/03/06 08:07:49 abe Exp $";
+static char *rcsid = "$Id: dstore.c,v 1.7 99/05/09 14:55:25 abe Exp $";
 #endif
 
 
@@ -69,4 +69,39 @@ struct l_vfs *Lvfs = NULL;	/* local vfs structure table */
 
 int Np = 0;			/* number of kernel processes */
 struct kinfo_proc *P = NULL;	/* local process table copy */
+
+#if	defined(HASFSTRUCT)
+/*
+ * Pff_tab[] - table for printing file flags
+ */
+
+struct pff_tab Pff_tab[] = {
+	{ (long)FREAD,		FF_READ		},
+	{ (long)FWRITE,		FF_WRITE	},
+	{ (long)FNONBLOCK,	FF_NBLOCK	},
+	{ (long)FNDELAY,	FF_NDELAY	},
+	{ (long)FAPPEND,	FF_APPEND	},
+	{ (long)FASYNC,		FF_ASYNC	},
+	{ (long)FFSYNC,		FF_FSYNC	},
+	{ (long)FMARK,		FF_MARK		},
+	{ (long)FHASLOCK,	FF_HASLOCK	},
+	{ (long)FDEFER,		FF_DEFER	},
+	{ (long)O_SHLOCK,	FF_SHLOCK	},
+	{ (long)O_EXLOCK,	FF_EXLOCK	},
+	{ (long)0,		NULL 		}
+};
+
+
+/*
+ * Pof_tab[] - table for print process open file flags
+ */
+
+struct pff_tab Pof_tab[] = {
+
+	{ (long)UF_EXCLOSE,	POF_CLOEXEC	},
+	{ (long)UF_MAPPED,	POF_MAPPED	},
+	{ (long)0,		NULL		}
+};
+#endif	/* defined(HASFSTRUCT) */
+
 int pgshift = 0;		/* kernel's page shift */
