@@ -1,5 +1,6 @@
 /*
- * ddev.c - Digital UNIX (DEC OSF/1) device support functions for lsof
+ * ddev.c - DEC OSF/1, Digital UNIX, Tru64 UNIX device support functions for
+ *	    lsof
  */
 
 
@@ -32,32 +33,24 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: ddev.c,v 1.7 98/03/18 15:08:06 abe Exp $";
+static char *rcsid = "$Id: ddev.c,v 1.11 99/06/22 08:13:44 abe Exp $";
 #endif
 
-
 #include "lsof.h"
 
 
-#if	defined(USELOCALREADDIR)
-_PROTOTYPE(static struct dirent *ReadDir,(DIR *dirp));
-#else	/* !defined(USELOCALREADDIR) */
-#define	ReadDir	readdir
-#endif	/* defined(USELOCALREADDIR) */
-
-_PROTOTYPE(static int rmdupdev,(struct l_dev ***dp, int n, char *nm));
-_PROTOTYPE(static void stkdir,(char ***d, int *n, int *x, char *p));
-
-
 /*
- * Local static definitions
+ * Local static values
  */
 
 #if	defined(USELOCALREADDIR)
-static struct stat DirSb;
+static struct stat Dirsb;
 #endif	/* defined(USELOCALREADDIR) */
 
 
+_PROTOTYPE(static int rmdupdev,(struct l_dev ***dp, int n, char *nm));
+
+
 #if	defined(HASDCACHE)
 /*
  * clr_sect() - clear cached clone and pseudo sections
@@ -91,7 +84,7 @@ printchdevname(rdev, f)
 	struct clone *c;
 	struct l_dev *dp;
 
-	readdev();
+	readdev(0);
 /*
  * Search for clone.
  */
@@ -102,7 +95,7 @@ printchdevname_again:
 
 #endif  /* defined(HASDCACHE) */
 
-	if (Clone && major(*rdev) == major(Clonedev)) {
+	if (Clone && HAVECLONEMAJ && major(*rdev) == CLONEMAJ) {
 	    for (c = Clone; c; c = c->next) {
 		if (Devtp[c->dx].rdev == *rdev) {
 
@@ -153,10 +146,7 @@ readdev(skip)
 
 	struct clone *c;
 	DIR *dfp;
-	int dn = 0;
-	struct dirent *dp;
-	char **dstk = (char **)NULL;
-	int dx = 0;
+	struct DIRTYPE *dp;
 	char *fp = (char *)NULL;
 	int i = 0;
 
@@ -185,48 +175,39 @@ readdev(skip)
 	    dcrd = 1;
 #endif	/* defined(HASDCACHE) */
 
-	(void) stkdir(&dstk, &dn, &dx, "/dev");
+	Dstk = (char **)NULL;
+	Dstkn = Dstkx = 0;
+	(void) stkdir("/dev");
 /*
  * Unstack the next /dev or /dev/<subdirectory> directory.
  */
-	while (--dx >= 0) {
-	    if (!(dfp = opendir(dstk[dx]))) {
+	while (--Dstkx >= 0) {
+	    if (!(dfp = OpenDir(Dstk[Dstkx]))) {
 
 #if	defined(WARNDEVACCESS)
 		if (!Fwarn) {
 		    (void) fprintf(stderr, "%s: WARNING: can't open: ", Pn);
-		    safestrprt(dstk[dx], stderr, 1);
+		    safestrprt(Dstk[Dstkx], stderr, 1);
 		}
 #endif	/* defined(WARNDEVACCESS) */
 
+		(void) free((FREE_P *)Dstk[Dstkx]);
+		Dstk[Dstkx] = (char *)NULL;
 		continue;
 	    }
-
-#if	defined(USELOCALREADDIR)
-	    if (fstat(dfp->dd_fd, &DirSb) < 0) {
-		if (!Fwarn) {
-		    int errno_save = errno;
-
-		    (void) fprintf(stderr, "%s: WARNING: can't stat ", Pn);
-		    safestrprt(dstk[dx], stderr, 0);
-		    (void) fprintf(stderr, ": %s\n", strerror(errno_save));
-		}
-		continue;
-	    }
-#endif	/* defined(USELOCALREADDIR) */
-
 	    if (path) {
 		(void) free((FREE_P *)path);
 		path = (char *)NULL;
 	    }
-	    if (!(path = mkstrcat(dstk[dx], -1, "/", 1, (char *)NULL, -1, &pl)))
+	    if (!(path = mkstrcat(Dstk[Dstkx], -1, "/", 1, (char *)NULL, -1,
+				  &pl)))
 	    {
 		(void) fprintf(stderr, "%s: no space for: ", Pn);
-		safestrprt(dstk[dx], stderr, 1);
+		safestrprt(Dstk[Dstkx], stderr, 1);
 		Exit(1);
 	    }
-	    (void) free((FREE_P *)dstk[dx]);
-	    dstk[dx] = (char *)NULL;
+	    (void) free((FREE_P *)Dstk[Dstkx]);
+	    Dstk[Dstkx] = (char *)NULL;
 	/*
 	 * Scan the directory.
 	 */
@@ -275,7 +256,7 @@ readdev(skip)
 	     * If it's a subdirectory, stack its name for later processing.
 	     */
 		if ((sb.st_mode & S_IFMT) == S_IFDIR) {
-		    (void) stkdir(&dstk, &dn, &dx, fp);
+		    (void) stkdir(fp);
 		    continue;
 		}
 		if ((sb.st_mode & S_IFMT) == S_IFCHR) {
@@ -309,12 +290,12 @@ readdev(skip)
 		/*
 		 * Save clone device location.
 		 */
-		    if (major(Devtp[i].rdev) == major(Clonedev)) {
+		    if (HAVECLONEMAJ && major(Devtp[i].rdev) == CLONEMAJ) {
 			if (!(c = (struct clone *)malloc(sizeof(struct clone))))
 			{
 			    (void) fprintf(stderr,
-				"%s: no space for clone device: %s\n", Pn);
-			    safestrprt(Devtp[i].name, stderr, 1);
+				"%s: no space for clone device: ", Pn);
+			    safestrprt(fp, stderr, 1);
 			    Exit(1);
 			}
 			c->dx = i;
@@ -355,13 +336,16 @@ readdev(skip)
 #endif	/* defined(HASBLKDEV) */
 
 	    }
-	    (void) closedir(dfp);
+	    (void) CloseDir(dfp);
 	}
 /*
  * Free any allocated space.
  */
-	if (!dstk)
-	    (void) free((FREE_P *)dstk);
+	if (Dstk) {
+	    (void) free((FREE_P *)Dstk);
+	    Dstk = (char **)NULL;
+	    Dstkn = Dstkx = 0;
+	}
 	if (fp)
 	    (void) free((FREE_P *)fp);
 	if (path)
@@ -504,56 +488,104 @@ readdev(skip)
  * - The BROKEN_GETDIRENTRIES symbol was deleted.  Use of this function
  *   is controlled in the lsof distribution by the HASLOCALREADDIR
  *   definition.
- *
- * - The buffer from an fstat() of the directory is supplied in the
- *   static stat structure DirSb.
  */
 
 
 /*
- * get next entry in a directory.
+ * CloseDir() - close directory
  */
 
-static struct dirent *
+int
+CloseDir(dirp)
+	register DIR *dirp;
+{
+	return(closedir(dirp));
+}
+
+
+/*
+ * OpenDir() - open directory
+ */
+
+DIR *
+OpenDir(dir)
+	char *dir;
+{
+	DIR *dirp;
+
+	if ((dirp = opendir(dir))) {
+
+	/*
+	 * Get a stat(2) buffer for the directory.
+	 *
+	 * Warn if the stat(2) buffer operation fails, close the directory,
+	 * and respond that the open failed.
+	 */
+	    if (statsafely(dir, &Dirsb)) {
+		int en = errno;
+
+		if (!Fwarn) {
+		    (void) fprintf(stderr,
+			"%s: WARNING: can't statsafely(", Pn);
+		    safestrprt(dir, stderr, 0);
+		    (void) fprintf(stderr, "): %s\n", strerror(en));
+		}
+		(void) CloseDir(dirp);
+		dirp = (DIR *)NULL;
+	 	errno = en;
+	    }
+	}
+	return(dirp);
+}
+
+
+/*
+ * ReadDir() - read next directory entry
+ */
+
+extern struct DIRTYPE *
 ReadDir(dirp)
 	register DIR *dirp;
 {
-	register struct dirent *dp;
+	register struct DIRTYPE *dp;
 
+/*
+ * Loop through the directory.
+ */
 	for (;;) {
-		if (dirp->dd_loc >= dirp->dd_size) {
-			dirp->dd_loc = 0;
-		}
-		if (dirp->dd_loc == 0) {
-			dirp->dd_size = getdirentries(dirp->dd_fd,
-			    dirp->dd_buf, dirp->dd_bufsiz, &dirp->dd_seek);
+	    if (dirp->dd_loc >= dirp->dd_size) {
+		dirp->dd_loc = 0;
+	    }
+	    if (dirp->dd_loc == 0) {
+	    	dirp->dd_size = getdirentries(dirp->dd_fd,
+			        dirp->dd_buf, dirp->dd_bufsiz, &dirp->dd_seek);
 
-			if (dirp->dd_size <= 0)
-				return((struct dirent *)NULL);
-		/*
-		 * If the size returned by getdirentries() exceeds what it
-		 * should be (as determined by an fstat() of the directory),
-		 * set it to the proper value.  This is an adjustment for an
-		 * apparent bug in the Digital UNIX 3.[02] getdirentries()
-		 * function, when applied to a /dev/fd mount point.
-		 *
-		 * This check was conceived by Duncan McEwan and modified by
-		 * Vic Abell.
-		 */
-			if (dirp->dd_size > (long)DirSb.st_size)
-				dirp->dd_size = (long)DirSb.st_size;
-			DirSb.st_size -= (off_t)dirp->dd_size;
-		}
-		dp = (struct dirent *)(dirp->dd_buf + dirp->dd_loc);
-		if ((long)dp & 03)	/* bogus pointer check */
-			return((struct dirent *)NULL);
-		if (dp->d_reclen <= 0 ||
-		    dp->d_reclen > dirp->dd_bufsiz + 1 - dirp->dd_loc)
-			return((struct dirent *)NULL);
-		dirp->dd_loc += dp->d_reclen;
-		if (dp->d_ino == 0)
-			continue;
-		return (dp);
+		if (dirp->dd_size <= 0)
+		    return((struct DIRTYPE *)NULL);
+	    /*
+	     * If the size returned by getdirentries() exceeds what it
+	     * should be (as determined by a stat(2) of the directory),
+	     * set it to the proper value.  This is an adjustment for an
+	     * apparent bug in the Digital UNIX 3.[02] getdirentries()
+	     * function, when applied to a /dev/fd mount point.
+	     *
+	     * This check was conceived by Duncan McEwan and modified by
+	     * Vic Abell.
+	     */
+		if (dirp->dd_size > (long)Dirsb.st_size)
+		    dirp->dd_size = (long)Dirsb.st_size;
+		Dirsb.st_size -= (off_t)dirp->dd_size;
+	    }
+	    dp = (struct DIRTYPE *)(dirp->dd_buf + dirp->dd_loc);
+	    if ((long)dp & 03)		/* bogus pointer check */
+		return((struct DIRTYPE *)NULL);
+	    if (dp->d_reclen <= 0
+	    ||  dp->d_reclen > dirp->dd_bufsiz + 1 - dirp->dd_loc)
+		return((struct DIRTYPE *)NULL);
+	    dirp->dd_loc += dp->d_reclen;
+	    if (dp->d_ino == 0)
+		continue;
+	    return (dp);
 	}
 }
 #endif	/* defined(USELOCALREADDIR) */
@@ -576,132 +608,6 @@ rereaddev()
 	readdev(1);
 	DCunsafe = 0;
 }
-
-
-/*
- * rw_clone_sect() - read/write the device cache file clone section
- */
-
-int
-rw_clone_sect(m)
-	int m;				/* mode: 1 = read; 2 = write */
-{
-	char buf[MAXPATHLEN*2], *cp, *cp1;
-	struct clone *c;
-	struct l_dev *dp;
-	int i, j, len, n;
-
-	if (m == 1) {
-
-	/*
-	 * Read the clone section header and validate it.
-	 */
-	    if (!fgets(buf, sizeof(buf), DCfs)) {
-
-bad_clone_sect:
-		if (!Fwarn) {
-		    (void) fprintf(stderr,
-			"%s: bad clone section header in %s: ",
-			Pn, DCpath[DCpathX]);
-		    safestrprt(buf, stderr, 1);
-		}
-		return(1);
-	    }
-	    (void) crc(buf, strlen(buf), &DCcksum);
-	    len = strlen("clone section: ");
-	    if (strncmp(buf, "clone section: ", len) != 0)
-		goto bad_clone_sect;
-	    if ((n = atoi(&buf[len])) < 0)
-		goto bad_clone_sect;
-	/*
-	 * Read the clone section lines and create the Clone list.
-	 */
-	    for (i = 0; i < n; i++) {
-		if (!fgets(buf, sizeof(buf), DCfs)) {
-		    if (!Fwarn) {
-			(void) fprintf(stderr,
-			    "%s: bad clone line in %s: ", Pn, DCpath[DCpathX]);
-			safestrprt(buf, stderr, 1);
-		    }
-		    return(1);
-		}
-		(void) crc(buf, strlen(buf), &DCcksum);
-	    /*
-	     * Assemble Devtp[] index and make sure it's correct.
-	     */
-		for (cp = buf, j = 0; *cp != ' '; cp++) {
-		    if (*cp < '0' || *cp > '9') {
-
-bad_clone_index:
-			if (!Fwarn) {
-			    (void) fprintf(stderr,
-				"%s: bad cached clone device index: ", Pn);
-			    safestrprt(buf, stderr, 1);
-			}
-			return(1);
-		    }
-		    j = (j * 10) + (int)(*cp - '0');
-		}
-		if (j < 0 || j >= Ndev || !(cp1 = strchr(++cp, '\n')))
-		    goto bad_clone_index;
-		if (strncmp(cp, Devtp[j].name, (cp1 - cp)) != 0)
-		    goto bad_clone_index;
-	    /*
-	     * Allocate and complete a clone structure.
-	     */
-		if (!(c = (struct clone *)malloc(sizeof(struct clone)))) {
-		    (void) fprintf(stderr,
-			"%s: no space for cached clone: ", Pn);
-		    safestrprt(buf, stderr, 1);
-		    Exit(1);
-		}
-		c->dx = j;
-		c->next = Clone;
-		Clone = c;
-	    }
-	    return(0);
-	} else if (m == 2) {
-
-	/*
-	 * Write the clone section header.
-	 */
-	    for (c = Clone, n = 0; c; c = c->next, n++)
-		;
-	    (void) sprintf(buf, "clone section: %d\n", n);
-	    if (wr2DCfd(buf, &DCcksum))
-		return(1);
-	/*
-	 * Write the clone section lines.
-	 */
-	    for (c = Clone; c; c = c->next) {
-		for (dp = &Devtp[c->dx], j = 0; j < Ndev; j++) {
-		    if (dp == Sdev[j])
-			break;
-		}
-		if (j >= Ndev) {
-		    if (!Fwarn) {
-			(void) fprintf(stderr,
-			    "%s: can't make index for clone: ", Pn);
-			safestrprt(dp->name, stderr, 1);
-		    }
-		    (void) unlink(DCpath[DCpathX]);
-		    (void) close(DCfd);
-		    DCfd = -1;
-		    return(1);
-		}
-		(void) sprintf(buf, "%d %s\n", j, dp->name);
-		if (wr2DCfd(buf, &DCcksum))
-		    return(1);
-	    }
-	    return(0);
-	}
-/*
- * A shouldn't-happen case: mode neither 1 nor 2.
- */
-	(void) fprintf(stderr, "%s: internal rw_clone_sect error: %d\n",
-	    Pn, m);
-	Exit(1);
-}
 #endif	/* defined(HASDCACHE) */
 
 
@@ -759,47 +665,6 @@ rmdupdev(dp, n, nm)
 }
 
 
-/*
- * stkdir() - stack directory name
- */
-
-static void
-stkdir(d, n, x, p)
-	char ***d;		/* array of directory pointers */
-	int *n;			/* number of pointers */
-	int *x;			/* current index */
-	char *p;		/* directory path */
-{
-	MALLOC_S len;
-/*
- * Provide adequate space for stack pointers.
- */
-	if (*x >= *n) {
-	    *n += 10;
-	    len = (MALLOC_S)(*n * sizeof(char *));
-	    if (!*d)
-		*d = (char **)malloc(len);
-	    else
-		*d = (char **)realloc((MALLOC_P *)*d, len);
-	    if (!*d) {
-		(void) fprintf(stderr,
-		    "%s: no space for directory stack at: ", Pn);
-		safestrprt(p, stderr, 1);
-		Exit(1);
-	    }
-	}
-/*
- * Allocate space for the name, copy it there and put its pointer on the stack.
- */
-	if (!((*d)[*x] = mkstrcpy(p, (MALLOC_S *)NULL))) {
-	    (void) fprintf(stderr, "%s: no space for: ", Pn);
-	    safestrprt(p, stderr, 1);
-	    Exit(1);
-	}
-	*x += 1;
-}
-
-
 #if	defined(HASDCACHE)
 /*
  * vfy_dev() - verify a device table entry (usually when DCunsafe == 1)
