@@ -1,5 +1,5 @@
 /*
- * dlsof.h - Digital UNIX (DEC OSF/1) header file for lsof
+ * dlsof.h - DEC OSF/1, Digital UNIX, Tru64 UNIX header file for lsof
  */
 
 
@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.12 98/04/29 13:39:05 abe Exp $
+ * $Id: dlsof.h,v 1.20 99/07/08 14:52:23 abe Exp $
  */
 
 
@@ -41,9 +41,9 @@
 #include <fcntl.h>
 #include <fstab.h>
 
-# if	DUV<30000
+# if	DUV<30000 || DUV>=50000
 #include <sys/mount.h>
-# endif	/* DUV<30000 */
+# endif	/* DUV<30000 || DUV>=50000 */
 
 #include <dirent.h>
 #include <nlist.h>
@@ -117,7 +117,9 @@
 #undef	m_next
 #include <sys/file.h>
 #include <sys/fifonode.h>
+#  if	DUV<50000
 #include <sys/mount.h>
+#  endif	/* DUV<50000 */
 # endif	/* DUV>=30000 */
 
 #undef	calloc
@@ -129,6 +131,14 @@
 #define	pt_entry_t ___pt_entry_t
 #undef	timer_t
 #define	timer_t	___timer_t
+
+# if	DUV>=50000
+#include "du5_sys_malloc.h"
+#undef	_SYS_WAIT_H_			/* allow <sys/user.h> to
+					 * #include <sys/wait.h> while
+					 * and _KERNEL is defined */
+# endif	/* DUV>=50000 */
+
 #include <kern/task.h>
 #undef	___calloc
 #define	___calloc calloc
@@ -151,7 +161,15 @@
 #include <sys/vnode.h>
 
 # if	DUV>=30000
+#  if	DUV>=50000
+#undef	_KERNEL			/* to avoid <sys/vfs_proto.h> and
+				 * <ufs/quota.h> conflict over the
+				 * quotactl() function prototype */
+#  endif	/* DUV>=50000 && defined(_KERNEL) */
 #include <nfs/nfs.h>
+#  if	DUV>=50000
+#define	_KERNEL	1
+#  endif	/* DUV>=50000 && defined(_KERNEL) */
 # endif	/* DUV>=30000 */
 
 #include <nfs/rnode.h>
@@ -200,10 +218,23 @@ struct advfsnode {
 
 	ino_t	a_number;
 	int	a_seq;
+
+# if	ADVFSV>=500
+	long	d3[3];
+# else	/* ADVFSV<500 */
 	long	d3;
+# endif	/* ADVFSV>=500 */
+
 	int	d4;
 	dev_t	a_rdev;
 	long	a_size;
+
+# if	ADVFSV>=400
+	long	d5[5];
+	int	d6;
+	int	a_nlink;
+# endif	/* ADVFSV>=400 */
+
 };
 
 # if	defined(HASTAGTOPATH)
@@ -220,9 +251,16 @@ struct advfsnode {
 # endif	/* defined(HASTAGTOPATH) */
 
 
+# if	DUV<50000
 #define	COMP_P		void
-#define DEVINCR		1024	/* device table malloc() increment */
+typedef	unsigned long	KA_T;
+# else	/* DUV>=50000 */
+#define	COMP_P		const void
 typedef	off_t		KA_T;
+#endif	/* DUV<50000 */
+
+#define DEVINCR		1024	/* device table malloc() increment */
+#define	DIRTYPE		dirent
 #define	KMEM		"/dev/kmem"
 #define MALLOC_P	char
 #define FREE_P		MALLOC_P
@@ -242,17 +280,10 @@ typedef	off_t		KA_T;
  * Global storage definitions (including their structure definitions)
  */
 
-struct clone {
-	int dx;			/* index of device entry in Devtp[] */
-	struct clone *next;	/* forward link */
-
-};
-extern struct clone *Clone;
-
-extern dev_t Clonedev;
-
+extern int CloneMaj;
 extern struct file *Fileptr;
 #define	FILEPTR	Fileptr			/* for process_file() in lib/prfp.c */
+extern int	HaveCloneMaj;
 extern int Kd;
 
 struct l_vfs {
@@ -317,13 +348,16 @@ extern int Vnmxp;
 
 
 /*
- * Definitions for dvch.c
+ * Definitions for dvch.c, isfn.c, and rdev.c
  */
 
-#define	DCACHE_CLONE	rw_clone_sect	/* clone function for read_dcache */
+#define	CLONEMAJ	CloneMaj	/* clone major variable name */
 #define	DCACHE_CLR	clr_sect	/* function to clear clone cache
 					 * when reading the device cache
 					 * file fails */
+#define	HASDNAMLEN	1		/* DIRTYPE has d_namlen element */
+#define	HAS_STD_CLONE	1		/* has standard clone structure */
+#define	HAVECLONEMAJ	HaveCloneMaj	/* clone major variable status name */
 
 
 /*
