@@ -1,5 +1,6 @@
 /*
- * dmnt.c - Digital UNIX (DEC OSF/1) mount support functions for lsof
+ * dmnt.c - DEC OSF/1, Digital UNIX, Tru64 UNIX mount support functions for
+ *	    lsof
  */
 
 
@@ -32,7 +33,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dmnt.c,v 1.7 98/08/03 07:44:51 abe Exp $";
+static char *rcsid = "$Id: dmnt.c,v 1.8 99/04/15 06:47:20 abe Exp $";
 #endif
 
 
