@@ -1,5 +1,6 @@
 /*
- * dproc.c - Digital UNIX (DEC OSF/1) process access functions for lsof
+ * dproc.c - DEC OSF/1, Digital UNIX, Tru64 UNIX process access functions for
+ *	     lsof
  */
 
 
@@ -32,7 +33,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.7 98/09/14 13:38:18 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.13 99/06/28 08:08:14 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -126,15 +127,14 @@ enter_vn_text(va, n)
 void
 gather_proc_info()
 {
+	MALLOC_S b;
 	struct file *fp;
 	int i, j;
-	static int nufb = 0;
 	struct pgrp pg;
 	int pgrp, px;
 	struct proc *p;
 	short pss, sf;
 	struct ucred pcred;
-	static struct file **ufb = NULL;
 	uid_t uid;
 	struct utask ut, *utp;
 
@@ -142,6 +142,23 @@ gather_proc_info()
 	struct pid_entry pe;
 #endif	/* DUV>=30000 */
 
+#if	DUV<50000
+	static int nufb = 0;
+	static struct file **ufb = (struct file **)NULL;
+#else	/* DUV>=50000 */
+	int k, l;
+	KA_T ka;
+	struct ufile_entry *ofb[U_FE_OF_ALLOC_SIZE];
+	struct ufile_entry ufe[U_FE_ALLOC_SIZE];
+	struct ufile_entry ufeo[U_FE_OF_ALLOC_SIZE];
+#endif	/* DUV<50000 */
+
+#if	defined(HASFSTRUCT) && DUV>=40000
+	static char *pof = (char *)NULL;
+	static int pofb = 0;
+	long pv;
+#endif	/* defined(HASFSTRUCT) && DUV>=40000 */
+
 #if     defined(HASNCACHE)
 /*
  * Read kernel name cache.
@@ -228,49 +245,159 @@ gather_proc_info()
 	/*
 	 * Save information on file descriptors.
 	 */
+
+#if	DUV<50000
 	    for (i = j = 0; i <= utp->uu_file_state.uf_lastfile; i++) {
-		if (i < NOFILE_IN_U)
+		if (i < NOFILE_IN_U) {
 		    fp = utp->uu_file_state.uf_ofile[i];
-		else {
-		    if (j == 0) {
-			if (!ufb) {
-			    nufb = utp->uu_file_state.uf_of_count;
-			    if (!(ufb = (struct file **)malloc((MALLOC_S)
-				    (nufb * sizeof(struct file *)))))
-			    {
+
+# if	defined(HASFSTRUCT) && DUV>=40000
+		    if (Fsv & FSV_FG)
+			pv = (long)utp->uu_file_state.uf_pofile[i];
+# endif	/* defined(HASFSTRUCT) && DUV>=40000 */
+
+		} else {
+		    if (!j) {
+			b = (MALLOC_S)(utp->uu_file_state.uf_of_count
+			  *	       sizeof(struct file *));
+			if (b > nufb) {
+			    if (!ufb)
+				ufb = (struct file **)malloc(b);
+			    else
+				ufb = (struct file **)realloc((MALLOC_P *)ufb,
+							      b);
+			    if (!ufb) {
 				(void) fprintf(stderr,
 				    "%s: PID %d, no file * space\n",
 				    Pn, Lp->pid);
 				Exit(1);
 			    }
-			} else if (utp->uu_file_state.uf_of_count > nufb) {
-			    nufb = utp->uu_file_state.uf_of_count;
-			    if (!(ufb = (struct file **)realloc(
-					(MALLOC_P *)ufb,
-					(nufb * sizeof(struct file *)))))
-			    {
-				(void) fprintf(stderr,
-				    "%s: PID %d, no realloc file * space\n",
-				    Pn, Lp->pid);
-				Exit(1);
-			    }
+			    nufb = b;
 			}
 			if (kread((KA_T)utp->uu_file_state.uf_ofile_of,
-			    (char *)ufb, utp->uu_file_state.uf_of_count
-			    * sizeof(struct file *)))
-			{
-			    break;
+			    (char *)ufb, b))
+				break;
+
+# if	defined(HASFSTRUCT) && DUV>=40000
+			if (Fsv & FSV_FG) {
+			    b = (MALLOC_S)(utp->uu_file_state.uf_of_count
+			      *		   sizeof(char));
+			    if (b > pofb) {
+				if (!pof)
+				    pof = (char *)malloc(b);
+				else
+				    pof = (char *)realloc((MALLOC_P *)pof, b);
+				if (!pof) {
+				    (void) fprintf(stderr,
+					"%s: PID %d: no file flags space\n",
+					Pn, Lp->pid);
+				    Exit(1);
+				}
+				pofb = b;
+			    }
+			    if (kread((KA_T)utp->uu_file_state.uf_pofile_of,
+				      pof, b))
+				zeromem(pof, b);
 			}
+# endif	/* defined(HASFSTRUCT) && DUV>=40000 */
+
 		    }
-		    fp = ufb[j++];
+		    fp = ufb[j];
+
+# if	defined(HASFSTRUCT) && DUV>=40000
+		    if (Fsv & FSV_FG)
+			pv = pof[j];
+# endif	/* defined(HASFSTRUCT) && DUV>=40000 */
+
+		    j++;
 		}
 		if (fp && (ulong)fp != 0xffffffffffffffff) {
 		    alloc_lfile(NULL, i);
 		    process_file((KA_T)fp);
-		    if (Lf->sf)
+		    if (Lf->sf) {
+
+# if	defined(HASFSTRUCT) && DUV>=40000
+			if (Fsv & FSV_FG) {
+			    if ((Lf->pof = pv))
+				Lf->fsv |= FSV_FG;
+			}
+# endif	/* defined(HASFSTRUCT) && DUV>=40000 */
+
 			link_lfile();
+		    }
 		}
 	    }
+#else	/* DUV>=50000 */
+	    for (i = j = k = 0; i <= utp->uu_file_state.uf_lastfile; i++) {
+		if (i < NOFILE_IN_U) {
+		    if (!k) {
+			l = i/U_FE_ALLOC_SIZE;
+		        if (!(ka = (KA_T)utp->uu_file_state.uf_entry[l])) {
+			    i += U_FE_ALLOC_SIZE - 1;
+			    continue;
+		        }
+		    }
+		} else {
+		    if (!j) {
+			ka = (KA_T)utp->uu_file_state.uf_of_entry;
+			if (!ka || kread(ka, (char *)&ofb, sizeof(ofb)))
+			    break;
+			k = 0;
+		    }
+		    if (!k) {
+			l = j/U_FE_OF_ALLOC_SIZE;
+			if (!(ka = (KA_T)ofb[l])) {
+			    j += U_FE_OF_ALLOC_SIZE;
+			    i += U_FE_OF_ALLOC_SIZE - 1;
+			    continue;
+			}
+			if (kread(ka, (char *)&ufeo, sizeof(ufeo)))
+			    break;
+		    }
+		    fp = ufeo[k].ufe_ofile;
+
+# if	defined(HASFSTRUCT) && DUV>=40000
+		    if (Fsv & FSV_FG)
+			pv = ufeo[k].ufe_oflags;
+# endif	/* defined(HASFSTRUCT) && DUV>=40000 */
+
+		    if (++k >= U_FE_OF_ALLOC_SIZE)
+			k = 0;
+		    j++;
+		}
+		if (!j) {
+		    if (!k) {
+			if (kread(ka, (char *)&ufe, sizeof(ufe)))
+			    break;
+		    }
+		    fp = ufe[k].ufe_ofile;
+
+# if	defined(HASFSTRUCT) && DUV>=40000
+		    if (Fsv & FSV_FG)
+			pv = ufe[k].ufe_oflags;
+# endif	/* defined(HASFSTRUCT) && DUV>=40000 */
+
+		    if (++k >= U_FE_ALLOC_SIZE)
+			k = 0;
+		}
+		if (fp && (ulong)fp != 0xffffffffffffffff) {
+		    alloc_lfile(NULL, i);
+		    process_file((KA_T)fp);
+		    if (Lf->sf) {
+
+# if	defined(HASFSTRUCT) && DUV>=40000
+			if (Fsv & FSV_FG) {
+			    if ((Lf->pof = pv))
+				Lf->fsv |= FSV_FG;
+			}
+# endif	/* defined(HASFSTRUCT) && DUV>=40000 */
+
+			link_lfile();
+		    }
+		}
+	    }
+#endif	/* DUV>=50000 */
+
 	/*
 	 * Examine results.
 	 */
@@ -287,6 +414,8 @@ gather_proc_info()
 static void
 get_kernel_access()
 {
+	dev_t dev;
+	int rv;
 	KA_T v;
 /*
  * Check kernel version.
@@ -295,6 +424,8 @@ get_kernel_access()
 /*
  * Set name list file path.
  */
+
+#if	DUV<40000
 	if (!Nmlst) {
 	    if (!(Nmlst = get_nlist_path(1))) {
 		(void) fprintf(stderr, "%s: can't get kernel name list path\n",
@@ -302,6 +433,7 @@ get_kernel_access()
 		Exit(1);
 	    }
 	}
+#endif	/* DUV<40000 */
 
 #if	defined(WILLDROPGID)
 /*
@@ -345,10 +477,18 @@ get_kernel_access()
  * Access kernel symbols.
  */
 	(void) build_Nl(Drive_Nl);
-        if (nlist(Nmlst, Nl) == -1) {
+
+#if	DUV>=40000
+	if (!Nmlst)
+	    rv = knlist(Nl);
+	else
+#endif	/* DUV>=40000 */
+	
+            rv = nlist(Nmlst, Nl);
+	if (rv == -1) {
 	    (void) fprintf(stderr,
 		"%s: can't read kernel name list from %s: %s\n",
-		Pn, Nmlst, strerror(errno));
+		Pn, Nmlst ? Nmlst : "kernel", strerror(errno));
 	    Exit(1);
 	}
 
@@ -374,10 +514,14 @@ get_kernel_access()
 	    Exit(1);
 	}
 	if (get_Nl_value("cldev", Drive_Nl, &v) < 0 || !v
-	||  kread(v, (char *)&Clonedev, sizeof(Clonedev))) {
+	||  kread(v, (char *)&dev, sizeof(dev))) {
 	    if (!Fwarn)
 		(void) fprintf(stderr, "%s: can't read clone device number\n",
 		    Pn);
+	    HaveCloneMaj = 0;
+	} else {
+	    CloneMaj = major(dev);
+	    HaveCloneMaj = 1;
 	}
 }
 
@@ -539,9 +683,17 @@ process_text(tp)
 	    return;
 	if (!vmm.vm_is_mainmap)
 	    return;
+
+#if	defined(VM_SKIPLIST)
+	for (i = 0, ka = (KA_T)vmm.vm_links.vml_sl_next[0];
+	     i < vmm.vm_nentries && ka != (KA_T)t.map;
+	     i++, ka = (KA_T)vmme.vme_links.vml_sl_next[0])
+#else	/* !defined(VM_SKIPLIST) */
 	for (i = 0, ka = (KA_T)vmm.vm_links.next;
 	     i < vmm.vm_nentries && ka != (KA_T)t.map;
 	     i++, ka = (KA_T)vmme.vme_links.next)
+#endif	/* defined(VM_SKIPLIST) */
+
 	{
 
 	/*
@@ -584,8 +736,9 @@ process_text(tp)
 	     * If this is a segment object, read the segment map, and search
 	     * for backing objects whose object type is OT_UBC or OT_VP.
 	     */
+
 	    case OT_SEG:
-		for (kb=(KA_T)vmme.vmet.tvme_seg; kb; kb=(KA_T)vms.seg_vnext) {
+		for (kb=(KA_T)vmme.vme_seg; kb; kb=(KA_T)vms.seg_vnext) {
 		    if (kread(kb, (char *)&vms, sizeof(vms)))
 			break;
 		    if (!vms.seg_vop
