@@ -1,5 +1,6 @@
 /*
- * dsock.c - Digital UNIX (DEC OSF/1) socket processing functions for lsof
+ * dsock.c - DEC OSF/1, Digital UNIX, Tru64 UNIX socket processing functions
+ *	     for lsof
  */
 
 
@@ -32,7 +33,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.8 98/08/03 07:41:04 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.12 99/11/28 06:39:40 abe Exp $";
 #endif
 
 
@@ -123,9 +124,16 @@ process_socket(sa)
  * Process an Internet domain socket.
  */
 	case AF_INET:
+
+#if	defined(HASIPv6)
+	case AF_INET6:
+	    (void) strcpy(Lf->type, (fam == AF_INET) ? "IPv4" : "IPv6");
+#else	/* !defined(HASIPv6) */
+	    (void) strcpy(Lf->type, "inet");
+#endif	/* defined(HASIPv6) */
+
 	    if (Fnet)
 		Lf->sf |= SELNET;
-	    (void) strcpy(Lf->type, "inet");
 	    printiproto(p.pr_protocol);
 	/*
 	 * Read protocol control block.
@@ -142,14 +150,48 @@ process_socket(sa)
 	 */
 	    ka = (KA_T)(inp.inp_ppcb ? inp.inp_ppcb : s.so_pcb);
 	    enter_dev_ch(print_kptr((ka & 0xffffffff), (char *)NULL));
+
+#if	defined(HASIPv6)
+	    if ((fam == AF_INET && IN6_IS_ADDR_UNSPECIFIED(&inp.inp_laddr))
+	    ||   IN6_IS_ADDR_V4MAPPED(&inp.inp_laddr)) {
+		la = (unsigned char *)&IN6_EXTRACT_V4ADDR(&inp.inp_laddr);
+		fam = AF_INET;
+		Lf->type[3] = '4';
+	    } else {
+		la = (unsigned char *)&inp.inp_laddr;
+		fam = AF_INET6;
+		Lf->type[3] = '6';
+	    }
+#else	/* !defined(HASIPv6) */
 	    la = (unsigned char *)&inp.inp_laddr;
+#endif	/* defined(HASIPv6) */
+
 	    lp = (int)ntohs(inp.inp_lport);
+
+#if	defined(HASIPv6)
+	    if (fam == AF_INET) {
+		if (inp.inp_fport
+		||  IN6_EXTRACT_V4ADDR(&inp.inp_faddr) != INADDR_ANY)
+		{
+		    fa = (unsigned char *)&IN6_EXTRACT_V4ADDR(&inp.inp_faddr);
+		    fp = (int)ntohs(inp.inp_fport);
+		}
+	    } else {
+		if (inp.inp_fport || !IN6_IS_ADDR_UNSPECIFIED(&inp.inp_faddr))
+		{
+		    fa = (unsigned char *)&inp.inp_faddr;
+		    fp = (int)ntohs(inp.inp_fport);
+		}
+	    }
+#else	/* !defined(HASIPv6) */
 	    if (inp.inp_faddr.s_addr != INADDR_ANY || inp.inp_fport != 0) {
 		fa = (unsigned char *)&inp.inp_faddr;
 		fp = (int)ntohs(inp.inp_fport);
 	    }
+#endif	/* defined(HASIPv6) */
+
 	    if (fa || la)
-		(void) ent_inaddr(la, lp, fa, fp, AF_INET);
+		(void) ent_inaddr(la, lp, fa, fp, fam);
 	    if (p.pr_protocol == IPPROTO_TCP && inp.inp_ppcb
 	    &&  !kread((KA_T)inp.inp_ppcb, (char *)&t, sizeof(t))) {
 		Lf->lts.type = 0;
@@ -232,15 +274,15 @@ process_socket(sa)
 		if (mb.m_len >= sizeof(struct sockaddr_un))
 		    mb.m_len = sizeof(struct sockaddr_un) - 1;
 		*((char *)ua + mb.m_len) = '\0';
-		if (Sfile && is_file_named(ua->sun_path, VSOCK))
+		if (Sfile && is_file_named(ua->sun_path, 0))
 		    Lf->sf |= SELNM;
-		else
+		if (!Namech[0])
 		    (void) strcpy(Namech, ua->sun_path);
 	    } else
 		(void) strcpy(Namech, "no address");
 	    break;
 	default:
-	    printunkaf(fam);
+	    printunkaf(fam, 1);
 	}
 	if (Namech[0])
 	    enter_nm(Namech);
