@@ -1,5 +1,5 @@
 /*
- * machine.h - Digital UNIX (DEC OSF/1) definitions for lsof
+ * machine.h - DEC OSF/1, Digital UNIX, Tru64 UNIX definitions for lsof
  */
 
 
@@ -31,23 +31,39 @@
 
 
 /*
- * $Id: machine.h,v 1.17 98/06/29 15:41:25 abe Exp $
+ * $Id: machine.h,v 1.24 99/07/28 20:34:23 abe Exp $
  */
 
 
-#if	!defined(MACHINE_H)
-#define	MACHINE_H	1
+#if	!defined(LSOF_MACHINE_H)
+#define	LSOF_MACHINE_H	1
 
 
 #include <sys/types.h>
 
 #define	_KERNEL	1
 #include <sys/signal.h>
+
+#if	DUV>=50000
+#include <sys/resource.h>
+#include <sys/time.h>
+#endif	/* DUV>=50000 */
+
 #undef	_KERNEL
 
 #include <sys/param.h>
 
 
+/*
+ * CAN_USE_CLNT_CREATE is defined for those dialects where RPC clnt_create()
+ * can be used to obtain a CLIENT handle in lieu of clnttcp_create().
+ */
+
+#if	DUV>=40000
+#define	CAN_USE_CLNT_CREATE	1
+#endif	/* DUV>=40000 */
+
+
 /*
  * GET_MAX_FD is defined for those dialects that provide a function other than
  * getdtablesize() to obtain the maximum file descriptor number plus one.
@@ -102,7 +118,7 @@
 #define	HASENVDC	"LSOFDEVCACHE"
 #define	HASPERSDC	"%h/%p.lsof_%L"
 #define	HASPERSDCPATH	"LSOFPERSDCPATH"
-/* #define	HASSYSDC	"/var/adm/lsof.dc" */
+/* #define	HASSYSDC	"/your/choice/of/path" */
 
 
 /*
@@ -127,6 +143,18 @@
 #define	HASFSINO	1
 
 
+/*
+ * HASFSTRUCT is defined if the dialect has a file structure.
+ *
+ * FSV_DEFAULT defines the default set of file structure values to list.
+ * It defaults to zero (0), but may be made up of a combination of the
+ * FSV_* symbols from lsof.h.
+ */
+
+#define	HASFSTRUCT	1
+/* #define	FSV_DEFAULT	FSV_? | FSV_? | FSV_? */
+
+
 /*
  * HASGNODE is defined for those dialects that have gnodes.
  */
@@ -179,22 +207,11 @@
  * them.  If any of the additional elements need to be preset in the
  * alloc_lfile() function of proc.c, the SETLFILEADD macro may be defined
  * to do that.  The HASXOPT definition may be used to select the conditions
- * under which provate lfile elements are used.
- *
- * Field identifiers 1 through 9 are reserved for dialect-specific field
- * output.  Define the string HASFIELDAP<x> as the help (-F?) name of the
- * private field and LISTLFIELAP<x> as a macro to list the field output.
- * (<x> is the field identifier character.)  HASFIELDAP<x> is used in proc.c
- * and store.c; LISTFIELDAP<x>, in proc.c.
- *
- * See ../dialectx/ptx for an example of the use of HASLFILEADD, SETLFILEADD,
- * and HASXOPT.
+ * under which private lfile elements are used.
  */
 
 #define HASLFILEADD int advfs_seq; unsigned char advfs_seq_stat;
 #define SETLFILEADD Lf->advfs_seq_stat = 0;
-/* #define HASFIELDAP1 <string name of field for -F? output> */
-/* #define LISTLFILEAP1 <macro to list field output>; */
 
 
 /*
@@ -233,7 +250,7 @@
  * NOTE: don't forget to define a prototype for this function in dproto.h.
  */
 
-/* #define	HASPIPEFN	process_pipe */
+/* #define	HASPIPEFN	process_pipe? */
 
 
 /*
@@ -267,10 +284,10 @@
  */
 
 #define	HASPRINTDEV	print_dev
-/* #define	HASPRINTINO	print_ino	*/
-/* #define	HASPRINTNM	print_nm	*/
-/* #define	HASPRINTOFF	print_off	*/
-/* #define	HASPRINTSZ	print_sz	*/
+/* #define	HASPRINTINO	print_ino?	*/
+/* #define	HASPRINTNM	print_nm?	*/
+/* #define	HASPRINTOFF	print_off?	*/
+/* #define	HASPRINTSZ	print_sz?	*/
 
 
 /*
@@ -283,7 +300,7 @@
  * HASPRIVFILETYPE is the name of the processing function.
  */
 
-/* #define	HASPRIVFILETYPE	process_shmf	*/
+/* #define	HASPRIVFILETYPE	process_shmf?	*/
 /* #define	PRIVFILETYPE	??	*/
 
 
@@ -465,4 +482,4 @@
 
 #define	zeromem(a, l)	bzero(a, l)
 
-#endif	/* !defined(MACHINE_H) */
+#endif	/* !defined(LSOF_MACHINE_H) */
