@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dmnt.c,v 1.6 98/07/17 08:49:40 abe Exp $";
+static char *rcsid = "$Id: dmnt.c,v 1.7 98/11/23 07:39:00 abe Exp $";
 #endif
 
 
@@ -45,12 +45,18 @@ static char *rcsid = "$Id: dmnt.c,v 1.6 98/07/17 08:49:40 abe Exp $";
 
 static struct mounts *Lmi = (struct mounts *)NULL;	/* local mount info */
 static int Lmist = 0;					/* Lmi status */
-static char *mnt_names[] =
+
+#undef	HAS_MNT_NAMES
 
 #if	FREEBSDV<200
-	{ "none", "ufs", "nfs", "mfs", "pc", "iso9600", "procfs", "devfs" };
+static char *mnt_names[] = { "none", "ufs", "nfs", "mfs", "pc", "iso9600",
+			     "procfs", "devfs" };
+#define	HAS_MNT_NAMES	1
 #else	/* FREEBSDV>=200 */
-	INITMOUNTNAMES;
+# if	defined(MOUNT_NONE)
+static char *mnt_names[] = INITMOUNTNAMES;
+#define	HAS_MNT_NAMES	1
+# endif	/* defined(MOUNT_NONE)) */
 #endif	/* FREEBSDV<200 */
 
 
@@ -85,7 +91,13 @@ readmnt()
  * Read mount information.
  */
 	for (; n; n--, mb++) {
+
+#if	defined(MOUNT_NONE)
 	    if (mb->f_type == MOUNT_NONE || mb->f_type >= MOUNT_MAXTYPE)
+#else	/* !defined(MOUNT_NONE) */
+	    if (!mb->f_type)
+#endif	/* defined(MOUNT_NONE) */
+
 		continue;
 	/*
 	 * Interpolate a possible symbolic directory link.
@@ -120,7 +132,13 @@ no_space_for_mount:
 	    if (statsafely(dn, &sb)) {
 		if (!Fwarn) {
 		    (void) fprintf(stderr, "%s: WARNING: can't stat() ", Pn);
+
+#if	defined(HAS_MNT_NAMES)
 		    safestrprt(mnt_names[mb->f_type], stderr, 0);
+#else	/* !defined(HAS_MNT_NAMES) */
+		    safestrprt(mb->f_fstypename, stderr, 0);
+#endif	/* defined(HAS_MNT_NAMES) */
+
 		    (void) fprintf(stderr, " file system ");
 		    safestrprt(mb->f_mntonname, stderr, 1);
 		    (void) fprintf(stderr,
@@ -144,7 +162,14 @@ no_space_for_mount:
 	    dn = (char *)NULL;
 
 #if	defined(HASPROCFS)
-	    if (mb->f_type == MOUNT_PROCFS) {
+
+#if	defined(MOUNT_NONE)
+	    if (mb->f_type == MOUNT_PROCFS)
+#else	/* !defined(MOUNT_NONE) */
+	    if (strcasecmp(mb->f_fstypename, "procfs") == 0)
+#endif	/* defined(MOUNT_NONE) */
+
+	    {
 
 	    /*
 	     * Save information on exactly one procfs file system.
@@ -228,7 +253,30 @@ readvfs(vm)
 	}
 	vp->addr = vm;
 	vp->fsid = m.m_stat.f_fsid;
+
+#if	defined(MOUNT_NONE)
 	vp->type = m.m_stat.f_type;
+#else	/* !defined(MOUNT_NONE) */
+	{
+	    int len;
+
+	    if ((len = strlen(m.m_stat.f_fstypename))) {
+		if (len > (MFSNAMELEN - 1))
+		    len = MFSNAMELEN - 1;
+		if (!(vp->typnm = mkstrcat(m.m_stat.f_fstypename, len,
+				  (char *)NULL, -1, (char *)NULL, -1,
+				  (MALLOC_S *)NULL)))
+		{
+		    (void) fprintf(stderr,
+			"%s: no space for fs type name: ", Pn);
+		    safestrprt(m.m_stat.f_fstypename, stderr, 1);
+		    Exit(1);
+		}
+	    } else
+		vp->typnm = "";
+	}
+#endif	/* defined(MOUNT_NONE) */
+
 	vp->next = Lvfs;
 	return(vp);
 }
