@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.5 98/06/25 07:17:33 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.7 99/05/09 14:55:42 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -109,8 +109,9 @@ gather_proc_info()
 {
 	struct filedesc fd;
 	int i, nf;
-	static int nofb = 0;
+	MALLOC_S nb;
 	static struct file **ofb = NULL;
+	static int ofbb = 0;
 	int pgrp, pid;
 	int ppid = 0;
 	short pss, sf;
@@ -125,6 +126,11 @@ gather_proc_info()
 	struct kinfo_proc *p;
 #endif	/* FREEBSDV<200 */
 
+#if	defined(HASFSTRUCT)
+	static char *pof = (char *)NULL;
+	static int pofb = 0;
+#endif	/* defined(HASFSTRUCT) */
+
 /*
  * Read the process table.
  */
@@ -256,28 +262,42 @@ gather_proc_info()
 	 */
 	    if (!fd.fd_ofiles || (nf = fd.fd_nfiles) <= 0)
 		continue;
-	    if (!ofb) {
-		nofb = nf;
-		if (!(ofb = (struct file **)malloc((MALLOC_S)
-			     (nofb * sizeof(struct file *)))))
-		{
+	    nb = (MALLOC_S)(sizeof(struct file *) * nf);
+	    if (nb > ofbb) {
+		if (!ofb)
+		    ofb = (struct file **)malloc(nb);
+		else
+		    ofb = (struct file **)realloc((MALLOC_P *)ofb, nb);
+		if (!ofb) {
 		    (void) fprintf(stderr, "%s: PID %d, no file * space\n",
 			Pn, p->P_PID);
 		    Exit(1);
 		}
-	    } else if (nf > nofb) {
-		nofb = nf;
-		if (!(ofb = (struct file **)realloc((MALLOC_P *)ofb,
-			     (nofb * sizeof(struct file *)))))
-		{
-		    (void) fprintf(stderr,
-			"%s: PID %d, no realloc file * space\n",
-			Pn, p->P_PID);
-		    Exit(1);
-		}
+		ofbb = nb;
 	    }
-	    if (kread((KA_T)fd.fd_ofiles,(char *)ofb,nf*sizeof(struct file *)))
+	    if (kread((KA_T)fd.fd_ofiles, (char *)ofb, nb))
 		continue;
+
+#if	defined(HASFSTRUCT)
+	    if (Fsv & FSV_FG) {
+		nb = (MALLOC_S)(sizeof(char) * nf);
+		if (nb > pofb) {
+		    if (!pof)
+			pof = (char *)malloc(nb);
+		    else
+			pof = (char *)realloc((MALLOC_P *)pof, nb);
+		    if (!pof) {
+			(void) fprintf(stderr,
+			    "%s: PID %d, no file flag space\n", Pn, p->P_PID);
+			Exit(1);
+		    }
+		    pofb = nb;
+		}
+		if (!fd.fd_ofileflags || kread((KA_T)fd.fd_ofileflags, pof, nb))
+		    zeromem(pof, nb);
+	    }
+#endif	/* defined(HASFSTRUCT) */
+
 	/*
 	 * Save information on file descriptors.
 	 */
@@ -285,8 +305,15 @@ gather_proc_info()
 		if (ofb[i]) {
 		    alloc_lfile(NULL, i);
 		    process_file((KA_T)(Cfp = ofb[i]));
-		    if (Lf->sf)
+		    if (Lf->sf) {
+
+#if	defined(HASFSTRUCT)
+			if (Fsv & FSV_FG)
+			    Lf->pof = (long)pof[i];
+#endif	/* defined(HASFSTRUCT) */
+
 			link_lfile();
+		    }
 		}
 	    }
 	/*
@@ -482,8 +509,6 @@ process_text(vm)
 /*
  * Read the vm_map structure.  Search its vm_map_entry structure list.
  */
-	if (!vmsp.vm_map.is_main_map)
-	    return;
 	for (i = 0; i < vmsp.vm_map.nentries; i++) {
 
 	/*
