@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dstore.c,v 1.2 97/05/30 16:31:43 abe Exp $";
+static char *rcsid = "$Id: dstore.c,v 1.5 99/05/09 14:55:46 abe Exp $";
 #endif
 
 
@@ -83,3 +83,36 @@ int Np = 0;				/* number of kernel processes */
 #if	FREEBSDV>=200
 struct kinfo_proc *P = NULL;		/* local process table copy */
 #endif	/* FREEBSDV>=200 */
+
+#if	defined(HASFSTRUCT)
+/*
+ * Pff_tab[] - table for printing file flags
+ */
+
+struct pff_tab Pff_tab[] = {
+	{ (long)FREAD,		FF_READ		},
+	{ (long)FWRITE,		FF_WRITE	},
+	{ (long)FNONBLOCK,	FF_NBLOCK	},
+	{ (long)FNDELAY,	FF_NDELAY	},
+	{ (long)FAPPEND,	FF_APPEND	},
+	{ (long)FASYNC,		FF_ASYNC	},
+	{ (long)FFSYNC,		FF_FSYNC	},
+	{ (long)FMARK,		FF_MARK		},
+	{ (long)FDEFER,		FF_DEFER	},
+	{ (long)FHASLOCK,	FF_HASLOCK	},
+	{ (long)O_NOCTTY,	FF_NOCTTY	},
+	{ (long)0,		NULL 		}
+};
+
+
+/*
+ * Pof_tab[] - table for print process open file flags
+ */
+
+struct pff_tab Pof_tab[] = {
+
+	{ (long)UF_EXCLOSE,	POF_CLOEXEC	},
+	{ (long)UF_MAPPED,	POF_MAPPED	},
+	{ (long)0,		NULL		}
+};
+#endif	/* defined(HASFSTRUCT) */
