@@ -31,18 +31,26 @@
 
 
 /*
- * $Id: machine.h,v 1.15 98/06/29 15:41:26 abe Exp $
+ * $Id: machine.h,v 1.20 10/.0/.1 .0:.0:.0 abe Exp $
  */
 
 
-#if	!defined(MACHINE_H)
-#define	MACHINE_H	1
+#if	!defined(LSOF_MACHINE_H)
+#define	LSOF_MACHINE_H	1
 
 
 #include <sys/types.h>
 #include <sys/param.h>
 
 
+/*
+ * CAN_USE_CLNT_CREATE is defined for those dialects where RPC clnt_create()
+ * can be used to obtain a CLIENT handle in lieu of clnttcp_create().
+ */
+
+#define	CAN_USE_CLNT_CREATE	1
+
+
 /*
  * GET_MAX_FD is defined for those dialects that provide a function other than
  * getdtablesize() to obtain the maximum file descriptor number plus one.
@@ -65,7 +73,9 @@
  * recorded in BDevtp[].
  */
 
+# if	FREEBSDV<400
 #define	HASBLKDEV	1
+# endif	/* FREEBSDV<400 */
 
 
 /*
@@ -97,7 +107,7 @@
 #define	HASENVDC	"LSOFDEVCACHE"
 #define	HASPERSDC	"%h/%p.lsof_%L"
 #define	HASPERSDCPATH	"LSOFPERSDCPATH"
-/* #define	HASSYSDC	"/var/adm/lsof.dc" */
+/* #define	HASSYSDC	"/your/choice/of/path" */
 
 
 /*
@@ -122,6 +132,18 @@
 /* #define	HASFSINO	1 */
 
 
+/*
+ * HASFSTRUCT is defined if the dialect has a file structure.
+ *
+ * FSV_DEFAULT defines the default set of file structure values to list.
+ * It defaults to zero (0), but may be made up of a combination of the
+ * FSV_* symbols from lsof.h.
+ */
+
+#define	HASFSTRUCT	1
+/* #define	FSV_DEFAULT	FSV_? | FSV_? | FSV_? */
+
+
 /*
  * HASGNODE is defined for those dialects that have gnodes.
  */
@@ -175,21 +197,10 @@
  * alloc_lfile() function of proc.c, the SETLFILEADD macro may be defined
  * to do that.  The HASXOPT definition may be used to select the conditions
  * under which private lfile elements are used.
- *
- * Field identifiers 1 through 9 are reserved for dialect-specific field
- * output.  Define the string HASFIELDAP<x> as the help (-F?) name of the
- * private field and LISTLFIELAP<x> as a macro to list the field output.
- * (<x> is the field identifier character.)  HASFIELDAP<x> is used in proc.c
- * and store.c; LISTFIELDAP<x>, in proc.c.
- *
- * See ../dialectx/ptx for an example of the use of HASLFILEADD, SETLFILEADD,
- * and HASXOPT.
  */
 
-/* #define HASLFILEADD <macro of new element definitions>; */
-/* #define SETLFILEADD <macro of code to preset new elements>; */
-/* #define HASFIELDAP1 <string name of field for -F? output> */
-/* #define LISTLFILEAP1 <macro to list field output>; */
+/* #define HASLFILEADD int ... */
+/* #define SETLFILEADD Lf->... */
 
 
 /*
@@ -265,11 +276,11 @@
  * called from print_file().
  */
 
-/* #define	HASPRINTDEV	print_dev	*/
-/* #define	HASPRINTINO	print_ino	*/
-/* #define	HASPRINTNM	print_nm	*/
-/* #define	HASPRINTOFF	print_off	*/
-/* #define	HASPRINTSZ	print_sz	*/
+/* #define	HASPRINTDEV	print_dev?	*/
+/* #define	HASPRINTINO	print_ino?	*/
+/* #define	HASPRINTNM	print_nm?	*/
+/* #define	HASPRINTOFF	print_off?	*/
+/* #define	HASPRINTSZ	print_sz?	*/
 
 
 /*
@@ -282,7 +293,7 @@
  * HASPRIVFILETYPE is the name of the processing function.
  */
 
-/* #define	HASPRIVFILETYPE	process_shmf	*/
+/* #define	HASPRIVFILETYPE	process_shmf?	*/
 /* #define	PRIVFILETYPE	??	*/
 
 
@@ -324,7 +335,7 @@
 #define	HASPROCFS	"proc"
 # endif	/* defined(HASPROCFS) */
 
-/* #define		HASPROCFS	"proc" */
+/* #define		HASPROCFS	"proc?" */
 /* #define		HASFSTYPE	1 */
 
 # if	FREEBSDV>=200
@@ -475,4 +486,4 @@
 
 #define	zeromem(a, l)	memset(a, 0, l)
 
-#endif	/* !defined(MACHINE_H) */
+#endif	/* !defined(LSOF_MACHINE_H) */
