@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.7 98/05/22 07:14:18 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.12 99/06/22 08:18:38 abe Exp $";
 #endif
 
 
@@ -63,7 +63,7 @@ process_inode(fi, fop)
 	static int ft = 1;
 	struct inode is;
 	struct file_lock lk, *lkf, *lkp;
-	short lt;
+	short lo, lt;
 	struct mounts *mp;
 	static unsigned long ndops;
 	static unsigned long nfops;
@@ -97,12 +97,16 @@ process_inode(fi, fop)
 	}
 
 #if	defined(HASNCACHE)
-/*
- * Save inode address if HASNCACHE defined.
- */
-	Lf->na = (unsigned long)fi;
+	Lf->na = (KA_T)fi;
 #endif	/* defined(HASNCACHE) */
 
+#if	defined(HASFSTRUCT)
+	if (Fsv & FSV_NI) {
+	    Lf->fna = (KA_T)fi;
+	    Lf->fsv |= FSV_NI;
+	}
+#endif	/* defined(HASFSTRUCT) */
+
 /*
  * Process node type.
  */
@@ -145,19 +149,23 @@ process_inode(fi, fop)
 	    do {
 		if (kread((KA_T)lkp, (char *)&lk, sizeof(lk)))
 		    break;
-		if (lk.fl_owner != Tsp)
-		    continue;
+		if (lk.fl_owner && lk.fl_owner == Tsp)
+		    lo = 1;
+		else
+		    lo = 0;
 
 #if	defined(HAS_FL_FILE)
-		if (lk.fl_file != Cfp)
-		    continue;
+		if (lk.fl_file && lk.fl_file == Cfp)
+		    lo++;
 #endif	/* defined(HAS_FL_FILE) */
 
 #if	defined(HAS_FL_FD)
-		if (lk.fl_fd != Cfd)
-		    continue;
+		if (lk.fl_fd && lk.fl_fd == Cfd)
+		    lo++;
 #endif	/* defined(HAS_FL_FD) */
 
+		if (!lo)
+		    continue;
 		if (lk.fl_start == 0
 
 #if	defined(HAS_FL_WHENCE)
@@ -254,6 +262,15 @@ process_inode(fi, fop)
 		}
 	    }
 	}
+/*
+ * Record the link count.
+ */
+	if (Fnlink) {
+	    Lf->nlink = (long)is.i_nlink;
+	    Lf->nlink_def = 1;
+	    if (Nlink && (Lf->nlink < Nlink))
+		Lf->sf |= SELNLINK;
+	}
 /*
  * Format the type name.
  */
@@ -343,7 +360,7 @@ process_inode(fi, fop)
 /*
  * Test for specified file.
  */
-	if (Sfile && is_file_named(NULL, type))
+	if (Sfile && is_file_named(NULL, (type == S_IFCHR) ? 1 : 0))
 	    Lf->sf |= SELNM;
 /*
  * Enter name characters.
