@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.14 98/06/25 07:18:53 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.21 99/06/22 08:18:54 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -44,7 +44,6 @@ static char *rcsid = "$Id: dproc.c,v 1.14 98/06/25 07:18:53 abe Exp $";
 
 
 _PROTOTYPE(static void get_kernel_access,(void));
-_PROTOTYPE(static int hash_ksym,(char *nm));
 _PROTOTYPE(static char *hasparmhash,(char *nm));
 _PROTOTYPE(static int nlist_private,(char *fnm, struct NLIST_TYPE *nl));
 _PROTOTYPE(static void process_mapmem,(struct inode *x, struct vm_area_struct *m));
@@ -74,7 +73,8 @@ static char *MapNm[] = {		/* possible system map file paths */
  * Local definitions
  */
 
-#define	KSHASHSZ	256		/* kernel symbol hash table size */
+#define	KSHASHSZ	512		/* kernel symbol hash table size -- must
+					 * be a power of 2! */
 
 
 /*
@@ -291,8 +291,17 @@ gather_proc_info()
 			continue;
 		    alloc_lfile((char *)NULL, Cfd);
 		    process_file((KA_T)f);
-		    if (Lf->sf)
+		    if (Lf->sf) {
+
+#if	defined(HASFSTRUCT) && LINUXV>13021
+			if (Fsv & FSV_FG) {
+			    if (FD_ISSET(Cfd, (fd_set *)&fis.close_on_exec))
+				Lf->pof = 1l;
+			}
+#endif	/* defined(HASFSTRUCT) && LINUXV>13021 */
+
 			link_lfile();
+		    }
 		}
 	    /*
 	     * Examine results.
@@ -366,12 +375,16 @@ get_kernel_access()
 	    if (!(Nmlst = get_nlist_path(1))) {
 		(void) fprintf(stderr,
 		    "%s: can't determine system map file path\n", Pn);
+		(void) fprintf(stderr,
+		    "%s: (See 00FAQ for more information.)\n", Pn);
 		Exit(1);
 	    }
 	}
 	if (nlist_private(Nmlst, Nl) < 0) {
 	    (void) fprintf(stderr,
 		"%s: can't read kernel name list from %s\n", Pn, Nmlst);
+	    (void) fprintf(stderr,
+		"%s: (See 00FAQ for more information.)\n", Pn);
 	    Exit(1);
 	}
 /*
@@ -381,6 +394,8 @@ get_kernel_access()
 	{
 	    (void) fprintf(stderr, "%s: no %stask kernel definition\n",
 		Pn, Coff ? "_" : "");
+	    (void) fprintf(stderr,
+		"%s: (See 00FAQ for more information.)\n", Pn);
 	    Exit(1);
 	}
 }
@@ -438,24 +453,6 @@ get_nlist_path(ap)
 }
 
 
-/*
- * hash_ksym() - hash kernel symbol by name
- */
-
-static int
-hash_ksym(nm)
-	char *nm;
-{
-	int i, j;
-
-	for (i = j = 0; *nm; nm++) {
-	    i ^= (int)*nm << j;
-	    j ^= 1;
-	}
-	return(i % KSHASHSZ);
-}
-
-
 /*
  * hasparmhash() - does symbol name have parameter hash
  *
@@ -498,6 +495,8 @@ initialize()
 	get_kernel_access();
 	if (Selinet == 0)
 	    (void) readmnt();
+	if (Fsv & FSV_NI)
+	    NiTtl = "INODE-ADDR";
 }
 
 
@@ -573,29 +572,29 @@ nlist_private(fnm, nl)
 	for (qb = (void *)NULL, qbl = 0;;) {
 	    if (query_module((char *)NULL, QM_SYMBOLS, qb, qbl, &qmr)) {
 		if (errno != ENOSPC) {
-		    if (!Fwarn) {
-			if (errno = ENOSYS) {
-			    (void) fprintf(stderr,
-				"%s: query_module unimplemented\n", Pn);
-			    (void) fprintf(stderr,
-				"      CONFIG_MODULES not defined in autoconf.h?\n");
-			} else {
-			    (void) fprintf(stderr,
-				"%s: query_module error: %s\n",
-				Pn, strerror(errno));
-			}
+		    if (errno == ENOSYS) {
 			(void) fprintf(stderr,
-			    "%s: WARNING: unable to verify symbols in %s\n",
-			    Pn, fnm);
-		    }
-		    qbl = 0;
-		    if (qb) {
-			(void) free((FREE_P *)qb);
-			qb = (void *)NULL;
-		    }
-		    break;
+			    "%s: query_module unimplemented\n", Pn);
+			(void) fprintf(stderr,
+			    "%s: (Is CONFIG_MODULES defined in autoconf.h?)\n",
+			    Pn);
+		    } else
+			(void) fprintf(stderr, "%s: query_module error: %s\n",
+			    Pn, strerror(errno));
+
+query_module_error:
+
+		    (void) fprintf(stderr,
+			"%s: unable to verify symbols in %s\n", Pn, fnm);
+		    (void) fprintf(stderr,
+			"%s: (See 00FAQ for more information.)\n", Pn);
+		    Exit(1);
+		}
+		if (!(qbl = qmr)) {
+		    (void) fprintf(stderr,
+			"%s: query_module reports no kernel symbols\n", Pn);
+		    goto query_module_error;
 		}
-		qbl = qmr;
 		if (!qb)
 		    qb = (void *)malloc(qbl + 1);
 		else
@@ -631,7 +630,7 @@ nlist_private(fnm, nl)
 	 * /[z]System.map.
 	 *
 	 * Look for the symbol "_system_utsname" or "system_utsname" to
-	 * determine the loader format of the kernel: it's * COFF if the
+	 * determine the loader format of the kernel: it's COFF if the
 	 * symbol has a leading `_'; ELF if it does not.
 	 */
 	    for (i = 0, qp = (struct module_symbol *)qb; i < qmr; i++, qp++) {
@@ -654,7 +653,7 @@ nlist_private(fnm, nl)
 			"%s: no space for kernel symbol name: %s\n", Pn, nm);
 		    Exit(1);
 		}
-		j = hash_ksym(cp);
+		j = hashbyname(cp, KSHASHSZ);
 		(void) strcpy(kp->name, cp);
 		if (strcmp(cp, "_system_utsname") == 0)
 		    Coff = 1;
@@ -672,92 +671,92 @@ nlist_private(fnm, nl)
  * Read the kernel symbols via get_kernel_syms().
  */
 	if ((nk = get_kernel_syms((struct kernel_sym *)NULL)) < 0) {
-	    if (!Fwarn) {
-		if (errno == ENOSYS) {
-		    (void) fprintf(stderr,
-			"%s: WARNING: get_kernel_syms() unimplemented\n", Pn);
-		    (void) fprintf(stderr,
-		        "      CONFIG_MODULES not defined in autoconf.h?\n");
-		} else {
-		    (void) fprintf(stderr,
-			"%s: get_kernel_syms() error: %s\n",
-			Pn, strerror(errno));
-		}
+	    if (errno == ENOSYS) {
 		(void) fprintf(stderr,
-		    "%s: WARNING: unable to verify symbols in %s\n", Pn, fnm);
-	    }
+		    "%s: get_kernel_syms() unimplemented\n", Pn);
+		(void) fprintf(stderr,
+		    "%s: (Is CONFIG_MODULES defined in autoconf.h?)\n", Pn);
+	    } else
+		(void) fprintf(stderr, "%s: get_kernel_syms() error: %s\n",
+		    Pn, strerror(errno));
+
+get_kernel_syms_error:
+
+	    (void) fprintf(stderr,
+		"%s: unable to verify symbols in %s\n", Pn, fnm);
+	    (void) fprintf(stderr, "%s: (See 00FAQ for more information.)\n",
+		Pn);
+	    Exit(1);
 	} else {
+	    if (!nk) {
+		(void) fprintf(stderr,
+		    "%s: get_kernel_syms reports no kernel symbols\n", Pn);
+		goto get_kernel_syms_error;
+	    }
 	    i = nk * sizeof(struct kernel_sym);
 	    if (!(ksa = (struct kernel_sym *)malloc((MALLOC_S)i))) {
 		(void) fprintf(stderr, "%s: no space for kernel symbols\n", Pn);
 		Exit(1);
 	    }
 	}
-	if (nk > 0) {
-	    if (get_kernel_syms(ksa) < 0) {
-		(void) fprintf(stderr, "%s: get_kernel_syms: %s\n",
-		    Pn, strerror(errno));
-		Exit(1);
+	if (get_kernel_syms(ksa) < 0) {
+	    (void) fprintf(stderr, "%s: get_kernel_syms() error: %s\n",
+		Pn, strerror(errno));
+	    Exit(1);
+	}
+/*
+ * Allocate hash table space.
+ */
+	if (!(kh = (struct ksym **)calloc((MALLOC_S)sizeof(struct ksym),
+		    KSHASHSZ)))
+	{
+	    (void) fprintf(stderr,
+		"%s: no space for %d byte kernel symbol hash table\n",
+		Pn, (KSHASHSZ * sizeof(struct ksym)));
+	    Exit(1);
+	}
+/*
+ * Scan the kernel symbol table and store their addresses, hashed by their
+ * names for later comparison to the same values in /[z]System.map.
+ *
+ * Look for the symbol "_system_utsname" or "system_utsname" to determine the
+ * loader format of the kernel: it's * COFF if the symbol has a leading `_';
+ * ELF if it does not.
+ *
+ * Symbols whose names begin with `#' are module markers.  Non-kernel module
+ * markers have the module name after the `#'; kernel module markers have a
+ * `\0' after the `#'.
+ */
+	for (i = 0, ik = 1, ks = ksa; i < nk; i++, ks++) {
+	    if (ks->name[0] == '#') {
+		ik = !ks->name[1] ? 1 : 0;
+		continue;
 	    }
-	/*
-	 * Allocate hash table space.
-	 */
-	    if (!(kh = (struct ksym **)calloc((MALLOC_S)sizeof(struct ksym),
-		       KSHASHSZ)))
-	    {
+	    if (!ik)
+		continue;
+	    if (!(kp = (struct ksym *)malloc((MALLOC_S)sizeof(struct ksym)))) {
 		(void) fprintf(stderr,
-		    "%s: no space for %d byte kernel symbol hash table\n",
-		    Pn, (KSHASHSZ * sizeof(struct ksym)));
+		    "%s: no space for kernel symbol structure: %s\n", Pn, nm);
 		Exit(1);
 	    }
-	/*
-	 * Scan the kernel symbol table and store their addresses, hashed
-	 * by their names for later comparison to the same values in
-	 * /[z]System.map.
-	 *
-	 * Look for the symbol "_system_utsname" or "system_utsname" to
-	 * determine the loader format of the kernel: it's * COFF if the
-	 * symbol has a leading `_'; ELF if it does not.
-	 *
-	 * Symbols whose names begin with `#' are module markers.  Non-kernel
-	 * module markers have the module name after the `#'; kernel module
-	 * markers have a `\0' after the `#'.
-	 */
-	    for (i = 0, ik = 1, ks = ksa; i < nk; i++, ks++) {
-		if (ks->name[0] == '#') {
-		    ik = !ks->name[1] ? 1 : 0;
-		    continue;
-		}
-		if (!ik)
-		    continue;
-		if (!(kp = (struct ksym *)malloc((MALLOC_S)sizeof(struct ksym)))
-		) {
-		    (void) fprintf(stderr,
-			"%s: no space for kernel symbol structure: %s\n",
-			Pn, nm);
-		    Exit(1);
-		}
-		if ((nm = hasparmhash(ks->name)))
-		    *nm = '\0';	/* chop off module version number */
-		if (!(kp->name = (char *)malloc((MALLOC_S)(strlen(ks->name)+1)))
-		) {
-		    (void) fprintf(stderr,
-			"%s: no space for kernel symbol name: %s\n", Pn, nm);
-		    Exit(1);
-		}
-		j = hash_ksym(ks->name);
-		(void) strcpy(kp->name, ks->name);
-		if (strcmp(ks->name, "_system_utsname") == 0)
-		    Coff = 1;
-		else if (strcmp(ks->name, "system_utsname") == 0)
-		    elf = 1;
-		kp->addr = ks->value;
-		kp->next = kh[j];
-		kh[j] = kp;
+	    if ((nm = hasparmhash(ks->name)))
+		*nm = '\0';	/* chop off module version number */
+	    if (!(kp->name = (char *)malloc((MALLOC_S)(strlen(ks->name)+1)))) {
+		(void) fprintf(stderr,
+		    "%s: no space for kernel symbol name: %s\n", Pn, nm);
+		Exit(1);
 	    }
-	    (void) free((FREE_P *)ksa);
-	} else
-	    kh = (struct ksym **)NULL;
+	    j = hashbyname(ks->name, KSHASHSZ);
+	    (void) strcpy(kp->name, ks->name);
+	    if (strcmp(ks->name, "_system_utsname") == 0)
+		Coff = 1;
+	    else if (strcmp(ks->name, "system_utsname") == 0)
+		elf = 1;
+	    kp->addr = ks->value;
+	    kp->next = kh[j];
+	    kh[j] = kp;
+	}
+	(void) free((FREE_P *)ksa);
 #endif	/* defined(HAS_QUERY_MODULE) */
 
 /*
@@ -785,6 +784,8 @@ nlist_private(fnm, nl)
 #endif	/* defined(KERN_LD_ELF) */
 
 			);
+		(void) fprintf(stderr,
+		    "%s: (See 00FAQ for more information.)\n", Pn);
 
 	    }
 	}
@@ -812,7 +813,7 @@ nlist_private(fnm, nl)
 	     * check the address; complain in detail about the first mismatch;
 	     * count mismatches.
 	     */
-		for (kp = kh[hash_ksym(nm)]; kp; kp = kp->next) {
+		for (kp = kh[hashbyname(nm, KSHASHSZ)]; kp; kp = kp->next) {
 		    if (strcmp(kp->name, nm) == 0)
 			break;
 		}
@@ -874,6 +875,8 @@ nlist_private(fnm, nl)
 			"      %s and the booted kernel may not be a",
 			fnm);
 		 (void) fprintf(stderr, " matched set.\n");
+		 (void) fprintf(stderr,
+			"      (See 00FAQ for more information.)\n");
 	    }
 	    Exit(1);
 	}
@@ -1169,8 +1172,8 @@ ncache_isroot(na)
 	    else
 		nc = (unsigned long *)malloc(j);
 	    if (!nc) {
-		(void) fprintf(stderr, "%s: no room (%d) for root node cache\n",
-		    Pn, j);
+		(void) fprintf(stderr,
+		    "%s: no space (%d) for root node cache\n", Pn, j);
 		Exit(1);
 	    }
 	    nca += 10;
@@ -1366,7 +1369,7 @@ ncache_load()
 			nb = (char *)malloc(nbl);
 		    if (!nb) {
 			(void) fprintf(stderr,
-			    "%s: can't allocate name buffer (%d)\n", Pn, nbl);
+			    "%s: no space (%d) for name buffer\n", Pn, nbl);
 			Exit(1);
 		    }
 		}
