@@ -33,12 +33,12 @@
 
 
 /*
- * $Id: dproto.h,v 1.2 97/04/15 10:40:00 abe Exp $
+ * $Id: dproto.h,v 1.3 99/01/25 07:04:18 abe Exp $
  */
 
 
 
 _PROTOTYPE(extern char *get_nlist_path,(int ap));
-_PROTOTYPE(extern int is_file_named,(char *p, umode_t ty));
+_PROTOTYPE(extern int is_file_named,(char *p, int cd));
 _PROTOTYPE(extern void process_inode,(struct inode *fi, struct file_operations *fop));
 _PROTOTYPE(extern void process_isock,(struct inode *i));
