@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.12 98/05/22 07:14:23 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.13 99/01/25 07:04:48 abe Exp $";
 #endif
 
 
@@ -777,6 +777,9 @@ process_isock(i)
 
 #if	defined(HASUNMINSOCK)
 	    (void) getunixnm(&sk, Namech, &Namech[MAXPATHLEN - 1]);
+	    if (Namech[0]
+	    &&  is_file_named(Namech,((i->i_mode & S_IFMT) == S_IFCHR) ? 1 : 0))
+		Lf->sf |= SELNM;
 	    if ((pr = sk.pair) || (pr = sk.protinfo.af_unix.other)) {
 		cp = endnm();
 		(void) sprintf(dev_ch, "->0x%08x", pr);
@@ -797,11 +800,28 @@ process_isock(i)
 		    len = MAXPATHLEN - 1;
 		(void) strncpy(Namech, (char *)&ud.sockaddr_un.sun_path, len);
 		Namech[len] = '\0';
+		if (Namech[0]
+		&&  is_file_named(Namech,
+				  ((i->i_mode & S_IFMT) == S_IFCHR) ? 1 : 0))
+		    Lf->sf |= SELNM;
 	    }
 	    if (ud.peerupd
 	    &&  kread((KA_T)ud.peerupd, (char *)&ud, sizeof(ud)) == 0) {
 		cp = endnm();
 		len = &Namech[MAXPATHLEN - 1] - cp;
+		if (len > 2) {
+		    (void) strcpy(cp, "->");
+		    cp += 2;
+		    len -= 2;
+		}
+	 	if (ud.sockaddr_len) {
+		    if (ud.sockaddr_len < sizeof(sockaddr_un.sun_path)) {
+			ud.sockaddr_un.sun_path[ud.sockaddr_len] = '\0';
+			if (is_file_named(ud.sockaddr_un.sun_path
+				  ((i->i_mode & S_IFMT) == S_IFCHR) ? 1 : 0))
+			    Lf->sf |= SELNM;
+		    }
+		}
 		if (ud.sockaddr_len < len)
 		    len = ud.sockaddr_len;
 		if (len) {
@@ -814,10 +834,6 @@ process_isock(i)
 	    (void) sprintf(dev_ch, "0x%08x", sp->SOCKET_SOCK);
 	    enter_dev_ch(dev_ch);
 	    Lf->off_def = 1;
-	    if (Namech[0] && Sfile) {
-		if (is_file_named(Namech, i->i_mode & S_IFMT))
-		    Lf->sf |= SELNM;
-	    }
 	    break;
 
 #if	defined(HASUNMINSOCK)
