@@ -31,20 +31,20 @@
 
 
 /*
- * $Id: machine.h,v 1.20 98/06/29 15:38:36 abe Exp $
+ * $Id: machine.h,v 1.25 99/10/13 09:16:28 abe Exp $
  */
 
 
-#if	!defined(MACHINE_H)
-#define	MACHINE_H	1
+#if	!defined(LSOF_MACHINE_H)
+#define	LSOF_MACHINE_H	1
 
 
-#if	LINUXV>=21035
+#if	LINUXV>=21035 || !defined(GLIBCV)
 #define	__KERNEL__	1
-#endif	/* LINUXV>=21035 */
+#endif	/* LINUXV>=21035 || !defined(GLIBCV) */
 
 
-#if	defined(GLIBCV) && GLIBCV>=2
+#if	defined(GLIBCV) && GLIBCV>=200
 
 /*
  * Avoid glibc2 types in favor of kernel types.
@@ -130,9 +130,9 @@ typedef	__kernel_pid_t		pid_t;
 typedef	__kernel_suseconds_t	suseconds_t;
 # endif	/* LINUXV>=21072 */
 typedef	__kernel_uid_t		uid_t;
-#else	/* !defined(GLIBCV) || GLIBCV<2 */
+#else	/* !defined(GLIBCV) || GLIBCV<200 */
 #include <sys/types.h>
-#endif	defined(GLIBCV) && GLIBCV>=2
+#endif	defined(GLIBCV) && GLIBCV>=200
 
 #include <sys/param.h>
 
@@ -141,6 +141,16 @@ typedef	__kernel_uid_t		uid_t;
 #endif	/* LINUXV>=21035 */
 
 
+/*
+ * CAN_USE_CLNT_CREATE is defined for those dialects where RPC clnt_create()
+ * can be used to obtain a CLIENT handle in lieu of clnttcp_create().
+ */
+
+#if	LINUXV>=20035
+#define	CAN_USE_CLNT_CREATE	1
+#endif	/* LINUXV>=20035 */
+
+
 /*
  * GET_MAX_FD is defined for those dialects that provide a function other than
  * getdtablesize() to obtain the maximum file descriptor number plus one.
@@ -195,7 +205,7 @@ typedef	__kernel_uid_t		uid_t;
 #define	HASENVDC	"LSOFDEVCACHE"
 #define	HASPERSDC	"%h/%p.lsof_%L"
 #define	HASPERSDCPATH	"LSOFPERSDCPATH"
-/* #define	HASSYSDC	"/var/adm/lsof.dc" */
+/* #define	HASSYSDC	"/your/choice/of/path" */
 
 
 /*
@@ -220,6 +230,18 @@ typedef	__kernel_uid_t		uid_t;
 #define	HASFSINO	1
 
 
+/*
+ * HASFSTRUCT is defined if the dialect has a file structure.
+ *
+ * FSV_DEFAULT defines the default set of file structure values to list.
+ * It defaults to zero (0), but may be made up of a combination of the
+ * FSV_* symbols from lsof.h.
+ */
+
+#define	HASFSTRUCT	1
+/* #define	FSV_DEFAULT	FSV_? | FSV_? | FSV_? */
+
+
 /*
  * HASGNODE is defined for those dialects that have gnodes.
  */
@@ -273,21 +295,10 @@ typedef	__kernel_uid_t		uid_t;
  * alloc_lfile() function of proc.c, the SETLFILEADD macro may be defined
  * to do that.  The HASXOPT definition may be used to select the conditions
  * under which private lfile elements are used.
- *
- * Field identifiers 1 through 9 are reserved for dialect-specific field
- * output.  Define the string HASFIELDAP<x> as the help (-F?) name of the
- * private field and LISTLFIELAP<x> as a macro to list the field output.
- * (<x> is the field identifier character.)  HASFIELDAP<x> is used in proc.c
- * and store.c; LISTFIELDAP<x>, in proc.c.
- *
- * See ../dialectx/ptx for an example of the use of HASLFILEADD, SETLFILEADD,
- * and HASXOPT.
  */
 
-/* #define HASLFILEADD <macro of new element definitions>; */
-/* #define SETLFILEADD <macro of code to preset new elements>; */
-/* #define HASFIELDAP1 <string name of field for -F? output> */
-/* #define LISTLFILEAP1 <macro to list field output>; */
+/* #define HASLFILEADD int ... */
+/* #define SETLFILEADD Lf->... */
 
 
 /*
@@ -326,7 +337,7 @@ typedef	__kernel_uid_t		uid_t;
  * NOTE: don't forget to define a prototype for this function in dproto.h.
  */
 
-/* #define	HASPIPEFN	process_pipe */
+/* #define	HASPIPEFN	process_pipe? */
 
 
 /*
@@ -361,11 +372,11 @@ typedef	__kernel_uid_t		uid_t;
  * called from print_file().
  */
 
-/* #define	HASPRINTDEV	print_dev	*/
-/* #define	HASPRINTINO	print_ino	*/
-/* #define	HASPRINTNM	print_nm	*/
-/* #define	HASPRINTOFF	print_off	*/
-/* #define	HASPRINTSZ	print_sz	*/
+/* #define	HASPRINTDEV	print_dev?	*/
+/* #define	HASPRINTINO	print_ino?	*/
+/* #define	HASPRINTNM	print_nm?	*/
+/* #define	HASPRINTOFF	print_off?	*/
+/* #define	HASPRINTSZ	print_sz?	*/
 
 
 /*
@@ -378,7 +389,7 @@ typedef	__kernel_uid_t		uid_t;
  * HASPRIVFILETYPE is the name of the processing function.
  */
 
-/* #define	HASPRIVFILETYPE	process_shmf	*/
+/* #define	HASPRIVFILETYPE	process_shmf?	*/
 /* #define	PRIVFILETYPE	??	*/
 
 
@@ -412,7 +423,7 @@ typedef	__kernel_uid_t		uid_t;
  * by inode number.
  */
 
-/* #define	HASPROCFS	"proc" */
+/* #define	HASPROCFS	"proc?" */
 /* #define	HASFSTYPE	1 */
 /* #define	HASPINODEN	1 */
 
@@ -511,7 +522,7 @@ typedef	__kernel_uid_t		uid_t;
 #define	USE_LIB_CKKV				1	/* ckkv.c */
 /* #define	USE_LIB_COMPLETEVFS		1	   cvfs.c */
 #define	USE_LIB_FIND_CH_INO			1	/* fino.c */
-/* #define	USE_LIB_IS_FILE_NAMED		1	   isfn.c */
+#define	USE_LIB_IS_FILE_NAMED			1	/* isfn.c */
 #define	USE_LIB_LKUPDEV				1	/* lkud.c */
 #define	USE_LIB_PRINTCHDEVNAME			1	/* pdvn.c */
 /* #define	USE_LIB_PROCESS_FILE		1	   prfp.c */
@@ -555,4 +566,4 @@ typedef	__kernel_uid_t		uid_t;
 
 #define	zeromem(a, l)	bzero(a, l)
 
-#endif	/* !defined(MACHINE_H) */
+#endif	/* !defined(LSOF_MACHINE_H) */
