@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.5 98/04/29 13:43:12 abe Exp $
+ * $Id: dlsof.h,v 1.9 99/07/19 14:01:54 abe Exp $
  */
 
 
@@ -39,6 +39,7 @@
 #define	LINUX_LSOF_H	1
 
 #include <dirent.h>
+#define	DIRTYPE	dirent			/* for arg.c's enter_dir() */
 #include <fcntl.h>
 #include <malloc.h>
 #include <signal.h>
@@ -46,14 +47,14 @@
 #include <string.h>
 #include <setjmp.h>
 #include <unistd.h>
-#include <linux/tcp.h>
-#undef	htonl
-#undef	ntohl
-#define	htonl	BYTEORDER_htonl
-#define	ntohl	BYTEORDER_ntohl
 #include <netinet/in.h>
-#undef	htonl
-#undef	ntohl
+
+# if	defined(GLIBCV)
+#include <netinet/tcp.h>
+# else	/* !defined(GLIBCV) */
+#include <linux/tcp.h>
+# endif	/* defined(GLIBCV) */
+
 #include <rpc/rpc.h>
 #include <rpc/pmap_prot.h>
 #include <sys/sysmacros.h>
@@ -74,7 +75,6 @@ typedef	off_t		KA_T;
 
 #define	COMP_P		const void
 #define DEVINCR		1024	/* device table malloc() increment */
-#define	DOSTAT_FUNCTION	stat	/* ask dostat() (misc.c) to use stat() */
 #define	FSNAMEL		4
 #define MALLOC_P	void
 #define FREE_P		MALLOC_P
@@ -112,7 +112,7 @@ struct sfile {
 	char *name;			/* file name (after readlink()) */
 	char *devnm;			/* device name (optional) */
 	dev_t dev;			/* device */
-	umode_t mode;			/* S_IFMT mode bits from stat() */
+	mode_t mode;			/* S_IFMT mode bits from stat() */
 	int type;			/* file type: 0 = file system
 				 	 *	      1 = regular file */
 	ino_t i;			/* inode number */
