@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1997 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.6 98/06/29 15:38:09 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.8 99/03/29 07:17:49 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -238,12 +238,13 @@ gather_proc_info()
 		if (f)
 		    continue;
 		(void) make_proc_path(dpath, i, &path, &pathl, fp->d_name);
-		if (HasNFS)
-		    sv = statsafely(path, &sb);
-		else
-		    sv = stat(path, &sb);
-		if (sv || lstat(path, &lsb))
-		    continue;
+		if (HasNFS) {
+		    if (statsafely(path, &sb) || lstatsafely(path, &lsb))
+			continue;
+		} else {
+		    if (stat(path, &sb) || lstat(path, &lsb))
+			continue;
+		}
 		if (getlinksrc(path, pbuf, sizeof(pbuf)) > 0) {
 		    (void) alloc_lfile((char *)NULL, fd);
 		    process_proc_node(pbuf, &sb, &lsb);
@@ -295,14 +296,6 @@ initialize()
 	int off = 0;
 	char path[MAXPATHLEN];
 	struct stat sb;
-
-#if	defined(HASKERNIDCK)
-/*
- * Check kernel identity.
- */
-	(void) ckkv("Linux", LSOF_VSTR, (char *)NULL, (char *)NULL);
-#endif	/* defined(HASKERNIDCK) */
-
 /*
  * Open LSTAT_TEST_FILE and seek to byte LSTAT_TEST_SEEK, then lstat the
  * /proc/<PID>/fd/<FD> for LSTAT_TEST_FILE to see what position is reported.
