@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1997 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.4 98/05/22 07:14:38 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.15 10/.0/.1 .0:.0:.4 abe Exp $";
 #endif
 
 
@@ -71,9 +71,21 @@ struct ipxsin {				/* IPX socket information */
 	struct ipxsin *next;
 };
 
-struct tcp_udp {			/* TCP and UDP socket information */
+struct rawsin {				/* raw socket information */
 	unsigned long inode;
-	unsigned long faddr, laddr;	/* foreign & local Internet addresses */
+	char *la;			/* local address */
+	char *ra;			/* remote address */
+	char *sp;			/* state characters */
+	MALLOC_S lal;			/* strlen(la) */
+	MALLOC_S ral;			/* strlen(ra) */
+	MALLOC_S spl;			/* strlen(sp) */
+	struct rawsin *next;
+};
+
+struct tcp_udp {			/* IPv4 TCP and UDP socket
+					 * information */
+	unsigned long inode;
+	unsigned long faddr, laddr;	/* foreign & local IPv6 addresses */
 	int fport, lport;		/* foreign & local ports */
 	unsigned long txq, rxq;		/* trasmit & receve queue values */
 	int proto;			/* 0 = TCP, 1 = UDP */
@@ -81,9 +93,22 @@ struct tcp_udp {			/* TCP and UDP socket information */
 	struct tcp_udp *next;
 };
 
+#if	defined(HASIPv6)
+struct tcp_udp6 {			/* IPv6 TCP and UDP socket
+					 * information */
+	unsigned long inode;
+	struct in6_addr faddr, laddr;	/* foreign and local IPv6 addresses */
+	int fport, lport;		/* foreign & local ports */
+	unsigned long txq, rxq;		/* trasmit & receve queue values */
+	int proto;			/* 0 = TCP, 1 = UDP */
+	int state;			/* protocol state */
+	struct tcp_udp6 *next;
+};
+#endif	/* defined(HASIPv6) */
+
 struct uxsin {				/* UNIX socket information */
 	unsigned long inode;
-	unsigned long pcb;
+	char *pcb;
 	char *path;
 	struct uxsin *next;
 };
@@ -97,8 +122,18 @@ static struct ax25sin **AX25sin = (struct ax25sin **)NULL;
 					/* AX25 socket info, hashed by inode */
 static struct ipxsin **Ipxsin = (struct ipxsin **)NULL;
 					/* IPX socket info, hashed by inode */
+static struct rawsin **Rawsin = (struct rawsin **)NULL;
+					/* raw socket info, hashed by inode */
 static struct tcp_udp **TcpUdp = (struct tcp_udp **)NULL;
-					/* TCP & UDP info, hashed by inode */
+					/* IPv4 TCP & UDP info, hashed by
+					 * inode */
+
+#if	defined(HASIPv6)
+static struct tcp_udp6 **TcpUdp6 = (struct tcp_udp6 **)NULL;
+					/* IPv6 TCP & UDP info, hashed by
+					 * inode */
+#endif	/* defined(HASIPv6) */
+
 static struct uxsin **Uxsin = (struct uxsin **)NULL;
 					/* UNIX socket info, hashed by inode */
 
@@ -109,14 +144,24 @@ static struct uxsin **Uxsin = (struct uxsin **)NULL;
 
 _PROTOTYPE(static struct ax25sin *check_ax25,(unsigned long i));
 _PROTOTYPE(static struct ipxsin *check_ipx,(unsigned long i));
+_PROTOTYPE(static struct rawsin *check_raw,(unsigned long i));
 _PROTOTYPE(static struct tcp_udp *check_tcpudp,(unsigned long i, char **p));
 _PROTOTYPE(static struct uxsin *check_unix,(unsigned long i));
 _PROTOTYPE(static void get_ax25,(char *p));
 _PROTOTYPE(static void get_ipx,(char *p));
+_PROTOTYPE(static void get_raw,(char *p));
 _PROTOTYPE(static void get_tcpudp,(char *p, int pr, int clr));
 _PROTOTYPE(static void get_unix,(char *p));
 _PROTOTYPE(static void print_ax25info,(struct ax25sin *ap));
 _PROTOTYPE(static void print_ipxinfo,(struct ipxsin *ip));
+_PROTOTYPE(static void print_rawinfo,(struct rawsin *ip));
+
+#if	defined(HASIPv6)
+_PROTOTYPE(static struct tcp_udp6 *check_tcpudp6,(unsigned long i, char **p));
+_PROTOTYPE(static void get_tcpudp6,(char *p, int pr, int clr));
+_PROTOTYPE(static int net6a2in6,(char *as, struct in6_addr *ad));
+#endif	/* defined(HASIPv6) */
+
 
 
 /*
@@ -162,7 +207,27 @@ check_ipx(i)
 
 
 /*
- * check_tcpudp() - check for TCP or UDP socket file
+ * check_raw() - check for raw socket file
+ */
+
+static struct rawsin *
+check_raw(i)
+	unsigned long i;		/* socket file's inode number */
+{
+	int h;
+	struct rawsin *rp;
+
+	h = INOHASH(i);
+	for (rp = Rawsin[h]; rp; rp = rp->next) {
+	    if (i == rp->inode)
+		return(rp);
+	}
+	return((struct rawsin *)NULL);
+}
+
+
+/*
+ * check_tcpudp() - check for IPv4 TCP or UDP socket file
  */
 
 static struct tcp_udp *
@@ -184,13 +249,38 @@ check_tcpudp(i, p)
 }
 
 
+#if	defined(HASIPv6)
+/*
+ * check_tcpudp6() - check for IPv6 TCP or UDP socket file
+ */
+
+static struct tcp_udp6 *
+check_tcpudp6(i, p)
+	unsigned long i;		/* socket file's inode number */
+	char **p;			/* protocol return */
+{
+	int h;
+	struct tcp_udp6 *tp6;
+
+	h = INOHASH(i);
+	for (tp6 = TcpUdp6[h]; tp6; tp6 = tp6->next) {
+	    if (i == tp6->inode) {
+		*p = tp6->proto ? "UDP" : "TCP";
+		return(tp6);
+	    }
+	}
+	return((struct tcp_udp6 *)NULL);
+}
+#endif	/* defined(HASIPv6) */
+
+
 /*
  * check_unix() - check for UNIX domain socket
  */
 
 static struct uxsin *
 check_unix(i)
-	unsigned long i;		/* socket's inode number */
+	unsigned long i;		/* socket file's inode number */
 {
 	int h;
 	struct uxsin *up;
@@ -294,7 +384,7 @@ get_ax25(p)
 		continue;
 	    }
 	/*
-	 * Assemble the inode number and see if is has already been recorded.
+	 * Assemble the inode number and see if it has already been recorded.
 	 */
 	    ep = (char *)NULL;
 	    if (!fp[18] || !*fp[18]
@@ -470,7 +560,7 @@ get_ipx(p)
 		continue;
 	    }
 	/*
-	 * Assemble the inode number and see if this inode is already
+	 * Assemble the inode number and see if the inode is already
 	 * recorded.
 	 */
 	    ep = (char *)NULL;
@@ -513,7 +603,8 @@ get_ipx(p)
 		    Exit(1);
 		}
 		(void) strcpy(la, fp[0]);
-	    }
+	    } else
+		la = (char *)NULL;
 	/*
 	 * Allocate space for the remote address, unless it is "Not_Connected".
 	 */
@@ -527,7 +618,8 @@ get_ipx(p)
 		    Exit(1);
 		}
 		(void) strcpy(ra, fp[1]);
-	    }
+	    } else
+		ra = (char *)NULL;
 	/*
 	 * Allocate space for an ipxsin entry, fill it, and link it to its
 	 * hash bucket.
@@ -569,17 +661,175 @@ get_net(p, pl)
 	(void) get_ax25(path);
 	(void) make_proc_path(p, pl, &path, &pathl, "ipx");
 	(void) get_ipx(path);
+	(void) make_proc_path(p, pl, &path, &pathl, "raw");
+	(void) get_raw(path);
 	(void) make_proc_path(p, pl, &path, &pathl, "tcp");
 	(void) get_tcpudp(path, 0, 1);
 	(void) make_proc_path(p, pl, &path, &pathl, "udp");
 	(void) get_tcpudp(path, 1, 0);
+
+#if	defined(HASIPv6)
+	(void) make_proc_path(p, pl, &path, &pathl, "tcp6");
+	(void) get_tcpudp6(path, 0, 1);
+	(void) make_proc_path(p, pl, &path, &pathl, "udp6");
+	(void) get_tcpudp6(path, 1, 0);
+#endif	/* defined(HASIPv6) */
+
 	(void) make_proc_path(p, pl, &path, &pathl, "unix");
 	(void) get_unix(path);
 }
 
 
 /*
- * get_tcpudp() - get TCP or UDP net info
+ * get_raw() - get /proc/net/raw info
+ */
+
+static void
+get_raw(p)
+	char *p;			/* /proc/net/raw path */
+{
+	char buf[MAXPATHLEN], *ep, **fp, *la, *ra, *sp;
+	int h;
+	unsigned long inode;
+	int nf = 12;
+	struct rawsin *np, *rp;
+	MALLOC_S lal, ral, spl;
+	FILE *xs;
+/*
+ * Do second time cleanup or first time setup.
+ */
+	if (Rawsin) {
+	    for (h = 0; h < INOBUCKS; h++) {
+		for (rp = Rawsin[h]; rp; rp = np) {
+		    np = rp->next;
+		    if (rp->la)
+			(void) free((FREE_P *)rp->la);
+		    if (rp->ra)
+			(void) free((FREE_P *)rp->ra);
+		    (void) free((FREE_P *)rp);
+		}
+		Rawsin[h] = (struct rawsin *)NULL;
+	    }
+	} else {
+	    Rawsin = (struct rawsin **)calloc(INOBUCKS,
+					      sizeof(struct rawsin *));
+	    if (!Rawsin) {
+		(void) fprintf(stderr,
+		    "%s: can't allocate %d raw hash pointer bytes\n",
+		    Pn, INOBUCKS * sizeof(struct rawsin *));
+		Exit(1);
+	    }
+	}
+/*
+ * Open and read the /proc/net/raw file.  Store raw socket info in the
+ * Rawsin[] hash buckets.
+ */
+	if (!(xs = fopen(p, "r")))
+	    return;
+	while (fgets(buf, sizeof(buf) - 1, xs)) {
+	    if (get_fields(buf, (char *)NULL, &fp) < nf)
+		continue;
+	    if (nf == 12) {
+
+	    /*
+	     * Check the column labels in the first line.
+	     */
+		if (!fp[1]  || strcmp(fp[1],  "local_address")
+		||  !fp[2]  || strcmp(fp[2],  "rem_address")
+		||  !fp[3]  || strcmp(fp[3],  "st")
+		||  !fp[11] || strcmp(fp[11], "inode"))
+		{
+		    if (!Fwarn) {
+			(void) fprintf(stderr,
+			    "%s: WARNING: unsupported format: %s\n",
+			    Pn, p);
+		    }
+		    break;
+		}
+		nf = 10;
+		continue;
+	    }
+	/*
+	 * Assemble the inode number and see if the inode is already
+	 * recorded.
+	 */
+	    ep = (char *)NULL;
+	    if (!fp[9] || !*fp[9]
+	    ||  (inode = strtoul(fp[9], &ep, 0)) == ULONG_MAX
+	    ||  !ep || *ep)
+		continue;
+	    h = INOHASH(inode);
+	    for (rp = Rawsin[h]; rp; rp = rp->next) {
+		if (inode == rp->inode)
+		    break;
+	    }
+	    if (rp)
+		continue;
+	/*
+	 * Save the local address, remote address, and state.
+	 */
+	    if (!fp[1] || !*fp[1] || (lal = strlen(fp[1])) < 1) {
+		la = (char *)NULL;
+		lal = (MALLOC_S)0;
+	    } else {
+		if (!(la = (char *)malloc(lal + 1))) {
+		    (void) fprintf(stderr,
+			"%s: can't allocate %d local raw address bytes: %s\n",
+			Pn, lal + 1, fp[1]);
+		    Exit(1);
+		}
+		(void) strcpy(la, fp[1]);
+	    }
+	    if (!fp[2] || !*fp[2] || (ral = strlen(fp[2])) < 1) {
+		ra = (char *)NULL;
+		ral = (MALLOC_S)0;
+	    } else {
+		if (!(ra = (char *)malloc(ral + 1))) {
+		    (void) fprintf(stderr,
+			"%s: can't allocate %d remote raw address bytes: %s\n",
+			Pn, ral + 1, fp[2]);
+		    Exit(1);
+		}
+		(void) strcpy(ra, fp[2]);
+	    }
+	    if (!fp[3] || !*fp[3] || (spl = strlen(fp[3])) < 1) {
+		sp = (char *)NULL;
+		spl = (MALLOC_S)0;
+	    } else {
+		if (!(sp = (char *)malloc(spl + 1))) {
+		    (void) fprintf(stderr,
+			"%s: can't allocate %d remote raw state bytes: %s\n",
+			Pn, spl + 1, fp[2]);
+		    Exit(1);
+		}
+		(void) strcpy(sp, fp[3]);
+	    }
+	/*
+	 * Allocate space for an rawsin entry, fill it, and link it to its
+	 * hash bucket.
+	 */
+	    if (!(rp = (struct rawsin *)malloc(sizeof(struct rawsin)))) {
+		(void) fprintf(stderr,
+		    "%s: can't allocate %d byte rawsin structure\n",
+		    Pn, sizeof(struct rawsin));
+		Exit(1);
+	    }
+	    rp->inode = inode;
+	    rp->la = la;
+	    rp->lal = lal;
+	    rp->ra = ra;
+	    rp->ral = ral;
+	    rp->sp = sp;
+	    rp->spl = spl;
+	    rp->next = Rawsin[h];
+	    Rawsin[h] = rp;
+	}
+	(void) fclose(xs);
+}
+
+
+/*
+ * get_tcpudp() - get IPv4 TCP or UDP IPv4 net info
  */
 
 static void
@@ -623,7 +873,7 @@ get_tcpudp(p, pr, clr)
 	    return;
 	nf = 12;
 	while(fgets(buf, sizeof(buf) - 1, fs)) {
-	    if (get_fields(buf, (nf == 12) ? (char *)NULL : ":", &fp) != nf)
+	    if (get_fields(buf, (nf == 12) ? (char *)NULL : ":", &fp) < nf)
 		continue;
 	    if (nf == 12) {
 		if (!fp[1]  || strcmp(fp[1],  "local_address")
@@ -716,6 +966,143 @@ get_tcpudp(p, pr, clr)
 }
 
 
+#if	defined(HASIPv6)
+/*
+ * get_tcpudp6() - get IPv6 TCP or UDP IPv4 net info
+ */
+
+static void
+get_tcpudp6(p, pr, clr)
+	char *p;			/* /proc/net/{tcp,udp} path */
+	int pr;				/* protocol: 0 = TCP, 1 = UDP */
+	int clr;			/* 1 == clear the table */
+{
+	char buf[MAXPATHLEN], *ep, **fp;
+	struct in6_addr faddr, laddr;
+	unsigned long fport, inode, lport, rxq, state, txq;
+	int h, nf;
+	FILE *fs;
+	struct tcp_udp6 *np6, *tp6;
+/*
+ * Delete previous table contents.  Allocate a table for the first time.
+ */
+	if (TcpUdp6) {
+	    if (clr) {
+		for (h = 0; h < INOBUCKS; h++) {
+		    for (tp6 = TcpUdp6[h]; tp6; tp6 = np6) {
+			np6 = tp6->next;
+			(void) free((FREE_P *)tp6);
+		    }
+		    TcpUdp6[h] = (struct tcp_udp6 *)NULL;
+		}
+	    }
+	} else {
+	    if (!(TcpUdp6 = (struct tcp_udp6 **)calloc(INOBUCKS,
+						sizeof(struct tcp_udp6 *))))
+	    {
+		(void) fprintf(stderr,
+		    "%s: can't allocate %d bytes for TCP6&UDP6 hash buckets\n",
+		    Pn, INOBUCKS * sizeof(struct tcp_udp6 *));
+		Exit(1);
+	    }
+	}
+/*
+ * Open and read the /proc/net file.
+ */
+	if (!(fs = fopen(p, "r")))
+	    return;
+	nf = 12;
+	while(fgets(buf, sizeof(buf) - 1, fs)) {
+	    if (get_fields(buf, (nf == 12) ? (char *)NULL : ":", &fp) < nf)
+		continue;
+	    if (nf == 12) {
+		if (!fp[1]  || strcmp(fp[1],  "local_address")
+		||  !fp[2]  || strcmp(fp[2],  "remote_address")
+		||  !fp[3]  || strcmp(fp[3],  "st")
+		||  !fp[4]  || strcmp(fp[4],  "tx_queue")
+		||  !fp[5]  || strcmp(fp[5],  "rx_queue")
+		||  !fp[11] || strcmp(fp[11], "inode"))
+		{
+		    if (!Fwarn) {
+			(void) fprintf(stderr,
+			    "%s: WARNING: unsupported format: %s\n",
+			    Pn, p);
+		    }
+		    break;
+		}
+		nf = 14;
+		continue;
+	    }
+	/*
+	 * Get the local and remote addresses.
+	 */
+	    if (!fp[1] || !*fp[1] || net6a2in6(fp[1], &laddr))
+		continue;
+	    ep = (char *)NULL;
+	    if (!fp[2] || !*fp[2]
+	    ||  (lport = strtoul(fp[2], &ep, 16)) == ULONG_MAX || !ep || *ep)
+		continue;
+	    if (!fp[3] || !*fp[3] || net6a2in6(fp[3], &faddr))
+		continue;
+	    ep = (char *)NULL;
+	    if (!fp[4] || !*fp[4]
+	    ||  (fport = strtoul(fp[4], &ep, 16)) == ULONG_MAX || !ep || *ep)
+		continue;
+	/*
+	 * Get the state, queue sizes, and inode.
+	 */
+	    ep = (char *)NULL;
+	    if (!fp[5] || !*fp[5]
+	    ||  (state = strtoul(fp[5], &ep, 16)) == ULONG_MAX || !ep || *ep)
+		continue;
+	    ep = (char *)NULL;
+	    if (!fp[6] || !*fp[6]
+	    ||  (txq = strtoul(fp[6], &ep, 16)) == ULONG_MAX || !ep || *ep)
+		continue;
+	    ep = (char *)NULL;
+	    if (!fp[7] || !*fp[7]
+	    ||  (rxq = strtoul(fp[7], &ep, 16)) == ULONG_MAX || !ep || *ep)
+		continue;
+	/*
+	 * Get the inode and use it for hashing and searching.
+	 */
+	    ep = (char *)NULL;
+	    if (!fp[13] || !*fp[13]
+	    ||  (inode = strtoul(fp[13], &ep, 0)) == ULONG_MAX || !ep || *ep)
+		continue;
+	    h = INOHASH(inode);
+	    for (tp6 = TcpUdp6[h]; tp6; tp6 = tp6->next) {
+		if (tp6->inode == inode)
+		    break;
+	    }
+	    if (tp6)
+		continue;
+	/*
+	 * Create a new entry and link it to its hash bucket.
+	 */
+	    if (!(tp6 = (struct tcp_udp6 *)malloc(sizeof(struct tcp_udp6)))) {
+		(void) fprintf(stderr,
+		    "%s: can't allocate %d bytes for tcp_udp6 struct\n",
+		    Pn, sizeof(struct tcp_udp6));
+		Exit(1);
+	    }
+	    tp6->inode = inode;
+	    tp6->faddr = faddr;
+	    tp6->fport = (int)(fport & 0xffff);
+	    tp6->laddr = laddr;
+	    tp6->lport = (int)(lport & 0xffff);
+	    tp6->txq = txq;
+	    tp6->rxq = rxq;
+	    tp6->proto = pr;
+	    tp6->state = (int)state;
+	    tp6->next = TcpUdp6[h];
+	    TcpUdp6[h] = tp6;
+	}
+	(void) fclose(fs);
+}
+#endif	/* defined(HASIPv6) */
+
+
 /*
  * get_unix() - get UNIX net info
  */
@@ -724,10 +1111,10 @@ static void
 get_unix(p)
 	char *p;			/* /proc/net/unix path */
 {
-	char buf[MAXPATHLEN], *ep, **fp, *path;
+	char buf[MAXPATHLEN], *ep, **fp, *path, *pcb;
 	int fl = 1;
 	int h, nf;
-	unsigned long inode, pcb;
+	unsigned long inode;
 	MALLOC_S len;
 	struct uxsin *np, *up;
 	FILE *us;
@@ -801,10 +1188,18 @@ get_unix(p)
 	    }
 	    if (up)
 		continue;
-	    ep = (char *)NULL;
-	    if (!fp[0] || !*fp[0]
-	    ||  (pcb = strtoul(fp[0], &ep, 16)) == ULONG_MAX || !ep || *ep)
-		continue;
+	    if (!fp[0] || !*fp[0])
+		pcb = (char *)NULL;
+	    else {
+		len = strlen(fp[0]) + 2;
+		if (!(pcb = (char *)malloc(len + 1))) {
+		    (void) fprintf(stderr,
+			"%s: can't allocate %d bytes for UNIX PCB: %s\n",
+			Pn, len + 1, fp[0]);
+		    Exit(1);
+		}
+		(void) sprintf(pcb, "0x%s", fp[0]);
+	    }
 	    if (nf >= 8
 	    &&  fp[7] && *fp[7] && *fp[7] != '@'
 	    &&  (len = strlen(fp[7]))) {
@@ -818,7 +1213,7 @@ get_unix(p)
 	    } else
 		path = (char *)NULL;
 	/*
-	 * Allocate and fill a Unix socket infor structure; link it to its
+	 * Allocate and fill a Unix socket info structure; link it to its
 	 * hash bucket.
 	 */
 	    if (!(up = (struct uxsin *)malloc(sizeof(struct uxsin)))) {
@@ -837,6 +1232,39 @@ get_unix(p)
 }
 
 
+#if	defined(HASIPv6)
+/*
+ * net6a2in6() - convert ASCII IPv6 address in /proc/net/{tcp,udp} form to
+ *		 an in6_addr
+ */
+
+static int
+net6a2in6(as, ad)
+	char *as;			/* address source */
+	struct in6_addr *ad;		/* address destination */
+{
+	char buf[9], *ep;
+	int i;
+	size_t len;
+/*
+ * Assemble four uint32_t's from 4 X 8 hex digits into s6_addr32[].
+ */
+	for (i = 0, len = strlen(as);
+	     (i < 4) && (len >= 8);
+	     as += 8, i++, len -= 8)
+	{
+	    (void) strncpy(buf, as, 8);
+	    buf[8] = '\0';
+	    ep = (char *)NULL;
+	    if ((ad->s6_addr32[i] = (uint32_t)strtoul(buf, &ep, 16))
+	    ==  ULONG_MAX || !ep || *ep)
+		break;
+	}
+	return((*as || (i != 4) || len) ? 1 : 0);
+}
+#endif	/* defined(HASIPv6) */
+
+
 /*
  * print_ax25info() - print AX25 socket info
  *
@@ -1060,8 +1488,15 @@ process_proc_sock(p, s, l)
 	struct ipxsin *ip;
 	int len, nl;
 	char *pr;
+	struct rawsin *rp;
 	struct tcp_udp *tp;
 	struct uxsin *up;
+
+#if	defined(HASIPv6)
+	int af;
+	struct tcp_udp6 *tp6;
+#endif	/* defined(HASIPv6) */
+
 /*
  * Enter offset, if possible.
  */
@@ -1076,19 +1511,25 @@ process_proc_sock(p, s, l)
 	if ((tp = check_tcpudp((unsigned long)s->st_ino, &pr))) {
 
 	/*
-	 * The inode is connected to a TCP or UDP /proc record.
+	 * The inode is connected to an IPv4 TCP or UDP /proc record.
 	 *
-	 * Set the type to "inet"; enter the protocol; put the inode number
-	 * in the DEVICE column in lieu of the PCB address; save the local
-	 * and foreign Internet addresses; save the type and protocol; and
-	 * (optionally) save the queue sizes.
+	 * Set the type to "inet" or "IPv4"; enter the protocol; put the
+	 * inode number in the DEVICE column in lieu of the PCB address;
+	 * save the local and foreign IPv4 addresses; save the type and
+	 * protocol; and (optionally) save the queue sizes.
 	 */
 	    if (Fnet)
 		Lf->sf |= SELNET;
+
+#if	defined(HASIPv6)
+	    (void) strcpy(Lf->type, "IPv4");
+#else	/* !defined(HASIPv6) */
 	    (void) strcpy(Lf->type, "inet");
+#endif	/* defined(HASIPv6) */
+
 	    (void) sprintf(Lf->iproto, "%.*s", IPROTOL-1, pr);
 	    Lf->inp_ty = 2;
-	    (void) sprintf(dev_ch, "%d", s->st_ino);
+	    (void) sprintf(dev_ch, "%ld", (long)s->st_ino);
 	    enter_dev_ch(dev_ch);
 	    if (tp->faddr || tp->fport) {
 		fs.s_addr = tp->faddr;
@@ -1112,6 +1553,57 @@ process_proc_sock(p, s, l)
 
 	    return;
 	}
+
+#if	defined(HASIPv6)
+	if ((tp6 = check_tcpudp6((unsigned long)s->st_ino, &pr))) {
+
+	/*
+	 * The inode is connected to an IPv6 TCP or UDP /proc record.
+	 *
+	 * Set the type to "IPv6"; enter the protocol; put the inode number
+	 * in the DEVICE column in lieu of the PCB address; save the local
+	 * and foreign IPv6 addresses; save the type and protocol; and
+	 * (optionally) save the queue sizes.
+	 */
+	    if (Fnet)
+		Lf->sf |= SELNET;
+	    (void) strcpy(Lf->type, "IPv6");
+	    (void) sprintf(Lf->iproto, "%.*s", IPROTOL-1, pr);
+	    Lf->inp_ty = 2;
+	    (void) sprintf(dev_ch, "%ld", (long)s->st_ino);
+	    enter_dev_ch(dev_ch);
+	    af = AF_INET6;
+	    if (!IN6_IS_ADDR_UNSPECIFIED(&tp6->faddr) || tp6->fport)
+		fa = (unsigned char *)&tp6->faddr;
+	    else
+		fa = (unsigned char *)NULL;
+	    if (!IN6_IS_ADDR_UNSPECIFIED(&tp6->laddr) || tp6->lport)
+		la = (unsigned char *)&tp6->laddr;
+	    else
+		la = (unsigned char *)NULL;
+	    if ((fa && IN6_IS_ADDR_V4MAPPED(&tp6->faddr))
+	    ||  (la && IN6_IS_ADDR_V4MAPPED(&tp6->laddr))) {
+		af = AF_INET;
+		if (fa)
+		    fa += 12;
+		if (la)
+		    la += 12;
+		Lf->type[3] = '4';
+	    }
+	    ent_inaddr(la, tp6->lport, fa, tp6->fport, af);
+	    Lf->lts.type = tp6->proto;
+	    Lf->lts.state.i = tp6->state;
+
+#if     defined(HASTCPTPIQ)
+	    Lf->lts.rq = tp6->rxq;
+	    Lf->lts.sq = tp6->txq;
+	    Lf->lts.rqs = Lf->lts.sqs = 1;
+#endif  /* defined(HASTCPTPIQ) */
+
+	    return;
+	}
+#endif	/* defined(HASIPv6) */
+
 	if ((up = check_unix((unsigned long)s->st_ino))) {
 
 	/*
@@ -1123,14 +1615,17 @@ process_proc_sock(p, s, l)
 	    if (Funix)
 		Lf->sf |= SELUNX;
 	    (void) strcpy(Lf->type, "unix");
-	    (void) sprintf(dev_ch, "0x%08lx", up->pcb);
-	    enter_dev_ch(dev_ch);
+	    if (up->pcb)
+		enter_dev_ch(up->pcb);
 	    Lf->inode = (unsigned long)s->st_ino;
 	    Lf->inp_ty = 1;
 	    path = up->path ? up->path : p;
 	    (void) enter_nm(path);
-	    if (Sfile && is_file_named(path, s->st_mode & S_IFMT))
+	    if (Sfile && is_file_named(path,
+				((s->st_mode & S_IFMT) == S_IFCHR) ? 1 : 0))
+	    {
 		Lf->sf |= SELNM;
+	    }
 	    return;
 	}
 	if ((ap = check_ax25((unsigned long)s->st_ino))) {
@@ -1155,7 +1650,7 @@ process_proc_sock(p, s, l)
 	if ((ip = check_ipx((unsigned long)s->st_ino))) {
 
 	/*
-	 * The inode is connected to a IPX /proc record.
+	 * The inode is connected to an IPX /proc record.
 	 *
 	 * Set the type to "ipx"; enter the inode and device numbers; store
 	 * the addresses, queue sizes, and state in the NAME column.
@@ -1204,6 +1699,62 @@ process_proc_sock(p, s, l)
 		enter_nm(Namech);
 	    return;
 	}
+	if ((rp = check_raw((unsigned long)s->st_ino))) {
+
+	/*
+	 * The inode is connected to a raw /proc record.
+	 *
+	 * Set the type to "raw"; enter the inode number; store the local
+	 * address, remote address, and state in the NAME column.
+	 */
+	    (void) strcpy(Lf->type, "raw");
+	    Lf->inode = (unsigned long)s->st_ino;
+	    Lf->inp_ty = 1;
+	    cp = Namech;
+	    nl = MAXPATHLEN - 2;
+	    if (rp->la && rp->lal) {
+
+	    /*
+	     * Store the local raw address.
+	     */
+		if (nl > rp->lal) {
+		    (void) strcpy(cp, rp->la);
+		    cp += rp->lal;
+		    *cp = '\0';
+		    nl -= rp->lal;
+		}
+	    }
+	    if (rp->ra && rp->ral) {
+
+	    /*
+	     * Store the remote raw address, prefixed with "->".
+	     */
+		if (nl > (rp->ral + 2)) {
+		    (void) sprintf(cp, "->%s", rp->ra);
+		    cp += (rp->ral + 2);
+		    *cp = '\0';
+		    nl -= (rp->ral + 2);
+		}
+	    }
+	    if (rp->sp && rp->spl) {
+
+	    /*
+	     * Store the state, optionally prefixed by a space, in the
+	     * form "st=x...x".
+	     */
+	    
+		if (nl > (len = ((cp == Namech) ? 0 : 1) + 3 + rp->spl)) {
+		    (void) sprintf(cp, "%sst=%s",
+			(cp == Namech) ? "" : " ", rp->sp);
+		    cp += len;
+		    *cp = '\0';
+		    nl -= len;
+		}
+	    }
+	    if (Namech[0])
+		enter_nm(Namech);
+	    return;
+	}
 /*
  * The socket's protocol can't be identified.
  */
