@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.10 98/05/22 07:18:22 abe Exp $
+ * $Id: dlsof.h,v 1.15 99/10/22 08:22:32 abe Exp $
  */
 
 
@@ -52,13 +52,29 @@
 #include <sys/mount.h>
 #include <rpc/types.h>
 #include <sys/protosw.h>
+
+# if	defined(NETBSDV) && NETBSDV>=1030
+#define	sockproto	NETBSD_sockproto
+# endif	/* defined(NETBSDV) && NETBSDV>=1030 */
+
 #include <sys/socket.h>
+
+# if	defined(NETBSDV) && NETBSDV>=1030
+#undef	sockproto
+# endif	/* defined(NETBSDV) && NETBSDV>=1030 */
+
 #include <sys/socketvar.h>
 #include <sys/un.h>
 #include <sys/unpcb.h>
 #include <netinet/in.h>
 #include <netinet/in_systm.h>
 #include <netinet/ip.h>
+
+# if	defined(HASIPv6) && defined(NETBSDV) && !defined(HASINRIAIPv6)
+#include <netinet6/ip6.h>
+#include <netinet6/in6_pcb.h>
+# endif	/* defined(HASIPv6) && defined(NETBSDV) && !defined(HASINRIAIPv6) */
+
 #include <net/route.h>
 #include <netinet/in_pcb.h>
 #include <netinet/ip_var.h>
@@ -79,6 +95,24 @@
 # endif	/* defined(UVM) */
 
 #include <sys/vnode.h>
+
+# if	defined(NETBSDV) && NETBSDV>=1030
+/*
+ * Because late in the 1.3I NetBSD development cycle the sockproto structure
+ * was placed under _KERNEL in <sys/socket.h>, and because defining _KERNEL
+ * before #include'ing <sys/socket.h> causes other #include problems, the
+ * sockproto structure definition that might have been in <sys/socket.h> is
+ * renamed NETBSD_sockproto, and the following definition is used instead.
+ *
+ * Ugly, isn't it?
+ */
+
+struct sockproto {
+	u_short sp_family;
+	u_short sp_protocol;
+};
+# endif	/* defined(NETBSDV) && NETBSDV>=1030 */
+
 #include <net/raw_cb.h>
 #include <sys/domain.h>
 #define	pmap	RPC_pmap
@@ -113,8 +147,6 @@
 #include <nfs/nfsnode.h>
 #include <sys/proc.h>
 #include <kvm.h>
-#undef	TRUE
-#undef	FALSE
 #include <sys/sysctl.h>
 #define	P_ADDR		kp_eproc.e_paddr
 #define	P_COMM		kp_proc.p_comm
@@ -162,6 +194,15 @@
 #include <vm/vm_pager.h>
 
 # if	defined(OPENBSDV)
+#  if	OPENBSDV==2030 && defined(__sparc__)
+#   if	defined(nbpg)
+#undef	nbpg
+#   endif	/* defined(nbpg) */
+#define	nbpg	4096		/* WARNING!!!  This should be 8192 for sun4,
+				 * but there's not much chance this value will
+				 * ever be used by any lsof code.  (See the
+				 * use of PIPE_NODIRECT in <sys/pipe.h>. */
+#  endif	/* OPENBSDV==2030 && defined(__sparc__) */
 #include <sys/pipe.h>
 #endif	/* defined(OPENBSDV) */
 
@@ -263,7 +304,7 @@ struct sfile {
 
 #  if	(defined(OPENBSDV) && OPENBSDV>=2010) || (defined(NETBSDV) && NETBSDV>=1020)
 #define	NCACHE_NXT	nc_hash.le_next	/* link in NCACHE */
-  #else	/* (defined(OPENBSDV) && OPENBSDV>=2010) || (defined(NETBSDV) && NETBSDV>=1020) */
+#  else	/* (defined(OPENBSDV) && OPENBSDV>=2010) || (defined(NETBSDV) && NETBSDV>=1020) */
 #   if	defined(NetBSD1_0) && NetBSD<1994101
 #define	NCACHE_NXT	nc_nxt		/* link in NCACHE */
 #   else	/* !defined(NetBSD1_0) || NetBSD>=1994101 */
