@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.6 98/06/29 15:54:17 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.9 99/06/22 16:04:01 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -151,12 +151,28 @@ gather_proc_info()
 {
 	struct filedesc fd;
 	int i, nf;
-	static int nofb = 0;
+	MALLOC_S nb;
 	static struct file **ofb = NULL;
+	static int ofbb = 0;
 	struct kinfo_proc *p;
 	short pss, sf;
 	int px;
 	uid_t uid;
+
+#if	defined(HASCWDINFO)
+	struct cwdinfo cw;
+#define	CDIR	cw.cwdi_cdir
+#define	RDIR	cw.cwdi_rdir
+#else	/* !defined(HASCWDINFO) */
+#define	CDIR	fd.fd_cdir
+#define	RDIR	fd.fd_rdir
+#endif	/* defined(HASCWDINFO) */
+
+#if	defined(HASFSTRUCT)
+	static char *pof = (char *)NULL;
+	static int pofb = 0;
+#endif	/* defined(HASFSTRUCT) */
+
 /*
  * Read the process table.
  */
@@ -199,6 +215,13 @@ gather_proc_info()
 		continue;
 	    if (!fd.fd_refcnt || fd.fd_lastfile > fd.fd_nfiles)
 		continue;
+
+#if	defined(HASCWDINFO)
+	    if (!p->kp_proc.p_cwdi
+	    ||  kread((KA_T)p->kp_proc.p_cwdi, (char *)&cw, sizeof(cw)))
+		CDIR = RDIR = (struct vnode *)NULL;
+#endif	/* defined(HASCWDINFO) */
+
 	/*
 	 * Allocate a local process structure.
 	 */
@@ -211,20 +234,20 @@ gather_proc_info()
 	/*
 	 * Save current working directory information.
 	 */
-	    if (fd.fd_cdir) {
+	    if (CDIR) {
 		alloc_lfile(CWD, -1);
 		Cfp = (struct file *)NULL;
-		process_node((KA_T)fd.fd_cdir);
+		process_node((KA_T)CDIR);
 		if (Lf->sf)
 		    link_lfile();
 	    }
 	/*
 	 * Save root directory information.
 	 */
-	    if (fd.fd_rdir) {
+	    if (RDIR) {
 		alloc_lfile(RTD, -1);
 		Cfp = (struct file *)NULL;
-		process_node((KA_T)fd.fd_rdir);
+		process_node((KA_T)RDIR);
 		if (Lf->sf)
 		    link_lfile();
 	    }
@@ -238,28 +261,42 @@ gather_proc_info()
 	 */
 	    if (!fd.fd_ofiles || (nf = fd.fd_nfiles) <= 0)
 		continue;
-	    if (!ofb) {
-		nofb = nf;
-		if (!(ofb = (struct file **)malloc((MALLOC_S)
-			    (nofb * sizeof(struct file *)))))
-		{
+	    nb = (MALLOC_S)(sizeof(struct file *) * nf);
+	    if (nb > ofbb) {
+		if (!ofb)
+		    ofb = (struct file **)malloc(nb);
+		else
+		    ofb = (struct file **)realloc((MALLOC_P *)ofb, nb);
+		if (!ofb) {
 		    (void) fprintf(stderr, "%s: PID %d, no file * space\n",
-			Pn, (int)p->P_PID);
-		    Exit(1);
-		}
-	    } else if (nf > nofb) {
-		nofb = fd.fd_nfiles;
-		if (!(ofb = (struct file **)realloc((MALLOC_P *)ofb,
-			    (nofb * sizeof(struct file *)))))
-		{
-		    (void) fprintf(stderr,
-			"%s: PID %d, no realloc file * space\n",
-			Pn, (int)p->P_PID);
+			Pn, p->P_PID);
 		    Exit(1);
 		}
+		ofbb = nb;
 	    }
-	    if (kread((KA_T)fd.fd_ofiles,(char *)ofb,nf*sizeof(struct file *)))
+	    if (kread((KA_T)fd.fd_ofiles, (char *)ofb, nb))
 		continue;
+
+#if	defined(HASFSTRUCT)
+	    if (Fsv & FSV_FG) {
+		nb = (MALLOC_S)(sizeof(char) * nf);
+		if (nb > pofb) {
+		    if (!pof)
+			pof = (char *)malloc(nb);
+		    else
+			pof = (char *)realloc((MALLOC_P *)pof, nb);
+		    if (!pof) {
+			(void) fprintf(stderr,
+			    "%s: PID %d, no file flag space\n", Pn, p->P_PID);
+		        Exit(1);
+		    }
+		    pofb = nb;
+		}
+		if (!fd.fd_ofileflags || kread((KA_T)fd.fd_ofileflags, pof, nb))
+		    zeromem(pof, nb);
+	    }
+#endif	/* defined(HASFSTRUCT) */
+
 	/*
 	 * Save information on file descriptors.
 	 */
@@ -267,8 +304,15 @@ gather_proc_info()
 		if (ofb[i]) {
 		    alloc_lfile(NULL, i);
 		    process_file((KA_T)(Cfp = ofb[i]));
-		    if (Lf->sf)
+		    if (Lf->sf) {
+
+#if	defined(HASFSTRUCT)
+			if (Fsv & FSV_FG)
+			    Lf->pof = (long)pof[i];
+#endif	/* defined(HASFSTRUCT) */
+
 			link_lfile();
+		    }
 		}
 	    }
 	/*
@@ -394,11 +438,11 @@ process_text(vm)
 	int i, j;
 	KA_T ka;
 	int n = 0;
-	struct pager_struct pg;
 	struct vm_map_entry vmme, *e;
 	struct vmspace vmsp;
 
 #if	!defined(UVM)
+	struct pager_struct pg;
 	struct vm_object vmo;
 #endif	/* !defined(UVM) */
 
@@ -410,8 +454,12 @@ process_text(vm)
 /*
  * Read the vm_map structure.  Search its vm_map_entry structure list.
  */
+
+#if	!defined(UVM)
 	if (!vmsp.vm_map.is_main_map)
 	    return;
+#endif	/* !defined(UVM) */
+
 	for (i = 0; i < vmsp.vm_map.nentries; i++) {
 
 	/*
