@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.8 98/08/03 07:43:23 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.13 99/11/28 06:40:56 abe Exp $";
 #endif
 
 
@@ -57,10 +57,15 @@ process_socket(sa)
 	struct protosw p;
 	struct socket s;
 	struct tcpcb t;
+	KA_T ta = (KA_T)NULL;
 	struct unpcb uc, unp;
 	struct sockaddr_un *ua = NULL;
 	struct sockaddr_un un;
 
+#if	defined(HASIPv6) && defined(NETBSDV) && !defined(HASINRIAIPv6)
+	struct in6pcb in6p;
+#endif	/* defined(HASIPv6) && defined(NETBSDV) && !defined(HASINRIAIPv6) */
+
 	(void) strcpy(Lf->type, "sock");
 	Lf->inp_ty = 2;
 /*
@@ -137,69 +142,91 @@ process_socket(sa)
 	    (void) strcpy(Lf->type, "inet");
 #endif	/* defined(HASIPv6) */
 
-	/*
-	 * Read protocol control block.
-	 */
-	    if (!s.so_pcb
-	    ||  kread((KA_T) s.so_pcb, (char *) &inp, sizeof(inp))) {
-		(void) sprintf(Namech, "can't read inpcb at %s",
-		    print_kptr((KA_T)s.so_pcb, (char *)NULL));
-		enter_nm(Namech);
-		return;
-	    }
-	/*
-	 * Print Internet socket information.
-	 */
-	    enter_dev_ch(print_kptr((KA_T)(inp.inp_ppcb ? inp.inp_ppcb
-							: s.so_pcb),
-				    (char *)NULL));
+#if	defined(HASIPv6) && defined(NETBSDV) && !defined(HASINRIAIPv6)
+	    if (fam == AF_INET6) {
 
-#if	defined(HASIPv6) && defined(HASINRIAIPv6)
-	    if (inp.inp_latype == IPATYPE_UNBD)
-		la = (unsigned char *)&in6addr_any;
-	    else if (inp.inp_latype == IPATYPE_IPV6) {
-		fam = AF_INET6;
-		Lf->type[3] = '6';
-		la = (unsigned char *)&inp.inp_laddr6;
-	    } else {
-		if (inp.inp_latype == IPATYPE_IPV4) {
-		    fam = AF_INET;
-		    Lf->type[3] = '4';
+	    /*
+	     * Read IPv6 protocol control block.
+	     */
+		if (!s.so_pcb
+		||  kread((KA_T)s.so_pcb, (char *)&in6p, sizeof(in6p))) {
+		    (void) sprintf(Namech, "can't read in6pcb at %s",
+			print_kptr((KA_T)s.so_pcb, (char *)NULL));
+		    enter_nm(Namech);
+		    return;
 		}
-		la = (unsigned char *)&inp.inp_laddr;
-	    }
-#else	/* !defined(HASIPv6) || !defined(HASINRIAIPv6) */
-	    la = (unsigned char *)&inp.inp_laddr;
-#endif	/* defined(HASIPv6) && defined(HASINRIAIPv6) */
+	    /*
+	     * Save IPv6 address information.
+	     */
+		enter_dev_ch(print_kptr((KA_T)(in6p.in6p_ppcb ? in6p.in6p_ppcb
+							      : s.so_pcb),
+					       (char *)NULL));
+		if (p.pr_protocol == IPPROTO_TCP)
+		    ta = (KA_T)in6p.in6p_ppcb;
+	        la = (unsigned char *)&in6p.in6p_laddr;
+	        lp = (int)ntohs(in6p.in6p_lport);
+		if (!IN6_IS_ADDR_UNSPECIFIED(&in6p.in6p_faddr)
+		||  in6p.in6p_fport)
+		{
+		    fa = (unsigned char *)&in6p.in6p_faddr;
+		    fp = (int)ntohs(in6p.in6p_fport);
+		}
+	    } else
+#endif	/* defined(HASIPv6) && defined(NETBSDV) && !defined(HASINRIAIPv6) */
 
-	    lp = (int)ntohs(inp.inp_lport);
+	    {
 
-#if	defined(HASIPv6) && defined(HASINRIAIPv6)
-	    if (inp.inp_fatype != IPATYPE_UNBD || inp.inp_fport) {
-		if (inp.inp_fatype == IPATYPE_IPV6) {
-		    fa = (unsigned char *)&inp.inp_faddr6;
-		    fam = AF_INET6;
-		    Lf->type[3] = '6';
-		} else {
-		    fa = (unsigned char *)&inp.inp_faddr;
-		    if (inp.inp_fatype == IPATYPE_IPV4) {
-			fam = AF_INET;
-			Lf->type[3] = '4';
+	    /*
+	     * Read IPv4 or IPv6 (OpenBSD) protocol control block.
+	     */
+		if (!s.so_pcb
+		||  kread((KA_T)s.so_pcb, (char *)&inp, sizeof(inp))) {
+		    (void) sprintf(Namech, "can't read inpcb at %s",
+			print_kptr((KA_T)s.so_pcb, (char *)NULL));
+		    enter_nm(Namech);
+		    return;
+		}
+		enter_dev_ch(print_kptr((KA_T)(inp.inp_ppcb ? inp.inp_ppcb
+							    : s.so_pcb),
+					       (char *)NULL));
+		if (p.pr_protocol == IPPROTO_TCP)
+		    ta = (KA_T)inp.inp_ppcb;
+		lp = (int)ntohs(inp.inp_lport);
+		if (fam == AF_INET) {
+
+		/*
+		 * Save IPv4 address information.
+		 */
+		    la = (unsigned char *)&inp.inp_laddr;
+		    if (inp.inp_faddr.s_addr != INADDR_ANY || inp.inp_fport) {
+			fa = (unsigned char *)&inp.inp_faddr;
+			fp = (int)ntohs(inp.inp_fport);
 		    }
 		}
-		fp = (int)ntohs(inp.inp_fport);
-	    }
-#else	/* !defined(HASIPv6) || !defined(HASINRIAIPv6) */
-	    if (inp.inp_faddr.s_addr != INADDR_ANY || inp.inp_fport) {
-		fa = (unsigned char *)&inp.inp_faddr;
-		fp = (int)ntohs(inp.inp_fport);
-	    }
-#endif	/* defined(HASIPv6) && defined(HASINRIAIPv6) */
 
+#if	defined(HASIPv6) && (defined(OPENBSDV) || defined(HASINRIAIPv6))
+		else {
+		    la = (unsigned char *)&inp.inp_laddr6;
+		    if (!IN6_IS_ADDR_UNSPECIFIED(&inp.inp_faddr6)
+		    ||  inp.inp_fport)
+		    {
+			fa = (unsigned char *)&inp.inp_faddr6;
+			fp = (int)ntohs(inp.inp_fport);
+		    }
+		}
+#endif	/* defined(HASIPv6) && (defined(OPENBSDV) !! defined(HASINRIAIPv6)) */
+
+	    }
+	/*
+	 * Enter local and remote addresses by address family.
+	 */
 	    if (fa || la)
 		(void) ent_inaddr(la, lp, fa, fp, fam);
-	    if (p.pr_protocol == IPPROTO_TCP && inp.inp_ppcb
-	    &&  !kread((KA_T)inp.inp_ppcb, (char *)&t, sizeof(t))) {
+	/*
+	 * If the protocol is TCP, and its address is available, read the
+	 * TCP protocol control block and save its state.
+	 */
+	    if (ta && !kread(ta, (char *)&t, sizeof(t))) {
 		Lf->lts.type = 0;
 		Lf->lts.state.i = (int)t.t_state;
 	    }
@@ -278,15 +305,15 @@ process_socket(sa)
 		if (mb.m_len >= sizeof(struct sockaddr_un))
 		    mb.m_len = sizeof(struct sockaddr_un) - 1;
 		*((char *)ua + mb.m_len) = '\0';
-		if (Sfile && is_file_named(ua->sun_path, VSOCK))
+		if (Sfile && is_file_named(ua->sun_path, 0))
 		    Lf->sf |= SELNM;
-		else
+		if (!Namech[0])
 		    (void) strcpy(Namech, ua->sun_path);
 	    } else
 		(void) strcpy(Namech, "no address");
 	    break;
 	default:
-	    printunkaf(fam);
+	    printunkaf(fam, 1);
 	}
 	if (Namech[0])
 	    enter_nm(Namech);
