@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1995 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.8 98/05/22 07:15:25 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.12 99/06/22 08:21:45 abe Exp $";
 #endif
 
 
@@ -142,9 +142,14 @@ process_node(na)
 	    return;
 	}
 
-# if    defined(HASNCACHE)
+#if	defined(HASNCACHE)
 	Lf->na = na;
-# endif /* defined(HASNCACHE) */
+#endif	/* defined(HASNCACHE) */
+
+#if	defined(HASFSTRUCT)
+	Lf->fna = na;
+	Lf->fsv |= FSV_NI;
+#endif	/* defined(HASFSTRUCT) */
 
 /*
  * Identify the node type.
@@ -593,6 +598,15 @@ process_node(na)
 		break;
 	    }
 	}
+/*
+ * Record link count.
+ */
+	if (Fnlink) {
+	    Lf->nlink = (long)i.i_nlink;
+	    Lf->nlink_def = 1;
+	    if (Nlink && (Lf->nlink < Nlink))
+		Lf->sf |= SELNLINK;
+	}
 /*
  * Format the type name.
  */
@@ -677,7 +691,7 @@ process_node(na)
 /*
  * Test for specified file.
  */
-	if (Sfile && is_file_named(NULL, type))
+	if (Sfile && is_file_named(NULL, (type == IFCHR) ? 1 : 0))
 	    Lf->sf |= SELNM;
 
 #if	OSRV>=500
