@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1995 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.9 98/06/29 15:54:19 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.13 99/11/28 06:42:23 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -70,11 +70,11 @@ _PROTOTYPE(static void readfsinfo,(void));
 
 
 /*
- * ckkv - check kernel version
+ * Ckkv - check kernel version
  */
 
 static void
-ckkv(d, er, ev, ea)
+Ckkv(d, er, ev, ea)
 	char *d;			/* dialect */
 	char *er;			/* expected release */
 	char *ev;			/* expected version */
@@ -120,15 +120,24 @@ gather_proc_info()
 	short pss, sf;
 	static struct user *u;
 	static char *ua = (char *)NULL;
+	static MALLOC_S ual = 0;
 	unsigned int uid;
+
+#if	defined(HASFSTRUCT)
+	static MALLOC_S npofb = 0; 
+	char *pof;
+	static char *pofb = (char *)NULL;
+#endif	/* defined(HASFSTRUCT) */
+
 /*
  * Allocate user structure buffer.
  */
 	if (!ua) {
-	    if ((ua = (char *)malloc(MAXUSIZE * NBPC)) == (char *)NULL) {
+	    ual = (MALLOC_S)(MAXUSIZE * NBPC);
+	    if (!(ua = (char *)malloc(ual))) {
 		(void) fprintf(stderr,
 		    "%s: no space for %d byte user structure buffer\n",
-		    Pn, MAXUSIZE * NBPC);
+		    Pn, ual);
 		Exit(1);
 	    }
 	    u = (struct user *)ua;
@@ -229,12 +238,56 @@ gather_proc_info()
 	    nf = u->u_nofiles ? u->u_nofiles : Var.v_nofiles;
 #endif	/* OSRV<42 */
 
+#if	defined(HASFSTRUCT)
+	    if (Fsv & FSV_FG) {
+
+	    /*
+	     * If u_pofile is in the u block, set its address.
+	     */
+		if (nf && u->u_pofile
+		&&  ((unsigned)u->u_pofile >= UVUBLK)
+		&&  ((MALLOC_S)((unsigned)u->u_pofile - UVUBLK + nf) <= ual))
+		{
+		     pof = ua + (unsigned)u->u_pofile - UVUBLK;
+		} else if (nf && u->u_pofile) {
+
+		/*
+		 * Allocate space for u_pofile and read it from kernel memory.
+		 */
+		    if (nf > npofb) {
+			if (!pofb)
+			    pofb = (char *)malloc((MALLOC_S)nf);
+			else
+			    pofb = (char *)realloc((MALLOC_P *)pofb,
+						   (MALLOC_S)nf);
+			if (!pofb) {
+			    (void) fprintf(stderr, "%s: no pofile space\n", Pn);
+			    Exit(1);
+			}
+			npofb = nf;
+		    }
+		    if (kread((KA_T)u->u_pofile, pofb, nf))
+			pof = (char *)NULL;
+		    else
+			pof = pofb;
+		} else
+		    pof = (char *)NULL;
+	    }
+#endif	/* defined(HASFSTRUCT) */
+
 	    for (i = 0; i < nf; i++) {
 		if (u->u_ofile[i]) {
 		    alloc_lfile((char *)NULL, i);
 		    process_file((KA_T)u->u_ofile[i]);
-		    if (Lf->sf)
+		    if (Lf->sf) {
+
+#if	defined(HASFSTRUCT)
+			if (Fsv & FSV_FG && pof)
+			    Lf->pof = (long)pof[i];
+#endif	/* defined(HASFSTRUCT) */
+
 			link_lfile();
+		    }
 		}
 	    }
 	/*
@@ -346,7 +399,7 @@ get_kernel_access()
 /*
  * Check kernel version.
  */
-	(void) ckkv("OSR", LSOF_VSTR, (char *)NULL, (char *)NULL);
+	(void) Ckkv("OSR", LSOF_VSTR, (char *)NULL, (char *)NULL);
 /*
  * See if the name list file is readable.
  */
@@ -619,6 +672,8 @@ initialize()
 	get_kernel_access();
 	get_cdevsw();
 	readfsinfo();
+	if (Fsv & FSV_NI)
+	    NiTtl = "INODE-ADDR";
 }
 
 
