@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1995 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.8 98/05/22 19:56:46 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.10 99/11/28 06:42:25 abe Exp $";
 #endif
 
 
@@ -291,7 +291,7 @@ process_socket(i)
 			= '\0';
 		    if (Sfile && is_file_named(ud.local_addr.sun_path, 0))
 			Lf->sf |= SELNM;
-		    else
+		    if (!Namech[0])
 			(void) strcpy(Namech, ud.local_addr.sun_path);
 		} else if (ud.for_addr.sun_family == AF_UNIX) {
 		    Lf->inode = (unsigned long)ud.bnd_param.user_addr.inode_no;
@@ -311,7 +311,7 @@ process_socket(i)
 #endif	/* OSRV>=500 */
 
 	default:
-	    printunkaf(fam);
+	    printunkaf(fam, 1);
 	}
 	enter_nm(Namech);
 }
