@@ -31,17 +31,27 @@
 
 
 /*
- * $Id: machine.h,v 1.15 98/06/29 15:41:31 abe Exp $
+ * $Id: machine.h,v 1.20 99/07/08 14:50:51 abe Exp $
  */
 
-#if	!defined(MACHINE_H)
-#define	MACHINE_H	1
+#if	!defined(LSOF_MACHINE_H)
+#define	LSOF_MACHINE_H	1
 
 
 #include <sys/types.h>
 #include <sys/param.h>
 
 
+/*
+ * CAN_USE_CLNT_CREATE is defined for those dialects where RPC clnt_create()
+ * can be used to obtain a CLIENT handle in lieu of clnttcp_create().
+ */
+
+#if	OSRV>=42
+#define	CAN_USE_CLNT_CREATE	1
+#endif	/* OSRV>=42 */
+
+
 /*
  * GET_MAX_FD is defined for those dialects that provide a function other than
  * getdtablesize() to obtain the maximum file descriptor number plus one.
@@ -96,7 +106,7 @@
 #define	HASENVDC	"LSOFDEVCACHE"
 #define	HASPERSDC	"%h/%p.lsof_%L"
 #define	HASPERSDCPATH	"LSOFPERSDCPATH"
-/* #define	HASSYSDC	"/var/adm/lsof.dc" */
+/* #define	HASSYSDC	"/your/choice/of/path" */
 
 
 /*
@@ -121,6 +131,18 @@
 #define	HASFSINO	1
 
 
+/*
+ * HASFSTRUCT is defined if the dialect has a file structure.
+ *
+ * FSV_DEFAULT defines the default set of file structure values to list.
+ * It defaults to zero (0), but may be made up of a combination of the
+ * FSV_* symbols from lsof.h.
+ */
+
+#define	HASFSTRUCT	1
+/* #define	FSV_DEFAULT	FSV_? | FSV_? | FSV_? */
+
+
 /*
  * HASGNODE is defined for those dialects that have gnodes.
  */
@@ -174,21 +196,10 @@
  * alloc_lfile() function of proc.c, the SETLFILEADD macro may be defined
  * to do that.  The HASXOPT definition may be used to select the conditions
  * under which private lfile elements are used.
- *
- * Field identifiers 1 through 9 are reserved for dialect-specific field
- * output.  Define the string HASFIELDAP<x> as the help (-F?) name of the
- * private field and LISTLFIELAP<x> as a macro to list the field output.
- * (<x> is the field identifier character.)  HASFIELDAP<x> is used in proc.c
- * and store.c; LISTFIELDAP<x>, in proc.c.
- *
- * See ../dialectx/ptx for an example of the use of HASLFILEADD, SETLFILEADD,
- * and HASXOPT.
  */
 
-/* #define HASLFILEADD <macro of new element definitions>; */
-/* #define SETLFILEADD <macro of code to preset new elements>; */
-/* #define HASFIELDAP1 <string name of field for -F? output> */
-/* #define LISTLFILEAP1 <macro to list field output>; */
+/* #define HASLFILEADD int ... */
+/* #define SETLFILEADD Lf->... */
 
 
 /*
@@ -227,7 +238,7 @@
  * NOTE: don't forget to define a prototype for this function in dproto.h.
  */
 
-/* #define	HASPIPEFN	process_pipe */
+/* #define	HASPIPEFN	process_pipe? */
 
 
 /*
@@ -262,9 +273,9 @@
 
 #define	HASPRINTDEV	print_dev
 #define	HASPRINTINO	print_ino
-/* #define	HASPRINTNM	print_nm	*/
-/* #define	HASPRINTOFF	print_off	*/
-/* #define	HASPRINTSZ	print_sz	*/
+/* #define	HASPRINTNM	print_nm?	*/
+/* #define	HASPRINTOFF	print_off?	*/
+/* #define	HASPRINTSZ	print_sz?	*/
 
 
 /*
@@ -277,7 +288,7 @@
  * HASPRIVFILETYPE is the name of the processing function.
  */
 
-/* #define	HASPRIVFILETYPE	process_shmf	*/
+/* #define	HASPRIVFILETYPE	process_shmf?	*/
 /* #define	PRIVFILETYPE	??	*/
 
 
@@ -311,7 +322,7 @@
  * by inode number.
  */
 
-/* #define	HASPROCFS	"proc" */
+/* #define	HASPROCFS	"proc?" */
 /* #define	HASFSTYPE	1 */
 /* #define	HASPINODEN	1 */
 
@@ -411,12 +422,12 @@
 /* #define	USE_LIB_CKKV			1	   ckkv.c */
 /* #define	USE_LIB_COMPLETEVFS		1	   cvfs.c */
 #define	USE_LIB_FIND_CH_INO			1	/* fino.c */
-/* #define	USE_LIB_IS_FILE_NAMED		1	   isfn.c */
+#define	USE_LIB_IS_FILE_NAMED			1	/* isfn.c */
 #define	USE_LIB_LKUPDEV				1	/* lkud.c */
-/* #define	USE_LIB_PRINTCHDEVNAME		1	   pdvn.c */
+#define	USE_LIB_PRINTCHDEVNAME			1	/* pdvn.c */
 /* #define	USE_LIB_PROCESS_FILE		1	   prfp.c */
 #define	USE_LIB_PRINT_TCPTPI			1	/* ptti.c */
-/* #define	USE_LIB_READDEV			1	   rdev.c */
+#define	USE_LIB_READDEV				1	/* rdev.c */
 /* #define	USE_LIB_READMNT			1	   rmnt.c */
 /* #define	USE_LIB_RNAM			1	   rnam.c */
 /* #define	USE_LIB_RNCH			1	   rnch.c */
@@ -454,4 +465,4 @@
  */
 
 #define	zeromem(a, l)	memset(a, 0, l)
-#endif	/* !defined(MACHINE_H) */
+#endif	/* !defined(LSOF_MACHINE_H) */
