@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1995 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: ddev.c,v 1.3 98/03/06 08:27:21 abe Exp $";
+static char *rcsid = "$Id: ddev.c,v 1.4 99/06/22 08:23:35 abe Exp $";
 #endif
 
 
@@ -49,11 +49,11 @@ findclones()
 	struct clone *c;
 	int i;
 
-	if ( ! HaveCloneMajor)
+	if (!HaveCloneMaj)
 		return;
 	readdev(0);
 	for (i = 0; i < Ndev; i++) {
-	    if (major(Devtp[i].rdev) == CloneMajor) {
+	    if (major(Devtp[i].rdev) == CloneMaj) {
 		if (!(c = (struct clone *)malloc(sizeof(struct clone)))) {
 		    (void) fprintf(stderr,
 			"%s: no space for clone device: %s\n",
