@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1995 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dfile.c,v 1.3 98/03/06 08:27:28 abe Exp $";
+static char *rcsid = "$Id: dfile.c,v 1.6 99/06/22 08:23:46 abe Exp $";
 #endif
 
 
@@ -61,31 +61,6 @@ print_dev(lf)
 }
 
 
-/*
- * print_nm() - print name
- */
-
-void
-print_nm(lf)
-	struct lfile *lf;		/* file whose name is to be printed */
-{
-
-#if	defined(HASLFILEADD)
-	if (Fxopt && (lf->fct_def || lf->ina_def)) {
-	    printname(0);
-	    if (lf->fct_def)
-		(void) printf(" ct=%ld", lf->fct);
-	    if (lf->ina_def)
-		(void) printf(" ina=%s\n", print_kptr(lf->ina, (char *)NULL));
-	    else
-		putchar('\n');
-	} else
-#endif	/* defined(HASLFILEADD) */
-
-	    printname(1);
-}
-
-
 /*
  * process_file() - process file
  */
@@ -105,11 +80,6 @@ process_file(fp)
 	}
 	Lf->off = (SZOFFTYPE)f.f_offset;
 
-#if	defined(HASLFILEADD)
-	Lf->fct = f.f_count;
-	Lf->fct_def = 1;
-#endif	/* defined(HASLFILEADD) */
-
 	if (f.f_count) {
 
 	/*
@@ -124,6 +94,29 @@ process_file(fp)
 	/*
 	 * Process structure.
 	 */
+
+#if	defined(HASFSTRUCT)
+	/*
+	 * Save file structure values.
+	 */
+	    if (Fsv & FSV_CT) {
+		Lf->fct = (long)f.f_count;
+		Lf->fsv |= FSV_CT;
+	    }
+	    if (Fsv & FSV_FA) {
+		Lf->fsa = fp;
+		Lf->fsv |= FSV_FA;
+	    }
+	    if (Fsv & FSV_FG) {
+		Lf->ffg = (long)f.f_flag;
+		Lf->fsv |= FSV_FG;
+	    }
+	    if ((Fsv & FSV_NI) && !Fxopt) {
+		Lf->fna = (KA_T)f.f_vnode;
+		Lf->fsv |= FSV_NI;
+	    }
+#endif	/* defined(HASFSTRUCT) */
+
 	    process_node((KA_T)f.f_vnode);
 	    return;
 	}
