@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1995 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.8 98/06/13 09:46:43 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.12 99/06/22 08:24:12 abe Exp $";
 #endif
 
 
@@ -57,6 +57,10 @@ _PROTOTYPE(static struct l_dev * finddev,(dev_t *dev, int stream));
 _PROTOTYPE(static int isip,(char *mn));
 _PROTOTYPE(static char isvlocked,(struct vnode *va));
 
+#if	TCPV>=450
+_PROTOTYPE(static int read_vl_un,(KA_T addr, int len));
+#endif	/* TCPV>=450 */
+
 
 struct iptab_proto {
 	char *mn;			/* module name */
@@ -251,8 +255,8 @@ process_node(va)
 	struct queue q;
 	struct qinit qi;
 	static KA_T sckvops = (KA_T)0;
-	static KA_T spvops = (KA_T)0;
 	struct so_so so;
+	static KA_T spvops = (KA_T)0;
 	struct sockaddr_un ua;
 	static KA_T uvops = (KA_T)0;
 	static KA_T vvops = (KA_T)0;
@@ -281,19 +285,16 @@ process_node(va)
 	int nmst = 1;
 #endif	/* PTXV>=400 */
 
-#if	PTXV>=413
+#if	TCPV>=420
+	struct datab db;
 	struct domain dm;
+	struct msgb mb;
 	struct socket_peer peer;
 	struct protosw pr;
 	struct socket sck;
-#endif	/* PTXV>=413 */
-
-#if	PTXV>=421
-	struct datab db;
-	struct msgb mb;
 	struct unpcb ub;
 	struct sockaddr_un un;
-#endif	/* PTXV>=421 */
+#endif	/* TCPV>=420 */
 
 #if	defined(HASVXFS)
 	int svx = 0;
@@ -336,9 +337,16 @@ read_vnode:
 	   return;
 	}
 
-# if	defined(HASNCACHE)
+#if	defined(HASNCACHE)
 	Lf->na = va;
-# endif	/* defined(HASNCACHE) */
+#endif	/* defined(HASNCACHE) */
+
+#if	defined(HASFSTRUCT)
+	if (!Fxopt) {
+	    Lf->fna = va;
+	    Lf->fsv |= FSV_NI;
+	}
+#endif	/* defined(HASFSTRUCT) */
 
 /*
  * Determine the vnode type.
@@ -353,10 +361,10 @@ read_vnode:
 	    Ntype = N_NFS;
 #endif	/* defined(HAS_NFS) */
 
-#if	PTXV>=413
+#if	TCPV>=420
 	else if (sckvops && sckvops == (KA_T)v.v_op)
 	    Ntype = N_SOCK;
-#endif	/* PTXV>=413 */
+#endif	/* TCPV>=420 */
 
 	else if (spvops && spvops == (KA_T)v.v_op)
 	    Ntype = N_SPEC;
@@ -480,6 +488,10 @@ read_vnode:
 	    Lf->na = (KA_T)v.v_data;
 # endif	/* defined(HASNCACHE) && defined(HASVXFSDNLC) */
 
+#if	defined(HASFSTRUCT)
+	    Lf->fna = (KA_T)v.v_data;
+#endif	/* defined(HASFSTRUCT) */
+
 	    break;
 #endif	/* defined(HASVXFS) */
 
@@ -554,7 +566,7 @@ read_vnode:
 	    break;
 #endif	/* PTXV>=400 */
 
-#if	PTXV>=413
+#if	TCPV>=420
 	case N_SOCK:
 	    if (!v.v_snode && !v.v_stream) {
 
@@ -572,8 +584,6 @@ read_vnode:
 		if (!pr.pr_domain
 		||  kread((KA_T)pr.pr_domain, (char *)&dm, sizeof(dm)))
 		    break;
-
-#if	PTXV>=421
 		if (dm.dom_family == AF_UNIX) {
 
 		/*
@@ -586,6 +596,7 @@ read_vnode:
 		/*
 		 * Read sockaddr_un.
 		 */
+		    Namech[0] = '\0';
 		    if (!kread((KA_T)ub.unp_addr, (char *)&mb, sizeof(mb))
 		    &&  mb.b_datap
 		    &&  !kread((KA_T)mb.b_datap, (char *)&db, sizeof(db))
@@ -595,47 +606,59 @@ read_vnode:
 		    &&  un.sun_path)
 		    {
 
+# if	TCPV>=450
+
+		    /*
+		     * Enter path from variable length sockaddr_un.
+		     */
+			(void) read_vl_un((KA_T)db.db_base, (int)un.sun_len);
+# else	/* TCPV<450 */
+
 		    /*
 		     * Enter path from sockaddr_un.
 		     */
 			un.sun_path[sizeof(un.sun_path) - 1] - '\0';
-			if ((l = strlen(un.sun_path)) >= (MAXPATHLEN - 1))
-			    l = MAXPATHLEN - 1;
+			if ((l = strlen(un.sun_path)) >= (MAXPATHLEN - 2))
+			    l = MAXPATHLEN - 2;
 			(void) strncpy(Namech, un.sun_path, l);
 			Namech[l] = '\0';
+# endif	/* TCPV>=450 */
+
 		    }
+
 		    if (ub.unp_vnode) {
 			
-		     /*
-		      * If the socket is associated with a vnode, try to
-		      * interpret it instead, possibly using only the name
-		      * from the sockaddr_un of this VSOCK vnode.
-		      */
+		    /*
+		     * If the socket is associated with a vnode, try to
+		     * interpret it instead, possibly using only the name
+		     * from the sockaddr_un of this VSOCK vnode.
+		     */
 			if (Funix)
 			    Lf->sf |= SELUNX;
 			usst = 1;
 			va = (KA_T)ub.unp_vnode;
 			goto read_vnode;
-		     }
+		    } else {
+			if (Namech[0] && is_file_named(Namech, 0))
+			    Lf->sf |= SELNM;
+		    }
 		    (void) process_unix_socket(&sck, &ub);
 		    return;
 		}
-# endif	/* PTXV>=421 */
-
 		if (dm.dom_family != AF_INET)
 		    break;
 		switch (pr.pr_protocol) {
 		case IPPROTO_ICMP:
 		    (void) process_socket("ICMP", AF_INET, IPPROTO_ICMP,
-			(KA_T)sck.so_pcb);
+			(KA_T)sck.so_pcb, (void *)&sck);
 		    return;
 		case IPPROTO_TCP:
 		    (void) process_socket("TCP", AF_INET, IPPROTO_TCP,
-			(KA_T)sck.so_pcb);
+			(KA_T)sck.so_pcb, (void *)&sck);
 		    return;
 		case IPPROTO_UDP:
 		    (void) process_socket("UDP", AF_INET, IPPROTO_UDP,
-			(KA_T)sck.so_pcb);
+			(KA_T)sck.so_pcb, (void *)&sck);
 		    return;
 		default:
 		    (void) printiproto(pr.pr_protocol);
@@ -645,7 +668,7 @@ read_vnode:
 	    }
 	    /* Change node type to N_SPEC and fall through. */
 	    Ntype = N_SPEC;
-#endif	/* PTXV>=413 */
+#endif	/* TCPV>=420 */
 
 	case N_SPEC:
 	    p = (KA_T)NULL;
@@ -681,15 +704,16 @@ read_vnode:
 		}
 		srv = 1;
 		if (rv.v_data) {
-		    if (!kread((KA_T)v.v_data, (char *)&i, sizeof(i))) {
+		    if (!kread((KA_T)rv.v_data, (char *)&i, sizeof(i))) {
 			sin = 1;
 
-#if	defined(HASLFILEADD)
-			Lf->ina = (KA_T)rv.v_data;
-			Lf->ina_def = 1;
-#endif	/* defined(HASLFILEADD) */
+#if	defined(HASFSTRUCT)
+			if (Fxopt) {
+			    Lf->fna = (KA_T)rv.v_data;
+			    Lf->fsv |= FSV_NI;
+			}
+#endif	/* defined(HASFSTRUCT) */
 
-		
 		    }
 		}
 	    }
@@ -748,32 +772,49 @@ read_vnode:
 			continue;
 		    if (!q.q_next && q.q_ptr && (px = isip(tbuf)) >= 0
 
-#if	PTXV>=413
+#if	TCPV>=420
 		    &&  !kread((KA_T)q.q_ptr, (char *)&peer, sizeof(peer))
+# if	TCPV>=450
+		    &&  peer.sop_socket
+		    &&  !kread((KA_T)peer.sop_socket, (char *)&sck, sizeof(sck))
+		    &&  sck.so_pcb
+# else	/* TCPV<450 */
 		    &&  peer.sop_inpcb
-#endif	/* PTXV>=413 */
+# endif	/* TCPV>=450 */
+#endif	/* TCPV>=420 */
 
 		    ) {
 			process_socket(IPtab[px].nm, IPtab[px].fam,
 				       IPtab[px].ipid,
 
-#if	PTXV>=413
-				       (KA_T)peer.sop_inpcb
-#else	/* PTXV<413 */
-
-				       (KA_T)q.q_ptr
-#endif	/* PTXV>=413 */
+#if	TCPV>=420
+# if	TCPV>=450
+				       (KA_T)sck.so_pcb,
+# else	/* TCPV<450 */
+				       (KA_T)peer.sop_inpcb,
+# endif	/* TCPV>=450 */
+#else	/* TCPV<420 */
+				       (KA_T)q.q_ptr,
+#endif	/* TCPV>=420 */
 
+				       (void *)NULL
 			);
 			    return;
 		    }
 		    if (cp && strcmp(cp, tbuf) == 0) {
 			if (q.q_ptr && pt < 0 && (px = isip(cp)) >= 0
 
-#if	PTXV>=413
-			 && !kread((KA_T)q.q_ptr, (char *)&peer, sizeof(peer))
+#if	TCPV>=420
+			&&  !kread((KA_T)q.q_ptr, (char *)&peer, sizeof(peer))
+# if	TCPV>=450
+			&&  peer.sop_socket
+			&&  !kread((KA_T)peer.sop_socket, (char *)&sck,
+				   sizeof(sck))
+			&&  sck.so_pcb
+# else	/* TCPV<450 */
 			 &&  peer.sop_inpcb
-#endif	/* PTXV>=413 */
+# endif	/* TCPV>=450 */
+#endif	/* TCPV>=420 */
 
 			) {
 
@@ -791,11 +832,15 @@ read_vnode:
 			    }
 			    if (pt >= 0) {
 
-#if	PTXV>=413
+#if	TCPV>=420
+# if	TCPV>=450
+				p = (KA_T)sck.so_pcb;
+# else	/* TCPV<450 */
 				p = (KA_T)peer.sop_inpcb;
-#else	/* PTXV<413 */
+# endif	/* TCPV>=450 */
+#else	/* TCPV<420 */
 				p = (KA_T)q.q_ptr;
-#endif	/* PTXV>=413 */
+#endif	/* TCPV>=420 */
 
 				(void) strcpy(Lf->iproto, IPtab[px].nm);
 			    }
@@ -878,13 +923,15 @@ read_vnode:
 		    return;
 		}
 		sin = 1;
+
+#if	defined(HASFSTRUCT)
+		if (Fxopt) {
+		    Lf->fna = (KA_T)v.v_data;
+		    Lf->fsv |= FSV_NI;
+		}
+#endif	/* defined(HASFSTRUCT) */
+
 	    }
-
-#if	defined(HASLFILEADD)
-	    Lf->ina = (KA_T)v.v_data;
-	    Lf->ina_def = 1;
-#endif	/* defined(HASLFILEADD) */
-
 	}
 	if (Selinet)
 	    return;
@@ -945,7 +992,7 @@ read_vnode:
 	    break;
 #endif	/* defined(HASVXFS) */
 
-#if	PTXV>=413
+#if	TCPV>=420
 	case N_SOCK:
 	    if (v.v_rdev == NODEV)
 		enter_dev_ch("NODEV");
@@ -954,7 +1001,7 @@ read_vnode:
 		Lf->dev_def = 1;
 	    }
 	    break;
-#endif	/* PTXV>=413 */
+#endif	/* TCPV>=420 */
 
 	case N_SPEC:
 	    if (sop)
@@ -996,7 +1043,6 @@ read_vnode:
 	    break;
 #endif	/* defined(HASVXFS) */
 
-
 	case N_FIFO:
 	    if (sin) {
 		Lf->inode = (unsigned long)i.i_number;
@@ -1038,7 +1084,7 @@ read_vnode:
 	     }
 	     break;
 
-#if	PTXV>=413
+#if	TCPV>=420
 	case N_SOCK:
 	    /* The sock_vnodeops type vnode has no inode number. */
 	    if (!Lf->iproto[0]) {
@@ -1046,7 +1092,7 @@ read_vnode:
 		Lf->inp_ty = 2;
 	    }
 	    break;
-#endif	/* PTXV>=413 */
+#endif	/* TCPV>=420 */
 
 	case N_SPEC:
 	    if (sop) {
@@ -1132,11 +1178,11 @@ read_vnode:
 		    Lf->off_def = 1;
 		break;
 
-#if	PTXV>=413
+#if	TCPV>=420
 	     case N_SOCK:
 		Lf->off_def = 1;
 		break;
-#endif	/* PTXV>=413 */
+#endif	/* TCPV>=420 */
 
 	     case N_SPEC:
 		if (type == VCHR && !Fsize)
@@ -1144,6 +1190,64 @@ read_vnode:
 		break;
 	    }
 	}
+/*
+ * Record link count.
+ */
+	if (Fnlink) {
+	    switch(Ntype) {
+
+#if	defined(HAS_CDFS)
+	    case N_CDFS:
+		/* Lf->nlink and Lf->nlink_def are set in read_cdnode() */
+	    break;
+#endif	/* defined(HAS_CDFS) */
+
+#if	defined(HASVXFS)
+	    case N_VXFS:
+		/* Lf->nlink and Lf->nlink_def are set in read_vxnode() */
+	    break;
+#endif	/* defined(HASVXFS) */
+
+	    case N_FIFO:
+		if (sin) {
+		    Lf->nlink = (long)i.i_nlink;
+		    Lf->nlink_def = 1;
+		}
+
+#if	defined(HAS_NFS)
+		else if (srn) {
+		    Lf->nlink = (long)r.r_nfsattr.na_nlink;
+		    Lf->nlink_def = 1;
+		}
+#endif	/* defined(HAS_NFS) */
+
+		break;
+
+#if	defined(HAS_NFS)
+	    case N_NFS:
+		Lf->nlink = (long)r.r_nfsattr.na_nlink;
+		Lf->nlink_def = 1;
+		break;
+#endif	/* defined(HAS_NFS) */
+
+#if	PTXV>=400
+	    case N_NM:
+		Lf->nlink = (long)n.nm_vattr.va_nlink;
+		Lf->nlink_def = 1;
+		break;
+#endif	/* PTXV>=400 */
+
+	    case N_SPEC:
+	    case N_REGLR:
+		if (sin) {
+		    Lf->nlink = (long)i.i_nlink;
+		    Lf->nlink_def = 1;
+		}
+		break;
+	    }
+	    if (Nlink && Lf->nlink_def && (Lf->nlink < Nlink))
+		Lf->sf |= SELNLINK;
+	}
 
 #if	defined(HAS_NFS)
 /*
@@ -1251,19 +1355,34 @@ read_vnode:
 		Lf->sf |= SELUNX;
 	    (void) strcpy(Lf->type, "unix");
 	    if ((Namech[0] == '\0' || !strncmp(Namech, "STR:", strlen("STR:")))
-	    &&  so.laddr.buf
-	    &&  so.laddr.len == sizeof(ua))
+	    &&  so.laddr.buf)
 	    {
-		if (kread((KA_T)so.laddr.buf, (char *)&ua, sizeof(ua)) == 0) {
+
+#if	TCPV>=450
+		if (!kread((KA_T)so.laddr.buf, (char *)&ua, sizeof(ua))
+		&&  ua.sun_family == AF_UNIX
+		&&  read_vl_un((KA_T)so.laddr.buf, (int)ua.sun_len))
+		{
+		    if (is_file_named(Namech, 0))
+			Lf->sf |= SELNM;
+		}
+#else	/* TCPV<450 */
+		if (so.laddr.len = sizeof(ua)
+		&& !kread((KA_T)so.laddr.buf, (char *)&ua, sizeof(ua)))
+		{
 		    ua.sun_path[sizeof(ua.sun_path) - 1] = '\0';
 		    (void) strcpy(Namech, ua.sun_path);
+		    if (is_file_named(Namech, 0))
+			Lf->sf |= SELNM;
 		}
+#endif	/* TCPV>=450 */
+
 	    }
 	}
 /*
  * Test for specified file.
  */
-	if (Sfile && is_file_named(NULL, type))
+	if (Sfile && is_file_named(NULL, (type == VCHR) ? 1 : 0))
 	    Lf->sf |= SELNM;
 /*
  * Enter name characters.
@@ -1271,3 +1390,52 @@ read_vnode:
 	if (Namech[0])
 	    enter_nm(Namech);
 }
+
+
+#if	TCPV>=450
+/*
+ * read_vl_un() - read variable length Unix socket address name for TCP
+ *		  versions 4.5 and above
+ */
+
+static int
+read_vl_un(addr, len)
+	KA_T addr;			/* kernel address */
+	int len;			/* length */
+{
+	int nml;
+	static struct sockaddr_un *vu = (struct sockaddr_un *)NULL;
+	static MALLOC_S vul = (MALLOC_S)0;
+
+	nml = len - offsetof(struct sockaddr_un, sun_path);
+	if (nml <= 0)
+	    return(0);
+/*
+ * Allocate space.
+ */
+	if (len > (int)vul) {
+	    vul = (MALLOC_S)len;
+	    if (vu)
+		vu = (struct sockaddr_un *)realloc((MALLOC_P *)vu, vul);
+	    else
+		vu = (struct sockaddr_un *)malloc(vul);
+	    if (!vu) {
+		(void) fprintf(stderr, "%s: no space (%d) for sockaddr_un\n",
+		    Pn, len);
+		Exit(1);
+	    }
+	}
+/*
+ * Enter path from variable length sockaddr_un.
+ */
+	if (!kread(addr, (char *)vu, len)) {
+	    nml = len - offsetof(struct sockaddr_un, sun_path);
+	    if (nml > (MAXPATHLEN - 2))
+		nml = MAXPATHLEN - 2;
+	    (void) strncpy(Namech, vu->sun_path, nml);
+	    Namech[nml - 1] = '\0';
+	    return(1);
+	}
+	return(0);
+}
+#endif	/* TCPV>=450 */
