@@ -35,7 +35,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1996 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode1.c,v 1.3 98/03/06 08:27:36 abe Exp $";
+static char *rcsid = "$Id: dnode1.c,v 1.4 99/05/04 09:02:41 abe Exp $";
 #endif
 
 
@@ -77,6 +77,17 @@ read_cdnode(v, vfs, dev)
  */
 	Lf->inode = (unsigned long)i.iso_extent;
 	Lf->inp_ty = 1;
+
+#if	defined(i_nlink)
+/*
+ * Record link count.
+ */
+	if (Fnlink) {
+	    Lf->nlink = (long)i.i_nlink;
+	    Lf->nlink_def = 1;
+	}
+#endif	/* defined(i_nlink) */
+
 /*
  * Record size.
  */
