@@ -35,7 +35,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1996 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode2.c,v 1.4 98/03/06 08:27:38 abe Exp $";
+static char *rcsid = "$Id: dnode2.c,v 1.5 99/05/04 09:02:43 abe Exp $";
 #endif
 
 
@@ -75,6 +75,13 @@ read_vxinode(v, vfs, dev)
  */
 	Lf->inode = (unsigned long)i.i_number;
 	Lf->inp_ty = 1;
+/*
+ * Record link count.
+ */
+	if (Fnlink) {
+	    Lf->nlink = (long)i.i_nlink;
+	    Lf->nlink_def = 1;
+	}
 /*
  * Record size.
  */
