@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1995 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.7 98/06/26 14:38:20 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.11 99/06/22 08:24:28 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -82,7 +82,6 @@ gather_proc_info()
 	struct proc ps;
 	short pss, sf;
 	int px;
-	off_t sp;
 	uid_t uid;
 	KA_T va;
 
@@ -303,8 +302,15 @@ no_ofx_space:
 		if (ofp->of_file) {
 		    alloc_lfile(NULL, i);
 		    process_file((KA_T)ofp->of_file);
-		    if (Lf->sf)
+		    if (Lf->sf) {
+
+# if	defined(HASFSTRUCT)
+			if (Fsv & FSV_FG)
+			    Lf->pof = (long)ofp->of_flags;
+# endif	/* defined(HASFSTRUCT) */
+
 			link_lfile();
+		    }
 		}
 	    }
 	/*
@@ -364,7 +370,14 @@ get_kernel_access()
 /*
  * Open kernel memory access.
  */
-	if ((Kd = open(Memory ? Memory : KMEM, O_RDONLY, 0)) < 0) {
+
+#if	PTXV>=444
+	if ((Kd = open64(Memory ? Memory : KMEM, O_RDONLY, 0)) < 0)
+#else	/* PTXV<444 */
+	if ((Kd = open(Memory ? Memory : KMEM, O_RDONLY, 0)) < 0)
+#endif	/* PTXV>=444 */
+
+	{
 	    (void) fprintf(stderr, "%s: can't open %s: %s\n", Pn,
 		Memory ? Memory : KMEM, strerror(errno));
 	    Exit(1);
@@ -427,11 +440,11 @@ get_kernel_access()
  */
 	v = (KA_T)0;
 	if (get_Nl_value("clmaj", Drive_Nl, &v) >= 0 && v
-	&&  !kread(v, (char *)&CloneMajor, sizeof(CloneMajor))) {
-	    HaveCloneMajor = 1;
+	&&  !kread(v, (char *)&CloneMaj, sizeof(CloneMaj))) {
+	    HaveCloneMaj = 1;
 	    findclones();
 	} else
-	    HaveCloneMajor = 0;
+	    HaveCloneMaj = 0;
 
 #if	PTXV>=400
 /*
@@ -470,6 +483,10 @@ void
 initialize()
 {
 	get_kernel_access();
+	if (Fxopt) {
+	   Fsv |= (FSV_CT | FSV_NI);
+	   NiTtl = "INODE-ADDR";
+	}
 }
 
 
@@ -485,7 +502,12 @@ kread(addr, buf, len)
 {
 	int br;
 
-	if (lseek(Kd, (long)addr, SEEK_SET) == (off_t)-1L)
+#if	PTXV>=444
+	if (lseek64(Kd, (off64_t)addr, SEEK_SET) == (off64_t)-1LL)
+#else	/* PTXV<444 */
+	if (lseek(Kd, (off_t)addr, SEEK_SET) == (off_t)-1L)
+#endif	/* PTXV>=444 */
+
 	    return(-1);
 	br = read(Kd, buf, len);
 	return((br == len) ? 0 : 1);
@@ -641,7 +663,8 @@ struct lnch {				/* local name cache structure */
 
 
 /*
- * Local name cache (LNC) definitions, macroes, and static values
+ * Local name cache (LNC) definitions, name cache structure element definitions
+ * (NCACHE),  macroes, and static values
  */
 
 static int LNC_asz = 0;			/* local cache allocated size */
@@ -653,6 +676,22 @@ static int LNC_csz = 0;			/* local cache current size */
 
 #define LNC_HASH(np)	((((int)(np)>>2)*31415)&(LNCHHLEN-1))
 
+# if	!defined(NCACHE_DP)
+#define	NCACHE_DP	dp
+# endif	/* !defined(NCACHE_DP) */
+
+# if	!defined(NCACHE_NAME)
+#define	NCACHE_NAME	name
+# endif	/* !defined(NCACHE_NAME) */
+
+# if	!defined(NCACHE_NAMLEN)
+#define	NCACHE_NAMLEN	namlen
+# endif	/* !defined(NCACHE_NAMLEN) */
+
+# if	!defined(NCACHE_VP)
+#define	NCACHE_VP	vp
+# endif	/* !defined(NCACHE_VP) */
+
 static struct lnch **LNC_hh = (struct lnch **)NULL;
 					/* local hash head pointers */
 static struct lnch *LNC_nc = (struct lnch *)NULL;
