@@ -105,9 +105,9 @@ typedef	unsigned long	_ulongfs_t;
 #include <netinet/tcp_timer.h>
 #include <netinet/tcp_var.h>
 
-# if	defined(RELIANTV)
+# if	defined(RELIANTV) && defined(HASMIPCH)
 #include <sys/mipc.h>
-# endif	/* defined(RELIANTV) */
+# endif	/* defined(RELIANTV) && defined(HASMIPCH) */
 
 #include <sys/protosw.h>
 #include <sys/socketvar.h>
@@ -141,6 +141,7 @@ typedef	unsigned long	_ulongfs_t;
 # endif	/* defined(RELIANTV) && RELIANTV>=54400 */
 
 #define DEVINCR		1024	/* device table malloc() increment */
+#define	DIRTYPE		dirent
 #define	FSNAMEL		4
 typedef	off_t		KA_T;
 #define	KMEM		"/dev/kmem"
@@ -169,20 +170,22 @@ typedef	off_t		KA_T;
  * Global storage definitions (including their structure definitions)
  */
 
-struct clone {
-	int dx;			/* index of device entry in Devtp[] */
-	struct clone *next;	/* forward link */
+struct bsdsnode {		/* should be in a Pyramid header file -- e.g.,
+				 * <fs/bsdsfs/*.h> */
+	struct vnode bn_vnode;	/* the original vnode */
+	struct vnode *bn_realvp;
+				/* pointer to the real vnode */
 };
-extern struct clone *Clone;
+
 extern int CloneMaj;
 extern char **Fsinfo;
 extern int Fsinfomax;
 extern int HaveCloneMaj;
 
-# if	defined(RELIANTV)
+# if	defined(RELIANTV) && defined(HASMIPCH)
 extern int HaveMipcMaj;
 extern int MipcMaj;
-# endif	/* defined(RELIANTV) */
+# endif	/* defined(RELIANTV) && defined(HASMIPCH) */
 
 extern int Kd;
 
@@ -190,7 +193,9 @@ struct l_ino {
 	dev_t dev;			/* device */
 	dev_t rdev;			/* real device */
 	ino_t number;			/* inode number */
+	int nlink_def;			/* nlink definition status */
 	long size;			/* file size */
+	long nlink;			/* link count */
 };
 
 struct mounts {
@@ -238,17 +243,13 @@ struct sfile {
 
 #define	CKFA_XDEVTST strcmp(sb.st_fstype,"cdfs")==0
 
-
 /*
- * Definitions for dvch.c
+ * Definitions for dvch.c, isfn.c, and rdev.c
  */
 
-# if	defined(HASDCACHE)
-#define	DCACHE_CLONE	rw_clone_sect
-#define	DCACHE_CLR	clr_sect	/* function to clear clone cache
-					 * when reading the device cache
-					 * file fails */
-# endif	/* defined(HASDCACHE) */
+#define	CLONEMAJ	CloneMaj	/* clone major variable name */
+#define	HAS_STD_CLONE	1		/* has standard clone structure */
+#define	HAVECLONEMAJ	HaveCloneMaj	/* clone major status variable name */
 
 /*
  * Definitions for rnch.c
