@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1996 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.13 98/09/14 13:39:50 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.19 99/06/22 08:10:48 abe Exp $";
 #endif
 
 
@@ -246,6 +246,7 @@ void
 process_node(na)
 	KA_T na;			/* vnode kernel space address */
 {
+	struct bsdsnode bn;
 	char *cp, tbuf[32];
 	dev_t dev;
 	struct l_dev *dp;
@@ -280,6 +281,10 @@ process_node(na)
 	enum vtype type;
 	struct sockaddr_un ua;
 
+#if	defined(HASOBJFS)
+	struct objnode ob;
+#endif	/* defined(HASOBJFS) */
+
 #if	defined(HASPROCFS)
 	struct as as;
 	struct proc p;
@@ -288,9 +293,9 @@ process_node(na)
 	struct prnode pr;
 #endif	/* defined(HASPROCFS) */
 
-#if	defined(HASOBJFS)
-	struct objnode ob;
-#endif	/* defined(HASOBJFS) */
+#if	defined(HASSOCKFS)
+	struct socknode sn;
+#endif	/* defined(HASSOCKFS) */
 
 
 /*
@@ -313,9 +318,14 @@ process_node(na)
             return;
         }
 
-# if    defined(HASNCACHE)
+#if    defined(HASNCACHE)
 	Lf->na = na;
-# endif /* defined(HASNCACHE) */
+#endif /* defined(HASNCACHE) */
+
+#if	defined(HASFSTRUCT)
+	Lf->fna = na;
+	Lf->fsv |= FSV_NI;
+#endif	/* defined(HASFSTRUCT) */
 
 /*
  * Determine the vnode type.
@@ -333,19 +343,27 @@ process_node(na)
 		else if (strcmp(Fsinfo[fx-1], "nfs") == 0)
 		    Ntype = N_NFS;
 
-#if	defined(HASPROCFS)
-		else if (strcmp(Fsinfo[fx-1], "proc") == 0)
-		    Ntype = N_PROC;
-#endif	/* defined(HASPROCFS) */
-
 #if	defined(HASOBJFS)
 		else if (strcmp(Fsinfo[fx-1], "objfs") == 0)
 		    Ntype = N_OBJF;
 #endif	/* defined(HASOBJFS) */
 
+#if	defined(HASPROCFS)
+		else if (strcmp(Fsinfo[fx-1], "proc") == 0)
+		    Ntype = N_PROC;
+#endif	/* defined(HASPROCFS) */
+
+		else if (strcmp(Fsinfo[fx-1], "bsdsfs") == 0)
+		    Ntype = N_SOCK;
+		else if (strcmp(Fsinfo[fx-1], "msockfs") == 0
+		     ||  strcmp(Fsinfo[fx-1], "sockfs")  == 0)
+		{
+		    if (process_bsdsock(&na, &v))
+			return;
+		}
 	    }
 	} else
-	    fx = Fsinfomax;
+	    fx = Fsinfomax + 1;
 	if (Ntype == N_REGLR) {
 	    if (v.v_type == VFIFO && !v.v_stream)
 		Ntype = N_FIFO;
@@ -423,6 +441,7 @@ process_node(na)
 #if	defined(HASPROCFS)
 	case N_PROC:
 	    ni = 1;
+	    i.nlink_def = 0;
 	    if (!v.v_data || kread((KA_T)v.v_data, (char *)&pr, sizeof(pr))) {
 		(void) sprintf(Namech, "vnode@%s; can't read prnode@%s",
 		    print_kptr(na, tbuf),
@@ -432,10 +451,14 @@ process_node(na)
 	    }
 	    if (pr.pr_proc == NULL) {
 		sd = 0;
-		if (v.v_type == VDIR)
+		i.nlink_def = 1;
+		if (v.v_type == VDIR) {
+		    i.nlink = 2l;
 		    i.number = PR_ROOTINO;
-		else
+		} else {
+		    i.nlink = 1l;
 		    i.number = 0L;
+		}
 		break;
 	    }
 	    if (kread((KA_T)pr.pr_proc, (char *)&p, sizeof(p))) {
@@ -462,11 +485,36 @@ process_node(na)
 	    else
 		i.size = as.a_size;
 	    (void) sprintf(Namech, "/%s/%d", HASPROCFS, pids.pid_id);
+	    i.nlink = 1l;
+	    i.nlink_def = 1;
 	    i.number = (long)pids.pid_id + PR_INOBIAS;
 	    enter_nm(Namech);
 	    break;
 #endif	/* defined(HASPROCFS) */
 
+	case N_SOCK:
+	    if (kread(na, (char *)&bn, sizeof(bn))) {
+		(void) sprintf(Namech, "vnode@%s: can't read real bsdsnode",
+		    print_kptr(na, tbuf));
+		enter_nm(Namech);
+		return;
+	    }
+	    if (!bn.bn_realvp
+	    ||  kread((KA_T)bn.bn_realvp, (char *)&v, sizeof(v))) {
+		(void) sprintf(Namech, "bsdsnode@%s: can't read real vnode@%s",
+		    print_kptr(na, tbuf),
+		    print_kptr((KA_T)bn.bn_realvp, (char *)NULL));
+		enter_nm(Namech);
+		return;
+	    }
+	    if (!v.v_stream) {
+		(void) sprintf(Namech,
+		    "bsdsnode real vnode@%s: no stream pointer",
+		    print_kptr((KA_T)bn.bn_realvp, tbuf));
+		enter_nm(Namech);
+		return;
+	    }
+	    /* fall through */
 	case N_STREAM:
 	    if (v.v_stream) {
 
@@ -479,7 +527,8 @@ process_node(na)
 		    return;
 		for (px = 0; px < NPROTOS; px++) {
 		    if (strcmp(mn, Protos[px].module) == 0) {
-			process_socket(Protos[px].proto, &q);
+			process_socket(Protos[px].proto, (KA_T)q.q_ptr,
+				       (SZOFFTYPE)q.q_count);
 			return;
 		    }
 		}
@@ -629,8 +678,9 @@ process_node(na)
 	 */
 	    if (readlino(fx, &v, &i, kvs ? &kv : (struct vfs *)NULL)) {
 		(void) sprintf(Namech,
-		    "vnode@%s; can't read successor node@%s",
+		    "vnode@%s; can't read %s successor node@%s",
 		    print_kptr(na, tbuf),
+		    (fx > 0 && fx <= Fsinfomax) ? Fsinfo[fx-1] : "(unknown fs)",
 		    print_kptr((KA_T)v.v_data, (char *)NULL));
 		enter_nm(Namech);
 		return;
@@ -782,6 +832,37 @@ process_node(na)
 		break;
 	    }
 	}
+/*
+ * Record link count.
+ */
+	if (Fnlink) {
+	    switch(Ntype) {
+	    case N_FIFO:
+		Lf->nlink = (long)f.fn_open;
+		Lf->nlink_def = 1;
+		break;
+	    case N_NFS:
+		Lf->nlink = (long)r.r_attr.va_nlink;
+		Lf->nlink_def = 1;
+		break;
+
+#if	defined(HASPROCFS)
+	    case N_PROC:
+		Lf->nlink = i.nlink;
+		Lf->nlink_def = i.nlink_def;
+		break;
+#endif	/* defined(HASPROCFS) */
+
+	    case N_REGLR:
+		if (!ni) {
+		    Lf->nlink = i.nlink;
+		    Lf->nlink_def = i.nlink_def;
+		}
+		break;
+	    }
+	    if (Nlink && Lf->nlink_def && (Lf->nlink < Nlink))
+		Lf->sf |= SELNLINK;
+	}
 /*
  * Record an NFS file selection.
  */
@@ -861,7 +942,7 @@ process_node(na)
 	if (!Lf->inp_ty && type == VCHR && Lf->dev_def)
 	    find_ch_ino();
 
-#if	defined(RELIANTV)
+#if	defined(RELIANTV) && defined(HASMIPCH)
 /*
  * If this is a Reliant UNIX VCHR file, missing an inode number, see if it's
  * an "mipc" file.
@@ -881,7 +962,7 @@ process_node(na)
 	    else
 		(void) sprintf(Namech, "MIPC (ns %d)", ns);
 	}
-#endif	/* defined(RELIANTV) */
+#endif	/* defined(RELIANTV) && defined(HASMIPCH) */
 
 /*
  * If this is a stream with a "sockmod" module whose q_ptr leads to an
@@ -939,7 +1020,7 @@ process_node(na)
 #endif	/* defined(HASPROCFS) */
 
 	{
-	    if (Sfile && is_file_named(NULL, type))
+	    if (Sfile && is_file_named(NULL, (type == VCHR) ? 1 : 0))
 		Lf->sf |= SELNM;
 	}
 /*
@@ -965,6 +1046,7 @@ readlino(fx, v, i, kv)
 	struct inode sn;
 	struct xnamnode xn;
 
+	i->nlink_def = 0;
 	if (fx < 1 || fx > Fsinfomax || v->v_data == NULL)
 	    return(1);
 	if (strcmp(Fsinfo[fx-1], "fifofs") == 0
@@ -973,6 +1055,8 @@ readlino(fx, v, i, kv)
 		return(1);
 	    i->dev = sn.i_dev;
 	    i->rdev = v->v_rdev;
+	    i->nlink = (long)sn.i_nlink;
+	    i->nlink_def = 1;
 	    i->number = sn.i_number;
 	    i->size = sn.i_size;
 	    return(0);
@@ -985,6 +1069,8 @@ readlino(fx, v, i, kv)
 		return(1);
 	    i->dev = nm.nm_vattr.va_fsid;
 	    i->rdev = nm.nm_vattr.va_rdev;
+	    i->nlink = (long)nm.nm_vattr.va_nlink;
+	    i->nlink_def = 1;
 	    i->number = nm.nm_vattr.va_nodeid;
 	    i->size = nm.nm_vattr.va_size;
 	    return(0);
@@ -996,6 +1082,8 @@ readlino(fx, v, i, kv)
 	    if (kread((KA_T)v->v_data, (char *)&xn, sizeof(xn)))
 		return(1);
 	    i->dev = xn.x_dev;
+	    i->nlink = (long)xn.x_count;
+	    i->nlink_def = 1;
 	    i->rdev = xn.x_fsid;
 	    i->size = xn.x_size;
 	    return(0);
