@@ -34,16 +34,12 @@
 
 
 _PROTOTYPE(extern int get_max_fd,(void));
-_PROTOTYPE(extern int is_file_named,(char *p, vtype_t ty));
-_PROTOTYPE(extern void process_socket,(char *pr, struct queue *q));
+_PROTOTYPE(extern int is_file_named,(char *p, int cd));
+_PROTOTYPE(extern int process_bsdsock,(KA_T *na, struct vnode *vp));
+_PROTOTYPE(extern void process_socket,(char *pr, KA_T pa, SZOFFTYPE sz));
 _PROTOTYPE(extern int readcdfslino,(struct vnode *v, struct l_ino *i));
 _PROTOTYPE(extern int reads5lino,(struct vnode *v, struct l_ino *i));
 _PROTOTYPE(extern int readtfslino,(struct vnode *v, struct l_ino *i, struct vfs *vfs));
 _PROTOTYPE(extern int readvxfslino,(struct vnode *v, struct l_ino *i));
 _PROTOTYPE(extern int strcasecmp,(char *s1, char *s2));
 _PROTOTYPE(extern int strncasecmp,(char *s1, char *s2, int n));
-
-#if	defined(HASDCACHE)
-_PROTOTYPE(extern int rw_clone_sect,(int m));
-_PROTOTYPE(extern void clr_sect,(void));
-#endif	/* defined(HASDCACHE) */
