@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: ddev.c,v 1.9 98/08/03 07:35:53 abe Exp $";
+static char *rcsid = "$Id: ddev.c,v 1.13 99/10/22 08:23:34 abe Exp $";
 #endif
 
 
@@ -46,9 +46,8 @@ static char *rcsid = "$Id: ddev.c,v 1.9 98/08/03 07:35:53 abe Exp $";
 static int Devx = 0;			/* current Devtp[] index */
 
 
-_PROTOTYPE(static void make_devtp,(struct stat *s, char *p, MALLOC_S nl));
+_PROTOTYPE(static void make_devtp,(struct stat *s, char *p));
 _PROTOTYPE(static int rmdupdev,(struct l_dev ***dp, int n, int ty));
-_PROTOTYPE(static void stkdir,(char ***d, int *n, int *x, char *p));
 
 
 /*
@@ -56,10 +55,9 @@ _PROTOTYPE(static void stkdir,(char ***d, int *n, int *x, char *p));
  */
 
 static void
-make_devtp(s, p, nl)
+make_devtp(s, p)
 	struct stat *s;			/* device lstat() buffer */
 	char *p;			/* device path name */
-	MALLOC_S nl;			/* path name length (including '\0') */
 {
 
 /*
@@ -187,7 +185,7 @@ read_clone()
 	struct clone *c;
 	char *cn;
 	DIR *dfp;
-	struct dirent *dp;
+	struct DIRTYPE *dp;
 	char *fp = (char *)NULL;
 	MALLOC_S fpl;
 	char *path;
@@ -206,7 +204,7 @@ read_clone()
 	    Exit(1);
 	}
 	path[pl - 1] = '\0';
-	if (!(dfp = opendir(path))) {
+	if (!(dfp = OpenDir(path))) {
 
 #if	defined(WARNDEVACCESS)
 	    if (!Fwarn) {
@@ -222,7 +220,7 @@ read_clone()
 /*
  * Scan the directory.
  */
-	for (dp = readdir(dfp); dp; dp = readdir(dfp)) {
+	for (dp = ReadDir(dfp); dp; dp = ReadDir(dfp)) {
 	    if (dp->d_ino == 0 || dp->d_name[0] == '.')
 		continue;
 	/*
@@ -266,7 +264,7 @@ read_clone()
 	/*
 	 * Make Devtp[] entry.
 	 */
-	    make_devtp(&sb, fp, fpl);
+	    make_devtp(&sb, fp);
 	 /*
 	  * Create a clone structure entry for "clone*:" devices.
 	  *
@@ -301,8 +299,7 @@ read_clone()
 	    /*
 	     * Make special note of a network clone device.
 	     */
-		cn++;
-		if (strcmp(cn, "tcp")  == 0 || strcmp(cn, "udp") == 0)
+		if (!strcmp(++cn, "tcp") || !strcmp(cn, "udp"))
 		    c->n = cn - fp;
 		else
 		    c->n = 0;
@@ -340,7 +337,7 @@ read_clone()
 		Pseudo = p;
 	    }
 	}
-	(void) closedir(dfp);
+	(void) CloseDir(dfp);
 	if (fp)
 	    (void) free((FREE_P *)fp);
 	if (path)
@@ -365,9 +362,7 @@ readdev(skip)
 
 	DIR *dfp;
 	int dn = 0;
-	struct dirent *dp;
-	char **dstk = (char **)NULL;
-	int dx = 0;
+	struct DIRTYPE *dp;
 	char *fp = (char *)NULL;
 	MALLOC_S fpl;
 	int i;
@@ -414,20 +409,24 @@ readdev(skip)
 	read_clone();
 #endif	/* defined(solaris) */
 
-	(void) stkdir(&dstk, &dn, &dx, DVCH_DEVPATH);
+	Dstk = (char **)NULL;
+	Dstkn = Dstkx = 0;
+	(void) stkdir(DVCH_DEVPATH);
 /*
  * Unstack the next directory.
  */
-	while (--dx >= 0) {
-	    if (!(dfp = opendir(dstk[dx]))) {
+	while (--Dstkx >= 0) {
+	    if (!(dfp = OpenDir(Dstk[Dstkx]))) {
 
 #if	defined(WARNDEVACCESS)
 		if (!Fwarn) {
 		    (void) fprintf(stderr, "%s: WARNING: can't open: ", Pn);
-		    safestrprt(dstk[dx], stderr, 1);
+		    safestrprt(Dstk[Dstkx], stderr, 1);
 		}
 #endif	/* defined(WARNDEVACCESS) */
 
+		(void) free((FREE_P *)Dstk[Dstkx]);
+		Dstk[Dstkx] = (char *)NULL;
 		continue;
 	    }
 	/*
@@ -437,18 +436,19 @@ readdev(skip)
 		(void) free((FREE_P *)NULL);
 		path = (char *)NULL;
 	    }
-	    if (!(path = mkstrcat(dstk[dx], -1, "/", 1, (char *)NULL, -1, &pl)))
+	    if (!(path = mkstrcat(Dstk[Dstkx], -1, "/", 1, (char *)NULL, -1,
+				  &pl)))
 	    {
 		(void) fprintf(stderr, "%s: no space for: ", Pn);
-		safestrprt(dstk[dx], stderr, 1);
+		safestrprt(Dstk[Dstkx], stderr, 1);
 		Exit(1);
 	    }
-	    (void) free((FREE_P *)dstk[dx]);
-	    dstk[dx] = (char *)NULL;
+	    (void) free((FREE_P *)Dstk[Dstkx]);
+	    Dstk[Dstkx] = (char *)NULL;
 	/*
 	 * Scan the directory.
 	 */
-	    for (dp = readdir(dfp); dp; dp = readdir(dfp)) {
+	    for (dp = ReadDir(dfp); dp; dp = ReadDir(dfp)) {
 		if (dp->d_ino == 0 || dp->d_name[0] == '.')
 		    continue;
 	    /*
@@ -503,7 +503,7 @@ readdev(skip)
 			continue;
 #endif	/* solaris */
 
-		    (void) stkdir(&dstk, &dn, &dx, fp);
+		    (void) stkdir(fp);
 			continue;
 		}
 		if ((sb.st_mode & S_IFMT) == S_IFCHR) {
@@ -511,7 +511,7 @@ readdev(skip)
 		/*
 		 * Make Devtp[] entry.
 		 */
-		    make_devtp(&sb, fp, fpl);
+		    make_devtp(&sb, fp);
 
 #if	!defined(solaris)
 		/*
@@ -582,13 +582,16 @@ readdev(skip)
 #endif	/* defined(HASBLKDEV) */
 
 	    }
-	    (void) closedir(dfp);
+	    (void) CloseDir(dfp);
 	}
 /*
  * Free any allocated space.
  */
-	if (dstk)
-	    (void) free((FREE_P *)dstk);
+	if (Dstk) {
+	    (void) free((FREE_P *)Dstk);
+	    Dstk = (char **)NULL;
+	    Dstkn = Dstkx = 0;
+	}
 	if (fp)
 	    (void) free((FREE_P *)fp);
 	if (path)
@@ -697,7 +700,7 @@ clr_sect()
 	    }
 	    Pseudo = (struct pseudo *)NULL;
 	}
-# endif	defined(solaris)
+# endif	/* defined(solaris) */
 
 }
 
@@ -846,8 +849,8 @@ bad_clone_sect:
 	 * not use the new format lines.
 	 */
 	    for (c = Clone; c; c = c->next) {
-		(void) sprintf(buf, " %x %d %ld %s\n", (int)c->cd.rdev, c->n,
-		    c->cd.inode, c->cd.name);
+		(void) sprintf(buf, " %lx %d %ld %s\n", (long)c->cd.rdev, c->n,
+		    (long)c->cd.inode, c->cd.name);
 		if (wr2DCfd(buf, &DCcksum))
 		    return(1);
 	    }
@@ -941,7 +944,7 @@ bad_pseudo_sect:
 	    /*
 	     * Enter the pseudo device number.
 	     *
-	     * New format clone lines (with an inode number) have a leading
+	     * New format pseudo lines (with an inode number) have a leading
 	     * space, so that older lsof versions, not expecting them, will
 	     * not use the new format lines.
 	     */
@@ -962,7 +965,7 @@ bad_pseudo_sect:
 		    if (*cp < '0' || *cp > '9') {
 			if (!Fwarn) {
 			    (void) fprintf(stderr,
-				"%s: bad cached clone inode number: ", Pn);
+				"%s: bad cached pseudo inode number: ", Pn);
 			    safestrprt(buf, stderr, 1);
 			}
 			return(1);
@@ -1012,13 +1015,13 @@ bad_pseudo_sect:
 	 * Write the pseudo section lines.
 	 *
 	 *
-	 * New format clone lines (with an inode number) have a leading
+	 * New format pseudo lines (with an inode number) have a leading
 	 * space, so that older lsof versions, not expecting them, will
 	 * not use the new format lines.
 	 */
 	    for (p = Pseudo; p; p = p->next) {
-		(void) sprintf(buf, " %x %ld %s\n", (int)p->pd.rdev,
-		    p->pd.inode, p->pd.name);
+		(void) sprintf(buf, " %lx %ld %s\n", (long)p->pd.rdev,
+		    (long)p->pd.inode, p->pd.name);
 		if (wr2DCfd(buf, &DCcksum))
 		    return(1);
 	    }
@@ -1153,44 +1156,3 @@ rmdupdev(dp, n, ty)
 	}
 	return(j);
 }
-
-
-/*
- * stkdir() - stack directory name
- */
-
-static void
-stkdir(d, n, x, p)
-	char ***d;		/* array of directory pointers */
-	int *n;			/* number of pointers */
-	int *x;			/* current index */
-	char *p;		/* directory path */
-{
-	MALLOC_S len;
-/*
- * Provide adequate directory stack space.
- */
-	if (*x >= *n) {
-	    *n += 10;
-	    len = (MALLOC_S)(*n * sizeof(char *));
-	    if (!*d)
-		*d = (char **)malloc(len);
-	    else
-		*d = (char **)realloc((MALLOC_P *)*d, len);
-	    if (!*d) {
-		(void) fprintf(stderr,
-		    "%s: no space for directory stack for: ", Pn);
-		safestrprt(p, stderr, 1);
-		Exit(1);
-	    }
-	}
-/*
- * Allocate space for the name, copy it there and put its pointer on the stack.
- */
-	if (!((*d)[*x] = mkstrcpy(p, (MALLOC_S *)NULL))) {
-	    (void) fprintf(stderr, "%s: no space for: ", Pn);
-	    safestrprt(p, stderr, 1);
-	    Exit(1);
-	}
-	*x += 1;
-}
