@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dmnt.c,v 1.7 98/07/17 08:50:44 abe Exp $";
+static char *rcsid = "$Id: dmnt.c,v 1.10 99/11/28 06:44:37 abe Exp $";
 #endif
 
 
@@ -67,6 +67,16 @@ getmntdev(o, l, s, f)
 	memset((char *)s, 0, sizeof(struct stat));
 	if (!(opte = x2dev(o + l + 1, &s->st_dev)))
 	    return((char *)NULL);
+
+#if	solaris>=70000 && L_BITSMAJOR!=L_BITSMAJOR32
+/*
+ * If this is a Solaris 7 system with a 64 bit kernel, convert the 32 bit
+ * device number to a 64 bit device number.
+ */
+	s->st_dev = (((s->st_dev >> L_BITSMINOR32) & L_MAXMAJ32) << L_BITSMINOR)
+		  | (s->st_dev & L_MAXMIN32);
+#endif	/* solaris>=70000 && L_BITSMAJOR!=L_BITSMAJOR32 */
+
 	s->st_mode = S_IFDIR | 0777;
 
 #if	defined(HASFSTYPE)
@@ -88,6 +98,7 @@ struct mounts *
 readmnt()
 {
 	int devl, ignore;
+	char *cp, *dir, *fs;
 	char *dn = (char *)NULL;
 	char *ln;
 	FILE *mfp;
@@ -141,9 +152,26 @@ readmnt()
 #endif	/* defined(solaris) */
 
 		continue;
-/*
- * Check for a "ignore" type (SunOS) or "ignore" option (Solaris).
- */
+	/*
+	 * Save pointers to the directory and file system names for later use.
+	 *
+	 * Check the file system name.  If it doesn't begin with a `/'
+	 * but contains a `:' not followed by a '/', ignore this entry.
+	 */
+
+#if	defined(solaris)
+	    dir = mp->mnt_mountp;
+	    fs = mp->mnt_special;
+#else	/* !defined(solaris) */
+	    dir = mp->mnt_dir;
+	    fs = mp->mnt_fsname;
+#endif	/* defined(solaris) */
+
+	    if (*fs != '/' && (cp = strchr(fs, ':')) && *(cp+1) != '/')
+		continue;
+	/*
+	 * Check for a "ignore" type (SunOS) or "ignore" option (Solaris).
+	 */
 
 #if	defined(solaris)
 	    if (hasmntopt(mp, MNTOPT_IGNORE))
@@ -159,29 +187,14 @@ readmnt()
 	 */
 	    if (dn)
 		(void) free((FREE_P *)dn);
-
-#if	defined(solaris)
-	    if (!(dn = mkstrcpy(mp->mnt_mountp, (MALLOC_S *)NULL)))
-#else	/* !defined(solaris) */
-	    if (!(dn = mkstrcpy(mp->mnt_dir, (MALLOC_S *)NULL)))
-#endif	/* defined(solaris) */
-
-	    {
+	    if (!(dn = mkstrcpy(dir, (MALLOC_S *)NULL))) {
 
 no_space_for_mount:
 
 		(void) fprintf(stderr, "%s: no space for mount ", Pn);
-
-#if	defined(solaris)
-		safestrprt(mp->mnt_special, stderr, 0);
+		safestrprt(fs, stderr, 0);
 		(void) fprintf(stderr, " (");
-		safestrprt(mp->mnt_mountp, stderr, 0);
-#else	/* !defined(solaris) */
-		safestrprt(mp->mnt_fsname, stderr, 0);
-		(void) fprintf(stderr, " (");
-		safestrprt(mp->mnt_dir, stderr, 0);
-#endif	/* defined(solaris) */
-
+		safestrprt(dir, stderr, 0);
 		(void) fprintf(stderr, ")\n");
 		Exit(1);
 	    }
@@ -237,13 +250,12 @@ no_space_for_mount:
 #if	defined(solaris)
 			safestrprt(mp->mnt_fstype, stderr, 0);
 			(void) fprintf(stderr, " file system ");
-			safestrprt(mp->mnt_mountp, stderr, 1);
 #else	/* !defined(solaris) */
 			safestrprt(mp->mnt_type, stderr, 0);
 			(void) fprintf(stderr, " file system ");
-			safestrprt(mp->mnt_dir, stderr, 1);
 #endif	/* defined(solaris) */
 
+			safestrprt(dir, stderr, 1);
 			(void) fprintf(stderr,
 			    "      Output information may be incomplete.\n");
 		}
@@ -300,6 +312,11 @@ no_space_for_mount:
 	    mtp->inode = sb.st_ino;
 	    mtp->mode = sb.st_mode;
 
+#if	defined(solaris) && solaris>=80000
+	    mtp->nlink = sb.st_nlink;
+	    mtp->size = sb.st_size;
+#endif	/* defined(solaris) && solaris>=80000 */
+
 #if	defined(HASPROCFS)
 
 	    if (strcmp(sb.st_fstype, HASPROCFS) == 0) {
@@ -319,19 +336,8 @@ no_space_for_mount:
 	/*
 	 * Interpolate a possible file system (mounted-on) device name link.
 	 */
-	    if (!(dn = mkstrcpy(
-
-#if	defined(solaris)
-				mp->mnt_special,
-#else	/* !defined(solaris) */
-				mp->mnt_fsname,
-#endif	/* defined(solaris) */
-
-				(MALLOC_S *)NULL)))
-
-	    {
+	    if (!(dn = mkstrcpy(fs, (MALLOC_S *)NULL)))
 		goto no_space_for_mount;
-	    }
 	    mtp->fsname = dn;
 	    ln = Readlink(dn);
 	    dn = (char *)NULL;
