@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.16 98/07/14 12:33:05 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.26 99/11/28 06:44:39 abe Exp $";
 #endif
 
 
@@ -50,12 +50,35 @@ _PROTOTYPE(static struct l_dev *finddev,(dev_t *dev, int flags));
  * Finddev() "look-in " flags
  */
 
-#define	LOOKDEV_TAB	1		/* look in device table */
-#define	LOOKDEV_CLONE	2		/* look in Clone table */
-#define	LOOKDEV_PSEUDO	4		/* look in Pseudo table */
+#define	LOOKDEV_TAB	0x01		/* look in device table */
+#define	LOOKDEV_CLONE	0x02		/* look in Clone table */
+#define	LOOKDEV_PSEUDO	0x04		/* look in Pseudo table */
 #define	LOOKDEV_ALL	(LOOKDEV_TAB | LOOKDEV_CLONE | LOOKDEV_PSEUDO)
 					/* look all places */
 
+#if	defined(AUSPEXV)
+/*
+ * LFS_node -- the Auspex LFS node structure (this definition
+ * should be in a header file distributed by Auspex.)
+ */
+
+struct LFS_node {
+	caddr_t d1[3];			/* filler */
+	struct vnode LFS_vnode;		/* LFS_node's vnode */
+	caddr_t d2[18];			/* filler */
+	short LFS_mode;			/* modes */
+	uid_t LFS_uid;			/* owning user ID */
+	gid_t LFS_gid;			/* owning group ID */
+	short d3[2];			/* filler */
+	dev_t LFS_dev;			/* device */
+	ino_t LFS_nodeid;		/* node ID */
+	short LFS_nlink;		/* link count */
+	short d4;			/* filler */
+	long LFS_size;			/* size */
+};
+#endif	/* defined(AUSPEXV) */
+
+
 #if	defined(solaris)
 _PROTOTYPE(static char isvlocked,(struct vnode *va));
 _PROTOTYPE(static int readinode,(KA_T ia, struct inode *i));
@@ -73,12 +96,18 @@ _PROTOTYPE(static int read_nusa,(struct soaddr *so, struct sockaddr_un *ua));
 # else	/* solaris<20600 */
 _PROTOTYPE(static int read_nan,(KA_T na, KA_T aa, struct autonode *a));
 # endif	/* solaris>=20600 */
-_PROTOTYPE(static int read_ndn,(KA_T na, struct door_node *d));
+_PROTOTYPE(static int idoorkeep,(struct door_node *d));
+_PROTOTYPE(static int read_ndn,(KA_T na, KA_T da, struct door_node *d));
 #endif	/* defined(solaris) && solaris>=20500 */
 
 _PROTOTYPE(static int read_nfn,(KA_T na, KA_T fa, struct fifonode *f));
 _PROTOTYPE(static int read_nhn,(KA_T na, KA_T ha, struct hsnode *h));
 _PROTOTYPE(static int read_nin,(KA_T na, KA_T ia, struct inode *i));
+
+#if	defined(AUSPEXV)
+_PROTOTYPE(static int read_nLFSn,(KA_T na, KA_T la, struct LFS_node *l));
+#endif	/* defined(AUSPEXV) */
+
 _PROTOTYPE(static int read_nmn,(KA_T na, KA_T ia, struct mvfsnode *m));
 _PROTOTYPE(static int read_npn,(KA_T na, KA_T pa, struct pcnode *p));
 _PROTOTYPE(static int read_nrn,(KA_T na, KA_T ra, struct rnode *r));
@@ -114,7 +143,13 @@ static KA_T Cvops;		/* cache_vnodeops address */
 static KA_T Dvops;		/* door_vnodeops address */
 static KA_T Fvops;		/* [_]fifo_vnodeops address */
 static KA_T Hvops;		/* [_]hsfs_vnodeops address */
+
+#if	defined(AUSPEXV)
+static KA_T LFSvops;		/* [_]lfs_vnodeops (Auspex) address */
+#endif	/* defined(AUSPEXV) */
+
 static KA_T Lvops;		/* lo_vnodeops address */
+static KA_T Mntops;		/* mntvnodeops address */
 static KA_T Mvops;		/* [_]mvfs_vnodeops address */
 static KA_T N3vops;		/* [_]nfs3_vnodeops address */
 static KA_T Nmvops;		/* nm_vnodeops address */
@@ -172,26 +207,90 @@ static int
 is_socket(v)
 	struct vnode *v;		/* vnode pointer */
 {
-	struct clone *c;
+	char *cp, *ep, *pf;
+	struct l_dev *dp;
+	int i, j, len, n, pfl;
+	static struct tcpudp {
+	    int ds;
+	    dev_t rdev;
+	    char *proto;
+	} tcpudp[] = {
+	    { 0, 0, "tcp" },
+	    { 0, 0, "udp" },
+
+#  if	defined(HASIPv6)
+	    { 0, 0, "tcp6" },
+	    { 0, 0, "udp6" },
+#  endif	/* defined(HASIPv6) */
+
+	};
+#define	NTCPUDP	(sizeof(tcpudp) / sizeof(struct tcpudp))
+
+	static int tcpudps = 0;
 
 	if (!v->v_stream)
 	    return(0);
+/*
+ * Fill in tcpudp[], as required.
+ */
+	if (!tcpudps) {
 
-#if	defined(HASDCACHE)
-is_socket_again:
-#endif	/* defined(HASDCACHE) */
+#  if	solaris<80000
+	    pf = "/devices/pseudo/clone";
+#  else	/* solaris>=80000 */
+	    pf = "/devices/pseudo/";
+#  endif	/* solaris<80000 */
 
-	for (c = Clone; c; c = c->next) {
-	    if (c->n && major(v->v_rdev) == minor(c->cd.rdev)) {
+	    for (i = n = 0, pfl = strlen(pf); (i < Ndev) && (n < NTCPUDP); i++)
+	    {
+		if (strncmp(Devtp[i].name, pf, pfl)
+		||  !(ep = strrchr((cp = &Devtp[i].name[pfl]), ':'))
+		||  (strncmp(++ep, "tcp", 3) && strncmp(ep, "udp", 3)))
+		    continue;
 
-#if     defined(HASDCACHE)
-		if (DCunsafe && !c->cd.v && !vfy_dev(&c->cd))
-		    goto is_socket_again;
-#endif  /* defined(HASDCACHE) */
+#  if	solaris<80000
+		if (*(ep + 3))
+		    continue;
+		for (j = 0; j < NTCPUDP; j++) {
+		    if (!tcpudp[j].ds && !strcmp(ep, tcpudp[j].proto)) {
+			tcpudp[j].ds = 1;
+			tcpudp[j].rdev = Devtp[i].rdev;
+			n++;
+			break;
+		    }
+		}
+#  else	/* solaris>=80000 */
+		len = (*(ep + 3) == '6') ? 4 : 3;
+		if (*(ep + len) || ((cp + len) >= ep) || strncmp(cp, ep, len))
+		    continue;
+		for (j = 0; j < NTCPUDP; j++) {
+		    if (!tcpudp[j].ds && !strcmp(ep, tcpudp[j].proto)) {
+			tcpudp[j].ds = 1;
+			tcpudp[j].rdev = Devtp[i].rdev;
+			n++;
+			break;
+		    }
+		}
+#  endif	/* solaris<80000 */
 
-		process_socket((KA_T)v->v_stream, &c->cd.name[c->n]);
-		return(1);
 	    }
+	    tcpudps = n ? 1 : -1;
+	}
+/*
+ * Check for known IPv[46] TCP or UDP device.
+ */
+	for (i = 0; (i < NTCPUDP) && (tcpudps > 0); i++) {
+
+#  if	solaris<80000
+	    if (!tcpudp[i].ds || (major(v->v_rdev) != minor(tcpudp[i].rdev)))
+		continue;
+#  else	/* solaris>=80000 */
+	    if (!tcpudp[i].ds || (major(v->v_rdev) != major(tcpudp[i].rdev)))
+		continue;
+#  endif	/* solaris<80000 */
+
+	    process_socket((KA_T)v->v_stream, tcpudp[i].proto);
+	    return(1);
 	}
 	return(0);
 }
@@ -200,7 +299,7 @@ is_socket_again:
 
 #if	defined(solaris)
 /*
- * isvlocked() - is Soalris vnode locked?
+ * isvlocked() - is Solaris vnode locked?
  */
 
 static char
@@ -457,6 +556,67 @@ finddev_again:
 }
 
 
+#if	defined(solaris) && solaris>=20500
+/*
+ * idoorkeep() -- identify door keeper process
+ */
+
+static int
+idoorkeep(d)
+	struct door_node *d;		/* door's node */
+{
+	char buf[64];
+	int cc, comml, pidl;
+	struct proc dp;
+	struct pid dpid;
+	int rv = 0;
+/*
+ * Get the proc structure and its pud structure for the door target.
+ */
+	if (!d->door_target
+	||  kread((KA_T)d->door_target, (char *)&dp, sizeof(dp)))
+	    return(rv);
+	if (!dp.p_pidp
+	||  kread((KA_T)dp.p_pidp, (char *)&dpid, sizeof(dpid)))
+	    return(rv);
+/*
+ * Try to form a name of "door to process-command-name[PID]; if PID is for
+ * this process, just form the name "door of this process".
+ *
+ * Drop the trailing "[PID]" if it doesn't fit.
+ *
+ * If the command name doesn't fit, try "door to PID".
+ */
+	if (Lp->pid == (int)dpid.pid_id) {
+	    (void) strcpy(Namech, "door of this process");
+	    rv = 1;
+	    return(rv);
+	}
+	(void) strcpy(Namech, "door to ");
+	cc = strlen(Namech);
+	comml = strlen(dp.p_user.u_comm);
+	(void) sprintf(buf, "%ld", (long)dpid.pid_id);
+	pidl = strlen(buf);
+	if ((cc + comml + 1) <= MAXPATHLEN) {
+	    (void) strcpy(&Namech[cc], dp.p_user.u_comm);
+	    cc += comml;
+	    rv = 1;
+	} else {
+	    if ((cc + pidl + 1) <= MAXPATHLEN) {
+		(void) strcpy(&Namech[cc], buf);
+		rv = 1;
+	    }
+	    return(rv);
+	}
+	if ((cc + 1 + pidl + 1 + 1) <= MAXPATHLEN) {
+	    (void) sprintf(&Namech[cc], "[%s]", buf);
+	    rv = 1;
+	}
+	return(rv);
+}
+#endif	/* defined(solaris) && solaris>=20500 */
+
+
 /*
  * process_node() - process vnode
  */
@@ -496,7 +656,13 @@ process_node(va)
 	int vty = 0;
 	int  vty_tmp;
 
+#if	defined(AUSPEXV)
+	struct LFS_node ln;
+#endif	/* defined(AUSPEXV) */
+
 #if	defined(solaris)
+	int fx;
+	int fxs = 0;
 	int len, nl, snl, sepl;
 # if	solaris>=20500
 #  if	solaris>=20600
@@ -505,7 +671,7 @@ process_node(va)
 	    short f;
 	    unsigned short p;
 	} *pa;
-	unsigned int peer;
+	KA_T peer;
 	struct sonode so;
 	KA_T soa;
 #  else	/* solaris<20600 */
@@ -513,6 +679,7 @@ process_node(va)
 #  endif	/* solaris>=20600 */
 
 	struct door_node dn;
+	int dns = 0;
 # endif	/* solaris >=20500 */
 
 	struct cnode cn;
@@ -543,6 +710,7 @@ process_node(va)
 #if	defined(HASVXFS)
 	dev_t vx_dev;
 	unsigned long vx_ino, vx_sz;
+	long vx_nl;
 #endif	/* defined(HASVXFS) */
 
 /*
@@ -564,8 +732,16 @@ process_node(va)
 		Fvops = (KA_T)0;
 	    if (get_Nl_value("hvops", Drive_Nl, &Hvops) < 0)
 		Hvops = (KA_T)0;
+
+#if	defined(AUSPEXV)
+	    if (get_Nl_value("lfsvops", Drive_Nl, &LFSvops) < 0)
+		LFSvops = (KA_T)0;
+#endif	/* defined(AUSPEXV) */
+
 	    if (get_Nl_value("lvops", Drive_Nl, &Lvops) < 0)
 		Lvops = (KA_T)0;
+	    if (get_Nl_value("mntops", Drive_Nl, &Mntops) < 0)
+		Mntops = (KA_T)0;
 	    if (get_Nl_value("mvops", Drive_Nl, &Mvops) < 0)
 		Mvops = (KA_T)0;
 	    if (get_Nl_value("n3vops", Drive_Nl, &N3vops) < 0)
@@ -630,9 +806,14 @@ process_node(va)
 	}
 
 #if	defined(HASNCACHE)
-	Lf->na = (KA_T)va;
+	Lf->na = va;
 #endif	/* defined(HASNCACHE) */
 
+#if	defined(HASFSTRUCT)
+	Lf->fna = va;
+	Lf->fsv |= FSV_NI;
+#endif	/* defined(HASFSTRUCT) */
+
 	vs = (KA_T)v->v_stream;
 
 #if	defined(solaris)
@@ -662,10 +843,11 @@ vfs_read_error:
  * Derive the virtual file system structure's device number from
  * its file system ID for NFS and High Sierra file systems.
  */
-	if (kvs && kv.vfs_fstype > 0 && kv.vfs_fstype <= Fsinfomax) {
-	    if (strcmp(Fsinfo[kv.vfs_fstype - 1], "nfs") == 0
-	    ||  strcmp(Fsinfo[kv.vfs_fstype - 1], "nfs3") == 0
-	    ||  strcmp(Fsinfo[kv.vfs_fstype - 1], "hsfs") == 0)
+	if (kvs && (fx = kv.vfs_fstype) > 0 && fx <= Fsinfomax) {
+	    fxs = 1;
+	    if (strcmp(Fsinfo[fx - 1], "nfs") == 0
+	    ||  strcmp(Fsinfo[fx - 1], "nfs3") == 0
+	    ||  strcmp(Fsinfo[fx - 1], "hsfs") == 0)
 		kv.vfs_dev = (dev_t)kv.vfs_fsid.val[0];
 	}
 /*
@@ -680,7 +862,11 @@ vfs_read_error:
 		Lf->is_stream = 1;
 	    }
 	    if (Ntype < 0) {
-		(void) sprintf(Namech, "unknown file system type, v_op: %s",
+		(void) sprintf(Namech,
+		    "unknown file system type%s%s%s, v_op: %s",
+		    fxs ? " (" : "",
+		    fxs ? Fsinfo[fx - 1] : "",
+		    fxs ? ")" : "",
 		    print_kptr((KA_T)v->v_op, (char *)NULL));
 		enter_nm(Namech);
 		return;
@@ -735,7 +921,7 @@ vfs_read_error:
 		    return;
 	    }
 	}
-	if (Selinet)
+	if (Selinet && Ntype != N_SOCK)
 	    return;
 /*
  * See if this Solaris node is served by spec_vnodeops.
@@ -825,7 +1011,7 @@ vfs_read_error:
 
 /*
  * Read the afsnode, autonode, cnode, door_node, fifonode, fnnode, lnode,
- * inode, pcnode, rnode, snode or tmpnode.
+ * inode, pcnode, rnode, snode, tmpnode, etc.
  */
 	switch (Ntype) {
 	case N_SPEC:
@@ -877,6 +1063,13 @@ vfs_read_error:
 	    break;
 #endif	/* defined(HAS_AFS) */
 
+#if	defined(AUSPEXV)
+	case N_AUSX:
+	    if (read_nLFSn(va, (KA_T)v->v_data, &ln))
+		return;
+	    break;
+#endif	/* defined(AUSPEXV) */
+
 #if	defined(solaris)
 # if	solaris>=20500
 	case N_AUTO:
@@ -890,8 +1083,9 @@ vfs_read_error:
 		return;
 	    break;
 	case N_DOOR:
-	    if (read_ndn(va, &dn))
+	    if (read_ndn(va, (KA_T)v->v_data, &dn))
 		return;
+	    dns = 1;
 	    break;
 # endif	/* solaris>=20500 */
 
@@ -908,6 +1102,9 @@ vfs_read_error:
 #endif	/* defined(solaris) */
 
 
+	case N_MNT:
+	    /* Information comes from the l_vfs structure. */
+	    break;
 	case N_MVFS:
 	    if (read_nmn(va, (KA_T)v->v_data, &m))
 		return;
@@ -954,6 +1151,20 @@ vfs_read_error:
 	    if (!realvp) {
 		Lf->inode = (unsigned long)f.fn_ino;
 		Lf->inp_ty = 1;
+
+# if	solaris>=80000	/* Solaris 8 Beta hack! */
+	    /*
+	     * !!!!!!DEBUG!!!   !!!DEBUG!!!   !!!DEBUG!!!   !!!DEBUG!!!!!!!
+	     * !                                                          !
+	     * ! The following if test is a Solaris 8 Beta hack!  Remove  !
+	     * ! it when possible.                                        !
+	     * !                                                          !
+	     * !!!!!!DEBUG!!!   !!!DEBUG!!!   !!!DEBUG!!!   !!!DEBUG!!!!!!!
+	     */
+		if ((unsigned long)f.fn_ino >= (unsigned long)0xbaddcafe)
+		    Lf->inp_ty = 3;
+# endif	/* solaris>=80000 Solaris 8 Beta hack! */
+
 		enter_dev_ch(print_kptr((KA_T)v->v_data, (char *)NULL));
 		if (f.fn_flag & ISPIPE)
 		    (void) strcpy(Namech, "PIPE");
@@ -1039,7 +1250,7 @@ vfs_read_error:
 
 #if	defined(HASVXFS)
 	case N_VXFS:
-	    if (read_vxnode(va, v, vfs, &vx_dev, &vx_ino, &vx_sz))
+	    if (read_vxnode(va, v, vfs, &vx_dev, &vx_ino, &vx_nl, &vx_sz))
 		return;
 	    break;
 #endif	/* defined(HASVXFS) */
@@ -1103,6 +1314,12 @@ vfs_read_error:
 		break;
 #endif	/* defined(HAS_AFS) */
 
+#if	defined(AUSPEXV)
+	    case N_AUSX:
+		if (read_nLFSn(va, (KA_T)v->v_data, &ln))
+		    return;
+		break;
+#endif	/* defined(AUSPEXV) */
 	
 #if	defined(solaris)
 # if	solaris>=20500
@@ -1119,12 +1336,13 @@ vfs_read_error:
 	    case N_DOOR:
 
 #  if	solaris<20600
-		if (read_ndn(realvp, &dn))
+		if (read_ndn(realvp, (KA_T)v->v_data, &dn))
 #  else	/* solaris>=20600 */
-		if (read_ndn(va, &dn))
+		if (read_ndn(va, (KA_T)v->v_data, &dn))
 #  endif	/* solaris<20500 */
 
 		    return;
+		dns = 1;
 		break;
 # endif	/* solaris>=20500 */
 	    case N_CACHE:
@@ -1144,6 +1362,9 @@ vfs_read_error:
 		if (read_nhn(va, (KA_T)v->v_data, &h))
 		    return;
 		break;
+	    case N_MNT:
+		/* Information comes from the l_vfs structure. */
+		break;
 	    case N_MVFS:
 		if (read_nmn(va, (KA_T)v->v_data, &m))
 		    return;
@@ -1180,7 +1401,7 @@ vfs_read_error:
 
 #if	defined(HASVXFS)
 	    case N_VXFS:
-		if (read_vxnode(va, v, vfs, &vx_dev, &vx_ino, &vx_sz))
+		if (read_vxnode(va, v, vfs, &vx_dev, &vx_ino, &vx_nl, &vx_sz))
 		    return;
 		break;
 #endif	/* defined(HASVXFS) */
@@ -1214,6 +1435,15 @@ vfs_read_error:
 	    break;
 #endif	/* defined(HAS_AFS) */
 
+#if	defined(AUSPEXV)
+	case N_AUSX:
+	    if (v->v_type == VCHR || v->v_type == VBLK)
+		dev = v->v_rdev;
+	    else
+		dev = ln.LFS_dev;
+	    break;
+#endif	/* defined(AUSPEXV) */
+
 #if	defined(solaris)
 # if	solaris>=20500
 	case N_AUTO:
@@ -1230,8 +1460,10 @@ vfs_read_error:
 		dev_def = 1;
 	    }
 #  else	/* solaris>=20600 */
-	    dev = (dev_t)dn.door_index;
-	    dev_def = 1;
+	    if (dns) {
+		dev = (dev_t)dn.door_index;
+		dev_def = 1;
+	    }
 #  endif	/* solaris<20600 */
 
 	    break;
@@ -1307,10 +1539,11 @@ vfs_read_error:
 		 * Print its identity.
 		 */
 		    pa = (struct pairaddr *)&ua;
-		    if (!(peer = (unsigned int)pa->p))
-			peer = (unsigned int)so.so_ux_laddr.sou_vp & 0xffff;
+		    if (!(peer = (KA_T)((int)pa->p)))
+			peer = (KA_T)so.so_ux_laddr.sou_vp;
 		    if (peer)
-			(void) sprintf(ubuf, "(socketpair: 0x%x)", peer);
+			(void) sprintf(ubuf, "(socketpair: %s)",
+			    print_kptr(peer, (char *)NULL));
 		    else
 			(void) strcpy(ubuf, "(socketpair)");
 		    len = strlen(ubuf);
@@ -1363,10 +1596,24 @@ vfs_read_error:
 	    break;
 #endif	/* defined(solaris) */
 
-	case N_MVFS:
+	case N_MNT:
+
+#if	defined(CVFS_DEVSAVE)
 	    if (vfs)
 		dev = vfs->dev;
 	    else
+#endif	/* defined(CVFS_DEVSAVE) */
+
+		dev_def = 0;
+	    break;
+	case N_MVFS:
+
+#if	defined(CVFS_DEVSAVE)
+	    if (vfs)
+		dev = vfs->dev;
+	    else
+#endif	/* defined(CVFS_DEVSAVE) */
+
 		dev_def = 0;
 	    break;
 	case N_NFS:
@@ -1507,6 +1754,13 @@ vfs_read_error:
 	    break;
 #endif	/* defined(HAS_AFS) */
 
+#if	defined(AUSPEXV)
+	case N_AUSX:
+	    Lf->inode = ln.LFS_nodeid;
+	    Lf->inp_ty = 1;
+	    break;
+#endif	/* defined(AUSPEXV) */
+
 #if	defined(solaris)
 # if	solaris>=20500
 	case N_AUTO:
@@ -1524,8 +1778,10 @@ vfs_read_error:
 		Lf->inp_ty = 1;
 		break;
 	    }
-	    if ((Lf->inode = (unsigned long)dn.door_active)) 
-		Lf->inp_ty = 1;
+	    if (dns) {
+		if ((Lf->inode = (unsigned long)dn.door_index)) 
+		    Lf->inp_ty = 1;
+	    }
 	    break;
 # endif	/* solaris>=20500 */
 
@@ -1537,15 +1793,25 @@ vfs_read_error:
 
 	case N_HSFS:
 
-#if	defined(solaris)
+#if	defined(solaris) || defined(HAS_HS_NODEID)
 	    Lf->inode = (unsigned long)h.hs_nodeid;
-#else	/* !defined(solaris) */
+#else	/* !defined(solaris) || defined(HAS_HS_NODEID) */
 	    Lf->inode = (unsigned long)h.hs_dirent.ext_lbn;
 #endif	/* defined(solaris) */
 
 	    Lf->inp_ty = 1;
 	    break;
 
+	case N_MNT:
+
+#if	defined(HASFSINO)
+	    if (vfs) {
+		Lf->inode = vfs->fs_ino;
+		Lf->inp_ty = 1;
+	    }
+#endif	/* defined(HASFSINO) */
+
+	    break;
 	case N_MVFS:
 	    Lf->inode = (unsigned long)m.m_ino;
 	    Lf->inp_ty = 1;
@@ -1578,7 +1844,7 @@ vfs_read_error:
 	    if (kvs && kv.vfs_data
 	    && !kread((KA_T)kv.vfs_data, (char *)&pcfs, sizeof(pcfs))) {
 
-# if	solaris>=20700
+# if	solaris>=70000
 		Lf->inode = (unsigned long)pc_makenodeid(pc.pc_eblkno,
 			    pc.pc_eoffset,
 			    pc.pc_entry.pcd_attr,
@@ -1587,12 +1853,12 @@ vfs_read_error:
 				  (ltohs(pc.pc_entry.un.pcd_scluster_hi) << 16)
 				: ltohs(pc.pc_entry.pcd_scluster_lo),
 			    pcfs.pcfs_entps);
-# else	/* solaris<20700 */
+# else	/* solaris<70000 */
 		Lf->inode = (unsigned long)pc_makenodeid(pc.pc_eblkno,
 			    pc.pc_eoffset,
 			    &pc.pc_entry,
 			    pcfs.pcfs_entps);
-# endif	/* solaris>=20700 */
+# endif	/* solaris>=70000 */
 
 		Lf->inp_ty = 1;
 	    }
@@ -1662,6 +1928,13 @@ vfs_read_error:
 		break;
 #endif	/* defined(HAS_AFS) */
 
+#if	defined(AUSPEXV)
+	    case N_AUSX:
+		Lf->sz = (SZOFFTYPE)ln.LFS_size;
+		Lf->sz_def = 1;
+		break;
+#endif	/* defined(AUSPEXV) */
+
 #if	defined(solaris)
 # if	solaris>=20500
 	    case N_AUTO:
@@ -1680,6 +1953,7 @@ vfs_read_error:
 		Lf->sz = (SZOFFTYPE)cn.c_size;
 		Lf->sz_def = 1;
 		break;
+
 # if	defined(solaris) && solaris>=20600
 	    case N_SOCK:
 		Lf->off_def = 1;
@@ -1703,6 +1977,17 @@ vfs_read_error:
 	    case N_DOOR:
 	    case N_FIFO:
 		if (!Fsize)
+		    Lf->off_def = 1;
+		break;
+	    case N_MNT:
+
+#if	defined(CVFS_SZSAVE)
+		if (vfs) {
+		    Lf->sz = (SZOFFTYPE)vfs->size;
+		    Lf->sz_def = 1;
+		} else
+#endif	/* defined(CVFS_SZSAVE) */
+
 		    Lf->off_def = 1;
 		break;
 	    case N_MVFS:
@@ -1765,6 +2050,118 @@ vfs_read_error:
 
 	    }
 	}
+/*
+ * Record link count.
+ */
+	if (Fnlink) {
+	    switch(Ntype) {
+
+#if	defined(HAS_AFS)
+	    case N_AFS:
+		Lf->nlink = an.nlink;
+		Lf->nlink_def = an.nlink_st;
+		break;
+#endif	/* defined(HAS_AFS) */
+
+#if	defined(AUSPEXV)
+	    case N_AUSX:
+		Lf->nlink = ln.LFS_nlink;
+		Lf->nlink_def = 1;
+		break;
+#endif	/* defined(AUSPEXV) */
+
+#if	defined(solaris) && solaris>=20500
+	    case N_AUTO:
+		Lf->nlink = 2l;
+		Lf->nlink_def = 1;
+		break;
+	    case N_CACHE:
+		Lf->nlink = (long)cn.c_attr.va_nlink;
+		Lf->nlink_def = 1;
+		break;
+#endif	/* defined(solaris) && solaris>=20500 */
+
+#if	defined(solaris) && solaris>=20600
+	    case N_SOCK:			/* no link count */
+		break;
+#endif	/* defined(solaris) && solaris>=20600 */
+
+#if	defined(solaris) || (defined(SUNOSV) && SUNOSV<40103)
+	    case N_HSFS:
+		Lf->nlink = (long)h.hs_dirent.nlink;
+		Lf->nlink_def = 1;
+		break;
+#endif	/* defined(solaris) || (defined(SUNOSV) && SUNOSV<40103) */
+
+#if	defined(solaris)
+	    case N_NM:
+		Lf->nlink = (long)nn.nm_vattr.va_nlink;
+		Lf->nlink_def = 1;
+		break;
+#endif	/* defined(solaris) */
+
+	    case N_DOOR:
+		Lf->nlink = (long)v->v_count;
+		Lf->nlink_def = 1;
+		break;
+	    case N_FIFO:
+		Lf->nlink = 0l;
+		Lf->nlink_def = 1;
+		break;
+	    case N_MNT:
+
+#if	defined(CVFS_NLKSAVE)
+		if (vfs) {
+		    Lf->nlink = (long)vfs->nlink;
+		    Lf->nlink_def = 1;
+		}
+#endif	/* defined(CVFS_NLKSAVE) */
+
+		break;
+	    case N_MVFS:			/* no link count */
+		break;
+	    case N_NFS:
+		Lf->nlink = (long)r.r_attr.va_nlink;
+		Lf->nlink_def = 1;
+		break;
+	    case N_PCFS:
+		Lf->nlink = 1l;
+		Lf->nlink_def = 1;
+		break;
+
+#if	defined(HASPROCFS)
+	    case N_PROC:
+		Lf->nlink = (type == VDIR) ? 2l : 1l;
+		Lf->nlink_def = 1;
+		break;
+#endif	/* defined(HASPROCFS) */
+
+	    case N_REGLR:
+		if (ins) {
+		    Lf->nlink = (long)i.i_nlink;
+		    Lf->nlink_def = 1;
+		}
+		break;
+	    case N_STREAM:
+		Lf->nlink = 0l;
+		Lf->nlink_def = 1;
+		break;
+	    case N_TMP:
+		Lf->nlink = (long)t.tn_attr.va_nlink;
+		Lf->nlink_def = 1;
+		break;
+
+#if	defined(HASVXFS)
+	    case N_VXFS:
+		Lf->nlink = vx_nl;
+		Lf->nlink_def = 1;
+		break;
+#endif	/* defined(HASVXFS) */
+
+	    }
+	    if (Nlink && Lf->nlink_def && (Lf->nlink < Nlink))
+		Lf->sf |= SELNLINK;
+	}
 /*
  * Record an NFS selection.
  */
@@ -1863,6 +2260,12 @@ vfs_read_error:
 	    Lf->dev = dev;
 	    Lf->dev_def = dev_def;
 	    ty = "DOOR";
+	    if (dns && !Namech[0]) {
+		if (idoorkeep(&dn) && nns && Lf->nma) {
+		    (void) free((FREE_P *)Lf->nma);
+		    Lf->nma = (char *)NULL;
+		}
+	    }
 	    break;
 #endif	/* defined(solaris) && solaris>=20500 */
 
@@ -1890,8 +2293,35 @@ vfs_read_error:
 		ty = "unix";
 		if (Funix)
 		    Lf->sf |= SELUNX;
-	    } else
-		ty = "sock";
+	    } else {
+		if (so.so_family == AF_INET) {
+
+#  if	defined(HASIPv6)
+		    ty = "IPv4";
+#  else	/* !defined(HASIPv6) */
+		    ty = "inet";
+#  endif	/* defined(HASIPv6) */
+
+		    (void) strcpy(Namech, printsockty(so.so_type));
+		    if (Fnet)
+			Lf->sf |= SELNET;
+		}
+
+#  if	defined(HASIPv6)
+		else if (so.so_family == AF_INET6) {
+		    ty = "IPv6";
+		    (void) strcpy(Namech, printsockty(so.so_type));
+		    if (Fnet)
+			Lf->sf |= SELNET;
+		}
+#  endif	/* defined(HASIPv6) */
+
+		else {
+		    ty = "sock";
+		    (void) printunkaf(so.so_family, 0);
+		    (void) sprintf(endnm(), ", %s", printsockty(so.so_type));
+		}
+	    }
 # else	/* !defined(solaris) || solaris<20600 */
 	    ty = "sock";
 # endif	/* defined(solaris) && solaris>=20600 */
@@ -2110,15 +2540,16 @@ readinode(ia, i)
  */
 
 static int
-read_ndn(na, dn)
+read_ndn(na, da, dn)
 	KA_T na;			/* containing vnode's address */
+	KA_T da;			/* door node's address */
 	struct door_node *dn;		/* door node receiver */
 {
 	char tbuf[32];
 
-	if (!na || kread((KA_T)na, (char *)dn, sizeof(struct door_node))) {
+	if (!da || kread((KA_T)da, (char *)dn, sizeof(struct door_node))) {
 	    (void) sprintf(Namech, "vnode at %s: can't read door_node: %s",
-		print_kptr(na, tbuf), print_kptr(na, (char *)NULL));
+		print_kptr(na, tbuf), print_kptr(da, (char *)NULL));
 	    enter_nm(Namech);
 	    return(1);
 	}
@@ -2372,6 +2803,30 @@ read_nin(na, ia, i)
 }
 
 
+#if	defined(AUSPEXV)
+/*
+ * Read_nLFSn() - read node's Auspex LFS node
+ */
+
+static int
+read_nLFSn(na, la, ln)
+	KA_T na;			/* containing node's address */
+	KA_T la;			/* LFS node address */
+	struct LFS_node *ln;		/* LFS node receiver */
+{
+	char tbuf[32];
+
+	if (!la || kread((KA_T)la, (char *)ln, sizeof(struct LFS_node))) {
+	    (void) sprintf(Namech, "node at %s: can't read LFS_node: %s",
+		print_kptr(na, tbuf), print_kptr(la, (char *)NULL));
+	    enter_nm(Namech);
+	    return(1);
+	}
+	return(0);
+}
+#endif	/* defined(AUSPEXV) */
+
+
 #if	defined(solaris)
 /*
  * read_nln(na, la, ln) - read node's loopback node
@@ -3114,11 +3569,19 @@ vop2ty(vp)
 		return(N_TMP);
 	else if (Auvops && Auvops == (KA_T)vp->v_op)
 		return(N_AUTO);
+
+#if	defined(AUSPEXV)
+	else if (LFSvops && LFSvops == (KA_T)vp->v_op)
+		return(N_AUSX);
+#endif	/* defined(AUSPEXV) */
+
 	else if (Hvops && Hvops == (KA_T)vp->v_op)
 		return(N_HSFS);
 	else if ((Pdvops && Pdvops == (KA_T)vp->v_op)
 	     ||  (Pfvops && Pfvops == (KA_T)vp->v_op))
 		return(N_PCFS);
+	else if (Mntops && Mntops == (KA_T)vp->v_op)
+		return(N_MNT);
 	else if (Mvops && Mvops == (KA_T)vp->v_op)
 		return(N_MVFS);
 	else if (Cvops && Cvops == (KA_T)vp->v_op)
