@@ -35,7 +35,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode2.c,v 1.8 98/04/21 10:04:29 abe Exp $";
+static char *rcsid = "$Id: dnode2.c,v 1.11 99/05/04 09:04:03 abe Exp $";
 #endif
 
 
@@ -52,10 +52,14 @@ static char *rcsid = "$Id: dnode2.c,v 1.8 98/04/21 10:04:29 abe Exp $";
 #include <sys/fs/vx_sol.h>
 # endif	/* defined(HSVXFS_SOL_H) */
 
+# if	defined(HASVXFS_SOLARIS_H) && defined(HASVXFS_U64_T)
+#include <sys/fs/vx_solaris.h>
+# endif	/* defined(HASVXFS_SOLARIS_H) && defined(HASVXFS_U64_T) */
+
 # if	defined(HASVXFS_MACHDEP_H)
-#  if	defined(HASVXFS_OFF32_T) && solaris>=20700
+#  if	defined(HASVXFS_OFF32_T) && solaris>=70000
 #define	off32_t	VXFS_off32_t
-#  endif	/* defined(HASVXFS_OFF32_T) && solaris>=20700 */
+#  endif	/* defined(HASVXFS_OFF32_T) && solaris>=70000 */
 #include <sys/fs/vx_machdep.h>
 # endif	/* defined(HASVXFS_MACHDEP_H) */
 
@@ -70,7 +74,10 @@ struct kdm_vnode {			/* dummy for <sys/fs/vx_inode.h> */
 #define	off64_t		VXFS_off64_t
 #  endif	/* defined(HASVXFS_OFF64_T) */
 
+# if	defined(HASVXFS_SOLARIS_H) && !defined(HASVXFS_U64_T)
 #include <sys/fs/vx_solaris.h>
+# endif	/* defined(HASVXFS_SOLARIS_H) && !defined(HASVXFS_U64_T) */
+
 #include <sys/fs/vx_layout.h>
 #include <sys/fs/vx_const.h>
 #include <sys/fs/vx_mlink.h>
@@ -84,12 +91,13 @@ struct kdm_vnode {			/* dummy for <sys/fs/vx_inode.h> */
  */
 
 int
-read_vxnode(va, v, vfs, dev, ino, sz)
+read_vxnode(va, v, vfs, dev, ino, nl, sz)
 	KA_T va;			/* containing vnode's address */
 	struct vnode *v;		/* containing vnode */
 	struct l_vfs *vfs;		/* local vfs structure */
 	dev_t *dev;			/* device number receiver */
 	unsigned long *ino;		/* inode number receiver */
+	long *nl;			/* number of links receiver */
 	unsigned long *sz;		/* size receiver */
 {
 	char tbuf[32];
@@ -120,6 +128,10 @@ read_vxnode(va, v, vfs, dev, ino, sz)
 	    *dev = (dev_t)vx.i_dev;
 	*ino = (unsigned long)vx.i_number;
 	*sz = (unsigned long)vx.i_size;
+/*
+ * Return link count.
+ */
+	*nl = (long)vx.i_nlink;
 	return(0);
 }
 #endif	/* defined(HASVXFS) */
