@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.10 98/08/18 08:07:56 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.16 99/10/22 08:23:48 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -101,10 +101,12 @@ close_kvm()
 void
 gather_proc_info()
 {
+	MALLOC_S bc;
 	int ft = 1;
 	int i, j;
 	struct proc *p;
 	int pgrp, pid, px;
+	long pofv;
 	short pss, sf;
 	struct user *u;
 	uid_t uid;
@@ -112,7 +114,17 @@ gather_proc_info()
 #if	defined(solaris)
 # if	solaris>=20400
 	int k;
-	uf_entry_t uf[NFPCHUNK];
+
+#  if	!defined(NFPCHUNK)
+#define	uf_ofile	uf_file
+#define	uf_pofile	uf_flag
+#define	u_flist		u_finfo.fi_list
+#define	u_nofiles	u_finfo.fi_nfiles
+#define	NFPREAD		64
+#  else	/* defined(NFPCHUNK) */
+#define	NFPREAD		NFPCHUNK
+#  endif	/* !defined(NFPCHUNK) */
+	uf_entry_t uf[NFPREAD];
 # endif	/* solaris>=20400 */
 # if	solaris>=20500
 	struct cred pc;
@@ -121,11 +133,16 @@ gather_proc_info()
 	struct ucwd cd;
 	int dl, nf;
 	struct file **uf;
-	unsigned ui;
 	static int xnf = 0;
 	struct file **xuf = (struct file **)NULL;
 #endif	/* solaris */
 
+#if	defined(HASFSTRUCT)
+	static int npofb = 0;
+	char *pofp;
+	static char *pofb = (char *)NULL;
+#endif	/* defined(HASFSTRUCT) */
+
 /*
  * Do first-time only operations.
  */
@@ -282,7 +299,7 @@ gather_proc_info()
 #if	defined(solaris)
 	/*
 	 * Under Solaris the file pointers are stored in dynamically-linked
-	 * ufchunk structures, each containing NFPCHUNK file pointers.  The
+	 * ufchunk structures, each containing NFPREAD file pointers.  The
 	 * first ufchunk structure is in the user area.
 	 *
 	 * Under Solaris 2.4 the file pointers are in a dynamically allocated,
@@ -300,11 +317,11 @@ gather_proc_info()
 		}
 		if (!u->u_flist.uf_ofile[j-1])
 # else	/* solaris>=20400 */
-	    for (i = 0, j = NFPCHUNK; i < u->u_nofiles; i++) {
-		if (++j > NFPCHUNK) {
+	    for (i = 0, j = NFPREAD; i < u->u_nofiles; i++) {
+		if (++j > NFPREAD) {
 		    k = u->u_nofiles - i;
-		    if (k > NFPCHUNK)
-			k = NFPCHUNK;
+		    if (k > NFPREAD)
+			k = NFPREAD;
 		    if (kread((KA_T)((unsigned long)u->u_flist +
 				     i * sizeof(uf_entry_t)),
 				     (char*)&uf, k * sizeof(uf_entry_t)))
@@ -327,48 +344,81 @@ gather_proc_info()
 		+ (KA_T)p->p_uarea))
 	    {
 		nf = NOFILE_IN_U;
+		pofp = &u->u_pofile_arr[0];
 		uf = &u->u_ofile_arr[0];
 	    } else {
 		nf = u->u_lastfile + 1;
-		ui = nf * sizeof(struct file *);
-		if (!xuf) {
-		    xnf = nf;
-		    if (!(xuf = (struct file **)malloc(ui))) {
-			(void) fprintf(stderr, "%s: PID %d, malloc file[]\n",
-			    Pn, Lp->pid);
-			Exit(1);
-		    }
-		} else if (nf > xnf) {
-		    xnf = nf;
-		    if (!(xuf = (struct file **)realloc((MALLOC_P *)xuf, ui))) {
-			(void) fprintf(stderr, "%s: PID %d, realloc file[]\n",
-			     Pn, Lp->pid);
+		bc = (MALLOC_S)(nf * sizeof(struct file *));
+		if (bc >xnf) {
+		    if (!xuf)
+			xuf = (struct file **)malloc(bc);
+		    else
+			xuf = (struct file **)realloc((MALLOC_P *)xuf, bc);
+		    if (!xuf) {
+			(void) fprintf(stderr,
+			    "%s: PID %d: no file * space\n", Pn, Lp->pid);
 			Exit(1);
 		    }
+		    xnf = bc;
 		}
-		if (kread((KA_T)u->u_ofile, (char *)xuf, (int)ui))
+		if (kread((KA_T)u->u_ofile, (char *)xuf, (int)bc))
 		    nf = 0;
 		uf = xuf;
+
+# if	defined(HASFSTRUCT)
+		if (Fsv & FSV_FG) {
+		    bc = (MALLOC_S)nf;
+		    if (bc > npofb) {
+			if (!pofb)
+			    pofb = (char *)malloc(bc);
+			else
+			    pofb = (char *)realloc((MALLOC_P *)pofb, bc);
+			if (!pofb) {
+			    (void) fprintf(stderr,
+				"%s: PID %d: no pofile space\n", Pn, Lp->pid);
+			    Exit(1);
+			}
+			npofb = bc;
+		    }
+		    if (!u->u_pofile
+		    ||  kread((KA_T)u->u_pofile, pofb, bc))
+			zeromem(pofb, bc);
+		    pofp = pofb;
+		} else
+		    pofp = (char *)NULL;
+# endif	/* defined(HASFSTRUCT) */
+
 	    }
 	    *Cwd = *Rtd = '\0';
 	    for (i = 0; i < nf; i++) {
 		if (!uf[i])
 #endif	/* solaris */
+
 		    continue;
 		alloc_lfile((char *)NULL, i);
 
 #if	defined(solaris)
 # if	solaris<20400
+		pofv = (long)u->u_flist.uf_pofile[j-1];
 		process_file((KA_T)u->u_flist.uf_ofile[j-1]);
 # else	/* solaris>=20400 */
+		pofv = uf[j-1].uf_pofile;
 		process_file((KA_T)uf[j-1].uf_ofile);
 # endif	/* solaris <20400 */
 #else	/* !solaris */
+		pofv = pofp ? pofp[i] : 0l;
 		process_file((KA_T)uf[i]);
 #endif	/* solaris */
 
-		if (Lf->sf)
+		if (Lf->sf) {
+
+#if	defined(HASFSTRUCT)
+		    if (Fsv & FSV_FG)
+			Lf->pof = pofv;
+#endif	/* defined(HASFSTRUCT) */
+
 		    link_lfile();
+		}
 	    }
 	/*
 	 * Examine results.
@@ -407,20 +457,20 @@ get_kernel_access()
 
 	);
 
-#if	defined(solaris) && solaris>=20700
+#if	defined(solaris) && solaris>=70000
 /*
- * Compare the lsof compilation bit size with the kernel bit size.
+ * Compare the Solaris 7 lsof compilation bit size with the kernel bit size.
  * Quit on a mismatch.
  */
 	{
 	    char *cp, isa[1024];
 	    short kbits = 32;
 
-#  if	defined(_LP64)
-	    int xkbits = 64;
-#  else	/* !defined(_LP64) */
+# if	defined(_LP64)
+	    short xkbits = 64;
+# else	/* !defined(_LP64) */
 	    short xkbits = 32;
-#  endif	/* defined(_LP64) */
+# endif	/* defined(_LP64) */
 
 	    if (sysinfo(SI_ISALIST, isa, (long)sizeof(isa)) < 0) {
 		(void) fprintf(stderr, "%s: can't get ISA list: %s\n",
@@ -439,14 +489,14 @@ get_kernel_access()
 	    if (kbits != xkbits) {
 		(void) fprintf(stderr,
 		    "%s: FATAL: lsof was compiled for a %d bit kernel,\n",
-		    Pn, xkbits);
+		    Pn, (int)xkbits);
 		(void) fprintf(stderr,
 		    "      but this machine has booted a %d bit kernel.\n",
-		    kbits);
+		    (int)kbits);
 		Exit(1);
 	    }
 	}
-#endif	/* defined(solaris) && solaris>=20700 */
+#endif	/* defined(solaris) && solaris>=70000 */
 
 /*
  * Get kernel symbols.
@@ -727,11 +777,7 @@ process_text(pa)
 	struct as as;
 	int i, j, k, l;
 	struct seg s;
-	union segdata {
-	    struct segvn_data vn;
-	    struct segdev_data dev;
-	    struct segmap_data map;
-	} sd;
+	struct segvn_data vn;
 	KA_T v[MAXSEGS];
 /*
  * Get address space description.
@@ -756,9 +802,9 @@ process_text(pa)
 	    ||  kread((KA_T)s.S_NEXT, (char *)&s, sizeof(s)))
 		break;
 	    if ((KA_T)s.s_ops == Sgvops && s.s_data) {
-		if (kread((KA_T)s.s_data, (char *)&sd, sizeof(sd)))
+		if (kread((KA_T)s.s_data, (char *)&vn, sizeof(vn)))
 		    break;
-		if (sd.vn.vp) {
+		if (vn.vp) {
 			
 		/*
 		 * This is a virtual node segment.
@@ -767,7 +813,7 @@ process_text(pa)
 		 * print its information.
 		 */
 		    for (l = 0; l < k; l++) {
-			if (v[l] == (KA_T)sd.vn.vp)
+			if (v[l] == (KA_T)vn.vp)
 			    break;
 		    }
 		    if (l >= k) {
@@ -777,12 +823,12 @@ process_text(pa)
 			FILEPTR = (struct file *)NULL;
 #endif	/* defined(FILEPTR) */
 
-			process_node((KA_T)sd.vn.vp);
+			process_node((KA_T)vn.vp);
 			if (Lf->sf) {
 			    link_lfile();
 			    i++;
 			}
-			v[k++] = (KA_T)sd.vn.vp;
+			v[k++] = (KA_T)vn.vp;
 		    }
 		}
 	    }
