@@ -3,7 +3,6 @@
  */
 
 
-
 /*
  * Copyright 1994 Purdue Research Foundation, West Lafayette, Indiana
  * 47907.  All rights reserved.
@@ -33,12 +32,42 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.10 98/08/03 07:37:35 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.16 99/11/28 06:44:44 abe Exp $";
 #endif
 
 
 #include "lsof.h"
 
+#if	defined(solaris)
+# if	defined(HASIPv6)
+
+/*
+ * IPv6_2_IPv4()  -- macro to define the address of an IPv4 address contained
+ *		     in an IPv6 address
+ */
+
+#define IPv6_2_IPv4(v6)	(((uint8_t *)((struct in6_addr *)v6)->s6_addr)+12)
+
+/*
+ * IPv_ADDR_UNSPEC() -- macro to test an IP[46] address for an unspecified
+ *			address value
+ */
+
+#define IPv_ADDR_UNSPEC(af, p) \
+    (((af) == AF_INET6) ? (IN6_IS_ADDR_UNSPECIFIED((struct in6_addr *)p)) \
+			: (((struct in_addr *)(p))->s_addr == INADDR_ANY))
+# else	/* !defined(HASIPv6) */
+
+/*
+ * IPv_ADDR_UNSPEC() -- IPv4-only form of macro to test for an unspecified
+ *			address value
+ */
+
+#define	IPv_ADDR_UNSPEC(af, p) (((struct in_addr *)(p))->s_addr == INADDR_ANY)
+
+# endif	/* !defined(HASIPv6) */
+#endif	/* defined(solaris) */
+
 
 #if	defined(solaris)
 /*
@@ -51,12 +80,12 @@ print_tcptpi(nl)
 {
 	char *cp = (char *)NULL;
 	char  sbuf[128];
-	int i, t;
+	int i;
 	int ps = 0;
 	unsigned int u;
 
 	if (Ftcptpi & TCPTPI_STATE) {
-	    switch ((t = Lf->lts.type)) {
+	    switch (Lf->lts.type) {
 	    case 0:				/* TCP */
 		switch ((i = Lf->lts.state.i)) {
 		case TCPS_CLOSED:
@@ -244,6 +273,8 @@ process_socket(sa, ty)
 	KA_T sa;			/* stream's data address in kernel */
 	char *ty;			/* socket type name */
 {
+	int af;
+	char *cp;
 	unsigned char *fa = (unsigned char *)NULL;
 	int fp = 0;
 	int i, lp;
@@ -259,6 +290,7 @@ process_socket(sa, ty)
 	KA_T qp;
 	u_short *s;
 	struct stdata sd;
+	unsigned char *ta;
 	char tbuf[32];
 
 # if	solaris<20600
@@ -319,13 +351,17 @@ process_socket(sa, ty)
 #   if	solaris<20500
 	    u_long d24[67];
 #   else	/* solaris>=20500 */
+#    if	solaris<20501
+	    u_long d25[6];
+#    else	/* solaris>=20501 */
 	    u_long d25[8];
+#    endif	/* solaris<20501 */
 	    u_long tcp_rack;	/* Seq # we have acked */
 #    if	solaris<20501
 	    u_long d26[29];
 #    else	/* solaris>=20501 */
 	    u_long d26[33];
-#    endif	/* solaris<20501 */
+#    endif	/* solaris>=20501 */
 #   endif	/* solaris<20500 */
 #  endif	/* solaris<20400 */
 
@@ -335,6 +371,10 @@ process_socket(sa, ty)
 	struct tcp_s tc;
 # endif	/* solaris<20600 */
 
+# if	solaris>=80000
+	tcpb_t	tcb;
+# endif	/* solaris>=80000 */
+
 	int tcs = 0;
 	tcph_t th;
 	struct ud_s {			/* should come from kernel source
@@ -345,16 +385,28 @@ process_socket(sa, ty)
 	    unsigned char udp_src[4];	/* source address of this stream */
 	} uc;
 	int ucs = 0;
-
 	if (Fnet)
 	    Lf->sf |= SELNET;
+
+# if	defined(HASIPv6)
+	if (strrchr(ty, '6')) {
+	    (void) strcpy(Lf->type, "IPv6");
+	    af = AF_INET6;
+	} else {
+	    (void) strcpy(Lf->type, "IPv4");
+	    af = AF_INET;
+	}
+# else	/* !defined(HASIPv6) */
 	(void) strcpy(Lf->type, "inet");
+	af = AF_INET;
+# endif	/* defined(HASIPv6) */
+
 	Lf->inp_ty = 2;
 /*
  * Convert type to upper case protocol name.
  */
 	if (ty) {
-	    for (i = 0; ty[i] != '\0' && i < IPROTOL; i++) {
+	    for (i = 0; (ty[i] != '\0') && (i < IPROTOL) && (i < 3); i++) {
 		if (islower((unsigned char)ty[i]))
 		    Lf->iproto[i] = toupper((unsigned char)ty[i]);
 		else
@@ -391,7 +443,18 @@ process_socket(sa, ty)
 	    }
 	    if (strncasecmp(tbuf, "TCP", 3) == 0) {
 		if (kread((KA_T)q.q_ptr, (char *)&tc, sizeof(tc)) == 0)
+
+# if	solaris>=80000
+		{
+		    if (tc.tcp_base
+		    &&  !kread((KA_T)tc.tcp_base, (char *)&tcb, sizeof(tcb)))
+			tcs = 1;
+		    tc.tcp_base = &tcb;		/* support for macroes */
+		    tcb.tcpb_tcp = &tc;		/* support for macroes */
+		}
+# else	/* solaris<80000 */
 		    tcs = 1;
+# endif	/* solaris>=80000 */
 		continue;
 	    }
 	    if (strncasecmp(tbuf, "UDP", 3) == 0) {
@@ -407,7 +470,7 @@ process_socket(sa, ty)
 	 */
 	    if (pcb)
 		enter_dev_ch(print_kptr(pcb, (char *)NULL));
-	    if (strcmp(Lf->iproto, "UDP") == 0) {
+	    if (strncmp(Lf->iproto, "UDP", 3) == 0) {
 
 	/*
 	 * Save UDP address and TPI state.
@@ -417,13 +480,17 @@ process_socket(sa, ty)
 		la = (unsigned char *)&ic.ipc_udp_addr;
 		p = (u_short)ic.ipc_udp_port;
 # else	/* solaris>=20600 */
+#  if	defined(HASIPv6)
+		la = (af == AF_INET6) ? (unsigned char *)&ic.ipc_v6laddr
+		   :  (unsigned char *)IPv6_2_IPv4(&ic.ipc_v6laddr);
+#  else	/* !defined(HASIPv6 */
 		la = (unsigned char *)&ic.ipc_laddr;
+#  endif	/* defined(HASIPv6) */
+
 		p = (u_short)ic.ipc_lport;
 # endif	/* solaris<20600 */
 
-		if (((struct in_addr *)la)->s_addr == INADDR_ANY
-		&&  p == 0 && ucs)
-		{
+		if (IPv_ADDR_UNSPEC(af, la) && !p && ucs) {
 
 		/*
 		 * If the ipc_s structure has no local address, use
@@ -432,15 +499,32 @@ process_socket(sa, ty)
 		    s = (u_short *)&uc.udp_port[0];
 		    p = *s;
 		}
+
+# if	defined(HASIPv6)
+		if ((af == AF_INET6) && la
+		&&  IN6_IS_ADDR_V4MAPPED((struct in6_addr *)la)) {
+
+		/*
+		 * Convert a local IPv4 address in an IPv6 structure to an IPv4
+		 * address in an IPv4 structure.  Change the address family to
+		 * AF_INET and file type type to IPv4.
+		 */
+		    la = (unsigned char *)IPv6_2_IPv4(la);
+		    if (Lf->type[3] == '6')
+			Lf->type[3] = '4';
+		    af = AF_INET;
+		}
+# endif	/* defined(HASIPv6) */
+
 		(void) ent_inaddr(la, (int)ntohs(p), (unsigned char *)NULL,
-				  -1, AF_INET);
+				  -1, af);
 		if (!Fsize)
 		    Lf->off_def = 1;
 		if (ucs) {
 		    Lf->lts.type = 1;
 		    Lf->lts.state.u = (unsigned int)uc.udp_state;
 		}
-	    } else if (strcmp(Lf->iproto, "TCP") == 0) {
+	    } else if (strncmp(Lf->iproto, "TCP", 3) == 0) {
 
 	    /*
 	     * Save TCP address.
@@ -454,24 +538,38 @@ process_socket(sa, ty)
 		la = (unsigned char *)&ic.ipc_tcp_laddr;
 		p = (u_short)((short *)&ic.ipc_tcp_ports)[1];
 #  else	/* solaris>=20600 */
+#   if	defined(HASIPv6)
+		la = (af == AF_INET6) ? (unsigned char *)&ic.ipc_v6laddr
+		   :  (unsigned char *)IPv6_2_IPv4(&ic.ipc_v6laddr);
+#else		/* !defined(HASIPv6 */
 		la = (unsigned char *)&ic.ipc_laddr;
+#   endif	/* defined(HASIPv6) */
+
 		p = (u_short)ic.ipc_lport;
 #  endif	/* solaris<20600 */
 # endif	/* solaris<20400 */
 
-		if (((struct in_addr *)la)->s_addr == INADDR_ANY
-		&&  p == 0 && tcs)
-		{
+		if (IPv_ADDR_UNSPEC(af, la) && !p && tcs) {
 
 		/*
 		 * If the ipc_s structure has no local address, use the local
-		 * address in its tcp_iph structure (except for Solaris 2.4),
-		 * and the port number in its tcph structure.
+		 * address in the stream's tcp_iph structure (except for
+		 * Solaris 2.4), and the port number in the stream's tcph
+		 * structure.
 		 */
 
-# if	solaris!=20400
+# if	solaris!=20400 && solaris<80000
 		    la = (unsigned char *)&tc.tcp_iph.iph_src[0];
-# endif	/* solaris!=20400 */
+# else	/* solaris==20400 || solaris<80000 */
+#  if	solaris>=80000
+#   if	defined(HASIPv6)
+		    la = (af == AF_INET6) ? (unsigned char *)&tcb.tcpb_ip_src_v6
+		       :  (unsigned char *)IPv6_2_IPv4(&tcb.tcpb_ip_src_v6);
+#   else	/* !defined(HASIPv6) */
+		    la = (unsigned char *)&tcb.tcpb_ip_src;
+#   endif	/* defined(HASIPv6) */
+#  endif	/* solaris>=80000 */
+# endif	/* solaris!=20400 && !defined(HASIPv6) */
 
 		    if (tc.tcp_hdr_len && tc.tcp_tcph
 		    &&  !kread((KA_T)tc.tcp_tcph, (char *)&th, sizeof(th))) {
@@ -483,29 +581,57 @@ process_socket(sa, ty)
 
 # if	solaris<20400
 		if ((int)ic.ipc_tcp_addr[2] != INADDR_ANY
-		|| ic.ipc_tcp_addr[4] != 0) {
+		||  ic.ipc_tcp_addr[4] != 0)
+		{
 		    fa = (unsigned char *)&ic.ipc_tcp_addr[2];
 		    fp = (int)ntohs(ic.ipc_tcp_addr[4]);
 		}
 # else	/* solaris>=20400 */
 #  if	solaris<20600
 		if ((int)ic.ipc_tcp_faddr != INADDR_ANY
-		|| ((u_short *) &ic.ipc_tcp_ports)[0] != 0)
+		||  ((u_short *) &ic.ipc_tcp_ports)[0] != 0)
 		{
 		    fa = (unsigned char *)&ic.ipc_tcp_faddr;
 		    fp = (int)ntohs(((u_short *)&ic.ipc_tcp_ports)[0]);
 		}
 #  else	/* solaris>=20600 */
-		if ((int)ic.ipc_faddr != INADDR_ANY || ((u_short)ic.ipc_fport))
-		{
-		    fa = (unsigned char *)&ic.ipc_faddr;
+
+#   if	defined(HASIPv6)
+		ta = (af == AF_INET6) ? (unsigned char *)&ic.ipc_v6faddr
+		   :  (unsigned char *)IPv6_2_IPv4(&ic.ipc_v6faddr);
+#   else	/* !defined(HASIPv6) */
+		ta = (unsigned char *)&ic.ipc_faddr;
+#   endif	/* defined(HASIPv6) */
+
+		if (!IPv_ADDR_UNSPEC(af, ta) || ((u_short)ic.ipc_fport)) {
+		    fa = ta;
 		    fp = (int)ntohs(((u_short)ic.ipc_fport));
 		}
 #  endif	/* solaris<20600 */
 # endif	/* solaris <20400 */
 
+# if	defined(HASIPv6)
+		if ((af == AF_INET6)
+		&&  ((la && IN6_IS_ADDR_V4MAPPED((struct in6_addr *)la))
+		||  ((fa && IN6_IS_ADDR_V4MAPPED((struct in6_addr *)fa))))) {
+
+		/*
+		 * Convert IPv4 addresses in IPv6 structures to IPv4 addresses
+		 * in IPv4 structures.  Change the address family to AF_INET
+		 * and the file type to IPv4.
+		 */
+		    if (la)
+			la = (unsigned char *)IPv6_2_IPv4(la);
+		    if (fa)
+			fa = (unsigned char *)IPv6_2_IPv4(fa);
+		    if (Lf->type[3] == '6')
+			Lf->type[3] = '4';
+		    af = AF_INET;
+		}
+# endif	/* defined(HASIPv6) */
+
 		if (fa || la)
-		    (void) ent_inaddr(la, lp, fa, fp, AF_INET);
+		    (void) ent_inaddr(la, lp, fa, fp, af);
 	    /*
 	     * Save TCP state information.
 	     */
@@ -782,12 +908,13 @@ process_socket(sa)
 		*((char *)ua + mb.m_len) = '\0';
 		if (Sfile && is_file_named(ua->sun_path, Ntype, VSOCK, 0))
 		    Lf->sf |= SELNM;
-		(void) strcpy(Namech, ua->sun_path);
+		if (!Namech[0])
+		    (void) strcpy(Namech, ua->sun_path);
 	    } else
 		(void) strcpy(Namech, "no address");
 	    break;
 	default:
-	    printunkaf(fam);
+	    printunkaf(fam, 1);
 	}
 	if (Namech[0])
 	    enter_nm(Namech);
