@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dstore.c,v 1.5 98/06/19 08:32:11 abe Exp $";
+static char *rcsid = "$Id: dstore.c,v 1.10 99/11/28 06:44:48 abe Exp $";
 #endif
 
 
@@ -91,6 +91,7 @@ struct drive_Nl Drive_Nl[] = {
 	{ "fvops",	"fifo_vnodeops"		},
 	{ "hvops",	"hsfs_vnodeops"		},
 	{ "lvops",	"lo_vnodeops"		},
+	{ "mntops",	"mntvnodeops"		},
 	{ "mvops",	"mvfs_vnodeops"		},
 	{ X_NCACHE,	"ncache"		},
 	{ X_NCSIZE,	"ncsize"		},
@@ -118,6 +119,7 @@ struct drive_Nl Drive_Nl[] = {
 	{ "avops",	"_afs_vnodeops"		},
 	{ "avol",	"_afs_volumes"		},
 	{ "fvops",	"_fifo_vnodeops"	},
+        { "lfsvops",	"_lfs_vnodeops"		},
         { "hvops",	"_hsfs_vnodeops"	},
 	{ X_NCACHE,	"_ncache"		},
 	{ X_NCSIZE,	"_ncsize"		},
@@ -153,11 +155,81 @@ struct l_vfs *Lvfs = NULL;		/* local vfs structure table */
 # if	defined(solaris)
 struct netclone *Netclone = NULL;	/* net clone devices from
 					 * /devices/pseudo */
-# endif
+# endif	/* defined(solaris) */
+
+# if	defined(HASFSTRUCT)
+/*
+ * Pff_tab[] - table for printing file flags
+ */
+struct pff_tab Pff_tab[] = {
+#  if	defined(solaris)
+	{ (long)FREAD,		FF_READ		},
+	{ (long)FWRITE,		FF_WRITE	},
+	{ (long)FNDELAY,	FF_NDELAY	},
+	{ (long)FAPPEND,	FF_APPEND	},
+	{ (long)FSYNC,		FF_SYNC		},
+
+#   if	defined(FREVOKED)
+	{ (long)FREVOKED,	FF_REVOKED	},
+#   endif	/* defined(FREVOKED) */
+
+	{ (long)FDSYNC,		FF_DSYNC	},
+	{ (long)FRSYNC,		FF_RSYNC	},
+
+#   if	defined(FOFFMAX)
+	{ (long)FOFFMAX,	FF_LARGEFILE	},
+#   endif	/* defined(FFOFFMAX) */
+
+	{ (long)FNONBLOCK,	FF_NBLOCK	},
+	{ (long)FNOCTTY,	FF_NOCTTY	},
+	{ (long)FASYNC,		FF_ASYNC	},
+	{ (long)FNODSYNC,	FF_NODSYNC	},
+#  else	/* !defined(solaris) */
+	{ (long)_FREAD,		FF_READ		},
+	{ (long)_FWRITE,	FF_WRITE	},
+	{ (long)_FNDELAY,	FF_NDELAY	},
+	{ (long)_FAPPEND,	FF_APPEND	},
+	{ (long)_FSETBLK,	FF_SETBLK	},
+	{ (long)_FASYNC,	FF_ASYNC	},
+	{ (long)_FSHLOCK,	FF_SHLOCK	},
+	{ (long)_FEXLOCK,	FF_EXLOCK	},
+	{ (long)_FNBIO,		FF_NBIO		},
+	{ (long)_FSYNC,		FF_SYNC		},
+	{ (long)_FNONBLOCK,	FF_NBLOCK	},
+	{ (long)_FNOCTTY,	FF_NOCTTY	},
+	{ (long)_FMARK,		FF_MARK		},
+	{ (long)_FDEFER,	FF_DEFER	},
+#  endif	/* defined(solaris) */
+
+	{ (long)0,		NULL		}
+};
+
+
+/*
+ * Pof_tab[] - table for print process open file flags
+ */
+
+struct pff_tab Pof_tab[] = {
+
+# if	defined(UF_EXCLOSE)
+	{ (long)UF_EXCLOSE,	POF_CLOEXEC	},
+# endif	/* defined(UF_EXCLOSE) */
+
+# if	defined(FD_CLOEXEC)
+	{ (long)FD_CLOEXEC,	POF_CLOEXEC	},
+# endif	/* defined(FD_CLOEXEC) */
+
+# if	defined(UF_FDLOCK)
+	{ (long)UF_FDLOCK,	POF_FDLOCK	},
+# endif	/* defined(UF_FDLOCK) */
+
+	{ (long)0,		NULL		}
+};
+#endif	/* defined(HASFSTRUCT) */
 
 #if	defined(solaris)
 struct pseudo *Pseudo = NULL;		/* non-clone devices from
 					 * /devices/pseudo */
-#else
+#else	/* !defined(solaris) */
 char *Rtd = NULL;			/* root directory path name buffer */
-#endif
+#endif	/* defined(solaris) */
