@@ -30,8 +30,8 @@
  */
 
 
-#if	!defined(MACHINE_H)
-#define	MACHINE_H	1
+#if	!defined(LSOF_MACHINE_H)
+#define	LSOF_MACHINE_H	1
 
 
 # if	defined(solaris) && solaris>=20600
@@ -48,14 +48,32 @@
 					 * the Solaris <sys/int_types.h> */
 #include <sys/poll.h>
 
+# if	solaris>=80000
+#include <sys/wait.h>
+#define	_KERNEL	1
+#include <netinet/in.h>
+#undef	_KERNEL
+#define	ipa_32	s6_ipaddr.ipa_32
+# endif	/* solaris>=80000 */
+
 # endif	/* defined(solaris) && solaris>=20600 */
 
 
 /*
- * $Id: machine.h,v 1.15 98/06/29 15:49:44 abe Exp $
+ * $Id: machine.h,v 1.20 99/10/22 08:23:52 abe Exp $
  */
 
 
+/*
+ * CAN_USE_CLNT_CREATE is defined for those dialects where RPC clnt_create()
+ * can be used to obtain a CLIENT handle in lieu of clnttcp_create().
+ */
+
+#if	!defined(solaris) || (defined(solaris) && solaris>=20501)
+#define	CAN_USE_CLNT_CREATE	1
+#endif	/* !defined(solaris) || (defined(solaris) && solaris>=20501) */
+
+
 /*
  * GET_MAX_FD is defined for those dialects that provide a function other than
  * getdtablesize() to obtain the maximum file descriptor number plus one.
@@ -112,7 +130,7 @@
 #define	HASENVDC	"LSOFDEVCACHE"
 #define	HASPERSDC	"%h/%p.lsof_%L"
 #define	HASPERSDCPATH	"LSOFPERSDCPATH"
-/* #define	HASSYSDC	"/var/adm/lsof.dc" */
+/* #define	HASSYSDC	"/your/choice/of/path" */
 
 
 /*
@@ -137,6 +155,18 @@
 #define	HASFSINO	1
 
 
+/*
+ * HASFSTRUCT is defined if the dialect has a file structure.
+ *
+ * FSV_DEFAULT defines the default set of file structure values to list.
+ * It defaults to zero (0), but may be made up of a combination of the
+ * FSV_* symbols from lsof.h.
+ */
+
+#define	HASFSTRUCT	1
+/* #define	FSV_DEFAULT	FSV_? | FSV_? | FSV_? */
+
+
 /*
  * HASGNODE is defined for those dialects that have gnodes.
  */
@@ -192,21 +222,10 @@
  * alloc_lfile() function of proc.c, the SETLFILEADD macro may be defined
  * to do that.  The HASXOPT definition may be used to select the conditions
  * under which private lfile elements are used.
- *
- * Field identifiers 1 through 9 are reserved for dialect-specific field
- * output.  Define the string HASFIELDAP<x> as the help (-F?) name of the
- * private field and LISTLFIELAP<x> as a macro to list the field output.
- * (<x> is the field identifier character.)  HASFIELDAP<x> is used in proc.c
- * and store.c; LISTFIELDAP<x>, in proc.c.
- *
- * See ../dialectx/ptx for an example of the use of HASLFILEADD, SETLFILEADD,
- * and HASXOPT.
  */
 
-/* #define HASLFILEADD <macro of new element definitions>; */
-/* #define SETLFILEADD <macro of code to preset new elements>; */
-/* #define HASFIELDAP1 <string name of field for -F? output> */
-/* #define LISTLFILEAP1 <macro to list field output>; */
+/* #define HASLFILEADD int ... */
+/* #define SETLFILEADD Lf->... */
 
 
 /*
@@ -245,7 +264,7 @@
  * NOTE: don't forget to define a prototype for this function in dproto.h.
  */
 
-/* #define	HASPIPEFN	process_pipe */
+/* #define	HASPIPEFN	process_pipe? */
 
 
 /*
@@ -284,9 +303,9 @@
 #define	HASPRINTINO	print_ino
 #endif	/* defined(solaris) */
 
-/* #define	HASPRINTNM	print_nm	*/
-/* #define	HASPRINTOFF	print_off	*/
-/* #define	HASPRINTSZ	print_sz	*/
+/* #define	HASPRINTNM	print_nm?	*/
+/* #define	HASPRINTOFF	print_off?	*/
+/* #define	HASPRINTSZ	print_sz?	*/
 
 
 /*
@@ -299,7 +318,7 @@
  * HASPRIVFILETYPE is the name of the processing function.
  */
 
-/* #define	HASPRIVFILETYPE	process_shmf	*/
+/* #define	HASPRIVFILETYPE	process_shmf?	*/
 /* #define	PRIVFILETYPE	??	*/
 
 
@@ -489,4 +508,4 @@
 #define	zeromem(a, l)		memset((char *)a, 0, l)
 # endif	/* defined(solaris) */
 
-#endif	/* !defined(MACHINE_H) */
+#endif	/* !defined(LSOF_MACHINE_H) */
