@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dstore.c,v 1.2 97/04/16 07:56:31 abe Exp $";
+static char *rcsid = "$Id: dstore.c,v 1.5 99/05/09 14:57:08 abe Exp $";
 #endif
 
 
@@ -87,4 +87,59 @@ struct drive_Nl Drive_Nl[] = {
 struct file *Fileptr;			/* for process_file() in lib/prfp.c */
 int Kd = -1;				/* /dev/kmem file descriptor */
 int Mem = -1;				/* /dev/mem file descriptor */
+
+#if	defined(HASFSTRUCT)
+/*
+ * Pff_tab[] - table for printing file flags
+ */
+
+struct pff_tab Pff_tab[] = {
+	{ (long)FREAD,		FF_READ		},
+	{ (long)FWRITE,		FF_WRITE	},
+	{ (long)FNBLOCK,	FF_NBLOCK	},
+	{ (long)FNDELAY,	FF_NDELAY	},
+	{ (long)FAPPEND,	FF_APPEND	},
+	{ (long)FMARK,		FF_MARK		},
+	{ (long)FDEFER,		FF_DEFER	},
+	{ (long)FASYNC,		FF_ASYNC	},
+	{ (long)FDEFER,		FF_DEFER	},
+	{ (long)FSHLOCK,	FF_SHLOCK	},
+	{ (long)FEXLOCK,	FF_EXLOCK	},
+	{ (long)FBLKINUSE,	FF_BLKINUSE	},
+	{ (long)FBLKANDSET,	FF_BLKANDSET	},
+	{ (long)FSYNCRON,	FF_SYNCRON	},
+	{ (long)FNBUF,		FF_NBUF		},
+	{ (long)FNBLOCK,	FF_NBLOCK	},
+	{ (long)FTERMIO,	FF_TERMIO	},
+	{ (long)FNOCTTY,	FF_NOCTTY	},
+	{ (long)0,		NULL		}
+};
+
+
+/*
+ * Pof_tab[] - table for print process open file flags
+ */
+
+struct pff_tab Pof_tab[] = {
+
+# if	defined(UF_EXCLOSE)
+	{ (long)UF_EXCLOSE,	POF_CLOEXEC	},
+# endif	/* defined(UF_EXCLOSE) */
+
+# if	defined(UF_MAPPED)
+	{ (long)UF_MAPPED,	POF_MAPPED	},
+# endif	/* defined(UF_MAPPED) */
+
+# if	defined(UF_INUSE)
+	{ (long)UF_INUSE,	POF_INUSE	},
+# endif	/* defined(UF_INUSE) */
+
+# if	defined(UF_FDLOCK)
+	{ (long)UF_FDLOCK,	POF_FDLOCK	},
+# endif	/* defined(UF_FDLOCK) */
+
+	{ (long)0,		NULL		}
+};
+#endif	/* defined(HASFSTRUCT) */
+
 int Swap = -1;				/* swap device file descriptor */
