@@ -144,6 +144,7 @@ typedef ulong_t channel_t;		/* also in types.h #if _KERNEL */
 
 #define	COMP_P		const void
 #define DEVINCR		1024	/* device table malloc() increment */
+#define	DIRTYPE		dirent
 #define	FSNAMEL		4
 typedef	off_t		KA_T;
 #define	KMEM		"/dev/kmem"
@@ -175,11 +176,6 @@ typedef	off_t		KA_T;
  * Global storage definitions (including their structure definitions)
  */
 
-struct clone {
-	int dx;			/* index of device entry in Devtp[] */
-	struct clone *next;	/* forward link */
-};
-extern struct clone *Clone;
 extern int CloneMaj;
 extern char **Fsinfo;
 extern int Fsinfomax;
@@ -190,7 +186,9 @@ struct l_ino {
 	dev_t dev;			/* device */
 	dev_t rdev;			/* real device */
 	ino_t number;			/* inode number */
+	int nlink_def;			/* link count is defined */
 	long size;			/* file size */
+	long nlink;			/* link count */
 };
 
 struct mounts {
@@ -242,15 +240,12 @@ struct sfile {
 
 
 /*
- * Definitions for dvch.c
+ * Definition for dvch.c, isfn.c, and rdev.c
  */
 
-# if	defined(HASDCACHE)
-#define	DCACHE_CLONE	rw_clone_sect
-#define	DCACHE_CLR	clr_sect	/* function to clear clone cache
-					 * when reading the device cache
-					 * file fails */
-# endif	/* defined(HASDCACHE) */
+#define	CLONEMAJ	CloneMaj	/* clone major variable name */
+#define	HAS_STD_CLONE	1		/* has standard clone handling */
+#define	HAVECLONEMAJ	HaveCloneMaj	/* clone major status variable name */
 
 
 /*
