@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1996 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.9 98/09/14 19:55:47 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.14 99/06/22 09:52:49 abe Exp $";
 #endif
 
 
@@ -410,9 +410,14 @@ process_node(na)
             return;
         }
 
-# if    defined(HASNCACHE)
+#if	defined(HASNCACHE)
 	Lf->na = na;
-# endif /* defined(HASNCACHE) */
+#endif	/* defined(HASNCACHE) */
+
+#if	defined(HASFSTRUCT)
+	Lf->fna = na;
+	Lf->fsv |= FSV_NI;
+#endif	/* defined(HASFSTRUCT) */
 
 /*
  * Determine the vnode type.
@@ -633,15 +638,10 @@ get_lock_state:
 		if (v.v_type == VCHR) {
 	    /*
 	     * If this is a VCHR stream, look up the device name and record it.
-	     *
-	     * Because the stream is also represented by a VCHR device, record
-	     * the device inode number and clear the stream flag so the file
-	     * can be located by the device name.
 	     */
 		    if ((dp = findstrdev((dev_t *)&v.v_rdev))) {
 			Lf->inode = (unsigned long)dp->inode;
 			Lf->inp_ty = 1;
-			Lf->is_stream = 0;
 			Namech[j++] = ':';
 			k = strlen(dp->name);
 			if ((j + k) <= (MAXPATHLEN - 1)) {
@@ -872,6 +872,34 @@ get_lock_state:
 		break;
 	    }
 	}
+/*
+ * Record link count.
+ */
+	if (Fnlink) {
+	    switch(Ntype) {
+	    case N_FIFO:
+		Lf->nlink = (long)f.fn_open;
+		Lf->nlink_def = 1;
+		break;
+	    case N_NFS:
+		Lf->nlink = (long)r.r_attr.va_nlink;
+		Lf->nlink_def = 1;
+		break;
+
+#if	defined(HASPROCFS)
+	    case N_PROC:
+#endif	/* defined(HASPROCFS) */
+
+	    case N_REGLR:
+		if (!ni) {
+		    Lf->nlink = (long)i.nlink;
+		    Lf->nlink_def = i.nlink_def;
+		}
+		break;
+	    }
+	    if (Nlink && Lf->nlink_def && (Lf->nlink < Nlink))
+		Lf->sf |= SELNLINK;
+	}
 /*
  * Record an NFS file selection.
  */
@@ -967,13 +995,15 @@ get_lock_state:
 	    &&  !kread((KA_T)so.laddr.buf, (char *)&ua, sizeof(ua))) {
 		ua.sun_path[sizeof(ua.sun_path) - 1] = '\0';
 		(void) strcpy(Namech, ua.sun_path);
+		if (Sfile && is_file_named(Namech, 0))
+		    Lf->sf = SELNM;
 		if (so.lux_dev.size >= 8) {
 		    Lf->inode = (unsigned long)so.lux_dev.addr.tu_addr.ino;
 		    Lf->inp_ty = 1;
 		}
 	    }
-	    if (!Namech[0] && so.so_conn)
-		(void) sprintf(Namech, "->%s",
+	    if (so.so_conn)
+		(void) sprintf(endnm(), "->%s",
 		    print_kptr((KA_T)so.so_conn, (char *)NULL));
 	    Lf->is_stream = 0;
 	}
@@ -1010,7 +1040,7 @@ get_lock_state:
 #endif	/* defined(HASPROCFS) */
 
 	{
-	    if (Sfile && is_file_named(NULL, type))
+	    if (Sfile && is_file_named(NULL, (type == VCHR) ? 1 : 0))
 		Lf->sf |= SELNM;
 	}
 /*
@@ -1039,7 +1069,7 @@ readlino(fx, v, i)
 	struct xnamnode xn;
 #endif	/* defined(HASXNAMNODE) */
 
-
+	i->nlink_def = 0;
 	if (fx < 1 || fx > Fsinfomax || !v->v_data)
 	    return(1);
 	if (!strcmp(Fsinfo[fx-1], "fifofs")
@@ -1049,6 +1079,8 @@ readlino(fx, v, i)
 		return(1);
 	    i->dev = sn.i_dev;
 	    i->rdev = v->v_rdev;
+	    i->nlink = (long)sn.i_nlink;
+	    i->nlink_def = 1;
 	    i->number = sn.i_number;
 	    i->size = sn.i_size;
 	    return(0);
@@ -1066,6 +1098,8 @@ readlino(fx, v, i)
 	    if (kread((KA_T)v->v_data, (char *)&xn, sizeof(xn)))
 		return(1);
 	    i->dev = xn.x_dev;
+	    i->nlink = (long)xn.x_count;
+	    i->nlink_def = 1;
 	    i->rdev = xn.x_fsid;
 	    i->size = xn.x_size;
 	    return(0);
@@ -1076,6 +1110,8 @@ readlino(fx, v, i)
 	    if (kread((KA_T)v->v_data, (char *)&mn, sizeof(mn)))
 		return(1);
 	    i->dev = mn.mno_fsid;
+	    i->nlink = (long)mn.mno_nlink;
+	    i->nlink_def = 1;
 	    i->number = mn.mno_nodeid;
 	    i->rdev = mn.mno_rdev;
 	    i->size = mn.mno_size;
