@@ -34,7 +34,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1996 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode2.c,v 1.3 98/04/07 17:24:30 abe Exp $";
+static char *rcsid = "$Id: dnode2.c,v 1.4 99/05/04 09:05:07 abe Exp $";
 #endif
 
 
@@ -64,9 +64,11 @@ struct vx_inode{
 	unsigned long d1[28];
 	dev_t i_dev;
 	unsigned long i_number;
-	unsigned long d2[79];
+	unsigned long d2[76];
+	unsigned long i_nlink;
+	unsigned long d3[2];
 	unsigned long i_size;
-	unsigned long d3[9];
+	unsigned long d4[9];
 	dev_t i_rdev;
 };
 # endif	/* UNIXWAREV<70000 */
@@ -89,6 +91,8 @@ readvxfslino(v, i)
 	if (kread((KA_T)v->v_data, (char *)&vx, sizeof(vx)))
 	    return(1);
 	i->dev = vx.i_dev;
+	i->nlink = (long)vx.i_nlink;
+	i->nlink_def = 1;
 	i->number = vx.i_number;
 	i->rdev = (v->v_type == VCHR) ? vx.i_rdev : 0;
 	i->size = vx.i_size;
