@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1996 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.6 98/08/20 15:26:24 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.8 99/06/22 09:53:09 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -183,8 +183,15 @@ gather_proc_info()
 		if (f->fd_file && (f->fd_status & FD_INUSE)) {
 		    alloc_lfile(NULL, i);
 		    process_file((KA_T)f->fd_file);
-		    if (Lf->sf)
+		    if (Lf->sf) {
+
+#if	defined(HASFSTRUCT)
+			if (Fsv & FSV_FG)
+			    Lf->pof = (long)f->fd_flag;
+#endif	/* defined(HASFSTRUCT) */
+
 			link_lfile();
+		    }
 		}
 	    }
 	/*
@@ -390,11 +397,7 @@ process_text(pa)
 	struct as as;
 	int i, j, k, l;
 	struct seg s;
-	union segdata {
-	    struct segvn_data vn;
-	    struct segdev_data dev;
-	    struct segmap_data map;
-	} sd;
+	struct segvn_data vn;
 	KA_T v[MAXSEGS];
 /*
  * Get address space description.
@@ -411,9 +414,9 @@ process_text(pa)
 	    if (!s.s_next || kread((KA_T)s.s_next, (char *)&s, sizeof(s)))
 		break;
 	    if (Sgvnops == (KA_T)s.s_ops && s.s_data) {
-		if (kread((KA_T)s.s_data, (char *)&sd, sizeof(sd)))
+		if (kread((KA_T)s.s_data, (char *)&vn, sizeof(vn)))
 		    break;
-		if (sd.vn.svd_vp && sd.vn.svd_offset) {
+		if (vn.svd_vp && vn.svd_offset) {
 			
 		/*
 		 * This is a virtual node segment.
@@ -422,17 +425,17 @@ process_text(pa)
 		 * print its information.
 		 */
 		    for (l = 0; l < k; l++) {
-			if (v[l] == (KA_T)sd.vn.svd_vp)
+			if (v[l] == (KA_T)vn.svd_vp)
 			    break;
 		    }
 		    if (l >= k) {
 			alloc_lfile(" txt", -1);
-			process_node((KA_T)sd.vn.svd_vp);
+			process_node((KA_T)vn.svd_vp);
 			if (Lf->sf) {
 			    link_lfile();
 			    i++;
 			}
-			v[k++] = (KA_T)sd.vn.svd_vp;
+			v[k++] = (KA_T)vn.svd_vp;
 		    }
 		}
 	    }
