@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1996 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.7 98/05/22 07:17:01 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.8 99/06/22 09:53:44 abe Exp $";
 #endif
 
 
@@ -219,12 +219,18 @@ process_socket(pr, q)
 	struct inpcb inp;
 	unsigned char *la = (unsigned char *)NULL;
 	struct tcpcb t;
+	int tcp = 0;
 	short ts = 0;
+	int udp = 0;
 
         (void) strcpy(Lf->type, "inet");
 	Lf->inp_ty = 2;
 	Lf->is_stream = 0;
 	(void) strcpy(Lf->iproto, pr);
+	if (strcasecmp(pr, "TCP") == 0)
+	   tcp = 1;
+	else if (strcasecmp(pr, "UDP") == 0)
+	    udp = 1;
 	if (Fnet)
 	    Lf->sf |= SELNET;
 /*
@@ -233,31 +239,34 @@ process_socket(pr, q)
  */
 	if (q->q_ptr) {
 	    enter_dev_ch(print_kptr((KA_T)q->q_ptr, (char *)NULL));
-	    if (kread((KA_T)q->q_ptr, (char *)&inp, sizeof(inp))) {
-		(void) sprintf(Namech, "can't read inpcb from %s",
-		    print_kptr((KA_T)q->q_ptr, (char *)NULL));
-		enter_nm(Namech);
-		return;
-	    }
-	    la = (unsigned char *)&inp.inp_laddr;
-	    lp = (int)ntohs(inp.inp_lport);
-	    if (inp.inp_faddr.s_addr != INADDR_ANY || inp.inp_fport != 0) {
-		fa = (unsigned char *)&inp.inp_faddr;
-		fp = (int)ntohs(inp.inp_fport);
-	    }
-	    if (fa || la)
-		(void) ent_inaddr(la, lp, fa, fp, AF_INET);
-	    if (!strcasecmp(pr, "TCP")) {
-		if (inp.inp_ppcb
-		&&  !kread((KA_T)inp.inp_ppcb, (char *)&t, sizeof(t))) {
-		    ts = 1;
-		    Lf->lts.type = 0;
-		    Lf->lts.state.i = (int)t.t_state;
+	    if (tcp || udp) {
+		if (kread((KA_T)q->q_ptr, (char *)&inp, sizeof(inp))) {
+		    (void) sprintf(Namech, "can't read inpcb from %s",
+			print_kptr((KA_T)q->q_ptr, (char *)NULL));
+		    enter_nm(Namech);
+		    return;
 		}
-	    } else if (!strcasecmp(pr, "UDP")) {
-		Lf->lts.type = 1;
-		Lf->lts.state.i = (int)inp.inp_tstate;
-	    }
+		la = (unsigned char *)&inp.inp_laddr;
+		lp = (int)ntohs(inp.inp_lport);
+		if (inp.inp_faddr.s_addr != INADDR_ANY || inp.inp_fport != 0) {
+		    fa = (unsigned char *)&inp.inp_faddr;
+		    fp = (int)ntohs(inp.inp_fport);
+		}
+		if (fa || la)
+		    (void) ent_inaddr(la, lp, fa, fp, AF_INET);
+		if (tcp) {
+		    if (inp.inp_ppcb
+		    &&  !kread((KA_T)inp.inp_ppcb, (char *)&t, sizeof(t))) {
+			ts = 1;
+			Lf->lts.type = 0;
+			Lf->lts.state.i = (int)t.t_state;
+		    }
+		} else {
+		    Lf->lts.type = 1;
+		    Lf->lts.state.i = (int)inp.inp_tstate;
+		} 
+	    } else
+		enter_nm("no address for this protocol");
 	} else
 	    enter_nm("no address");
 /*
