@@ -37,7 +37,7 @@
 # if	!defined(lint)
 static char copyright[] =
 "@(#) Copyright 1997 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dvch.c,v 1.5 98/03/06 08:40:55 abe Exp $";
+static char *rcsid = "$Id: dvch.c,v 1.9 99/10/23 14:39:54 abe Exp $";
 # endif	/* !defined(lint) */
 
 #include "../lsof.h"
@@ -47,13 +47,19 @@ static char *rcsid = "$Id: dvch.c,v 1.5 98/03/06 08:40:55 abe Exp $";
  *
  * The caller may define the following:
  *
- *	DCACHE_CLONE	is the name of the function that reads and writes
- *			the clone section of the device cache file.  The
- *			clone section follows the device section.
+ *	DCACHE_CLONE	is the name of the function that reads and writes the
+ *			clone section of the device cache file.  The clone
+ *			section follows the device section.  If DCACHE_CLONE
+ *			isn't defined, but HAS_STD_CLONE is defined to be 1,
+ *			DCACHE_CLONE defaults to the local static function
+ *			rw_clone_sect() that reads and writes a standard
+ *			clone cache.
  *
- *	DCACHE_CLR	is the name of the function that clears the clone
- *			and pseudo caches when reading the device cache
- *			fails.
+ *	DCACHE_CLR	is the name of the function that clears the clone and
+ *			pseudo caches when reading the device cache fails.   If
+ *			DCACHE_CLR isn't defined, but HAS_STD_CLONE is defined
+ *			to be 1, DCACHE_CLR defaults to the local static
+ *			function clr_sect() that clears a standard clone cache.
  *
  *	DCACHE_PSEUDO	is the name of the function that reads and writes
  *			the pseudo section of the device cache file.  The
@@ -87,6 +93,29 @@ static char *rcsid = "$Id: dvch.c,v 1.5 98/03/06 08:40:55 abe Exp $";
 static int crctbl[CRC_TBLL];		/* crc partial results table */
 
 
+/*
+ * Local function prototypes
+ */
+
+#undef	DCACHE_CLR_LOCAL
+# if	!defined(DCACHE_CLR)
+#  if	defined(HAS_STD_CLONE) && HAS_STD_CLONE==1
+#define	DCACHE_CLR		clr_sect
+#define	DCACHE_CLR_LOCAL	1
+_PROTOTYPE(static void clr_sect,(void));
+#  endif	/* defined(HAS_STD_CLONE) && HAS_STD_CLONE==1 */
+# endif	/* !defined(DCACHE_CLR) */
+
+#undef	DCACHE_CLONE_LOCAL
+# if	!defined(DCACHE_CLONE)
+#  if	defined(HAS_STD_CLONE) && HAS_STD_CLONE==1
+#define	DCACHE_CLONE		rw_clone_sect
+#define	DCACHE_CLONE_LOCAL	1
+_PROTOTYPE(static int rw_clone_sect,(int m));
+#  endif	/* defined(HAS_STD_CLONE) && HAS_STD_CLONE==1 */
+# endif	/*!defined(DCACHE_CLONE) */
+
+
 # if	defined(HASBLKDEV)
 /*
  * alloc_bdcache() - allocate block device cache
@@ -181,6 +210,27 @@ clr_devtab()
 }
 
 
+# if	defined(DCACHE_CLR_LOCAL)
+/*
+ * clr_sect() - clear cached standard clone sections
+ */
+
+static void
+clr_sect()
+{
+	struct clone *c, *c1;
+
+	if (Clone) {
+	    for (c = Clone; c; c = c1) {
+		c1 = c->next;
+		(void) free((FREE_P *)c);
+	    }
+	    Clone = (struct clone *)NULL;
+	}
+}
+# endif	/* defined(DCACHE_CLR_LOCAL) */
+
+
 /*
  * crc(b, l, s) - compute a crc for a block of bytes
  */
@@ -693,8 +743,6 @@ close_exit:
 			(void) fprintf(stderr,
 			     "%s: WARNING: can't change ownerships of %s: %s\n",
 			     Pn, DCpath[DCpathX], strerror(errno));
-		    (void) unlink(DCpath[DCpathX]);
-		    goto close_exit;
 	 	}
 	    }
 	    if (!Fwarn && DCstate != 1 && !DCunsafe)
@@ -1066,6 +1114,136 @@ read_dhdr:
 }
 
 
+# if	defined(DCACHE_CLONE_LOCAL)
+/*
+ * rw_clone_sect() - read/write the device cache file clone section
+ */
+
+static int
+rw_clone_sect(m)
+	int m;				/* mode: 1 = read; 2 = write */
+{
+	char buf[MAXPATHLEN*2], *cp, *cp1;
+	struct clone *c;
+	struct l_dev *dp;
+	int i, j, len, n;
+
+	if (m == 1) {
+
+	/*
+	 * Read the clone section header and validate it.
+	 */
+	    if (!fgets(buf, sizeof(buf), DCfs)) {
+
+bad_clone_sect:
+		if (!Fwarn) {
+		    (void) fprintf(stderr,
+			"%s: bad clone section header in %s: ",
+			Pn, DCpath[DCpathX]);
+		    safestrprt(buf, stderr, 1);
+		}
+		return(1);
+	    }
+	    (void) crc(buf, strlen(buf), &DCcksum);
+	    len = strlen("clone section: ");
+	    if (strncmp(buf, "clone section: ", len) != 0)
+		goto bad_clone_sect;
+	    if ((n = atoi(&buf[len])) < 0)
+		goto bad_clone_sect;
+	/*
+	 * Read the clone section lines and create the Clone list.
+	 */
+	    for (i = 0; i < n; i++) {
+		if (fgets(buf, sizeof(buf), DCfs) == NULL) {
+		    if (!Fwarn) {
+			(void) fprintf(stderr,
+			    "%s: bad clone line in %s: ", Pn, DCpath[DCpathX]);
+			safestrprt(buf, stderr, 1);
+		    }
+		    return(1);
+		}
+		(void) crc(buf, strlen(buf), &DCcksum);
+	    /*
+	     * Assemble Devtp[] index and make sure it's correct.
+	     */
+		for (cp = buf, j = 0; *cp != ' '; cp++) {
+		    if (*cp < '0' || *cp > '9') {
+
+bad_clone_index:
+			if (!Fwarn) {
+			    (void) fprintf(stderr,
+				"%s: bad cached clone device index: ", Pn);
+			    safestrprt(buf, stderr, 1);
+			}
+			return(1);
+		    }
+		    j = (j * 10) + (int)(*cp - '0');
+		}
+		if (j < 0 || j >= Ndev || (cp1 = strchr(++cp, '\n')) == NULL)
+		    goto bad_clone_index;
+		if (strncmp(cp, Devtp[j].name, (cp1 - cp)) != 0)
+		    goto bad_clone_index;
+	    /*
+	     * Allocate and complete a clone structure.
+	     */
+		if (!(c = (struct clone *)malloc(sizeof(struct clone)))) {
+		    (void) fprintf(stderr,
+			"%s: no space for cached clone: ", Pn);
+		    safestrprt(buf, stderr, 1);
+		    Exit(1);
+		}
+		c->dx = j;
+		c->next = Clone;
+		Clone = c;
+	    }
+	    return(0);
+	} else if (m == 2) {
+
+	/*
+	 * Write the clone section header.
+	 */
+	    for (c = Clone, n = 0; c; c = c->next, n++)
+		;
+	    (void) sprintf(buf, "clone section: %d\n", n);
+	    if (wr2DCfd(buf, &DCcksum))
+		return(1);
+	/*
+	 * Write the clone section lines.
+	 */
+	    for (c = Clone; c; c = c->next) {
+		for (dp = &Devtp[c->dx], j = 0; j < Ndev; j++) {
+		    if (dp == Sdev[j])
+			break;
+		}
+		if (j >= Ndev) {
+		    if (!Fwarn) {
+			(void) fprintf(stderr,
+			    "%s: can't make index for clone: ", Pn);
+			safestrprt(dp->name, stderr, 1);
+		    }
+		    (void) unlink(DCpath[DCpathX]);
+		    (void) close(DCfd);
+		    DCfd = -1;
+		    return(1);
+		}
+		(void) sprintf(buf, "%d %s\n", j, dp->name);
+		if (wr2DCfd(buf, &DCcksum))
+		    return(1);
+	    }
+	    return(0);
+	}
+/*
+ * A shouldn't-happen case: mode neither 1 nor 2.
+ */
+	(void) fprintf(stderr, "%s: internal rw_clone_sect error: %d\n",
+	    Pn, m);
+	Exit(1);
+	return(1);		/* This useless return(1) keeps some
+				 * compilers happy. */
+}
+# endif	/* defined(DCACHE_CLONE_LOCAL) */
+
+
 /*
  * write_dcache() - write device cache file
  */
@@ -1113,7 +1291,7 @@ write_dcache()
 	    return;
 	for (i = 0; i < Ndev; i++) {
 	    dp = Sdev[i];
-	    (void) sprintf(buf, "%x %ld %s\n", (int)dp->rdev, (long)dp->inode,
+	    (void) sprintf(buf, "%lx %ld %s\n", (long)dp->rdev, (long)dp->inode,
 		dp->name);
 	    if (wr2DCfd(buf, &DCcksum))
 		return;
