@@ -37,7 +37,7 @@
 # if	!defined(lint)
 static char copyright[] =
 "@(#) Copyright 1997 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: prfp.c,v 1.3 98/03/06 08:40:58 abe Exp $";
+static char *rcsid = "$Id: prfp.c,v 1.6 99/06/22 09:56:03 abe Exp $";
 # endif	/* !defined(lint) */
 
 #include "../lsof.h"
@@ -65,9 +65,15 @@ process_file(fp)
 	int flag;
 
 #if	defined(FILEPTR)
+/*
+ * Save file structure address for process_node().
+ */
 	FILEPTR = &f;
 #endif	/* defined(FILEPTR) */
 
+/*
+ * Read file structure.
+ */
 	if (kread((KA_T)fp, (char *)&f, sizeof(f))) {
 	    (void) sprintf(Namech, "can't read file struct from %s",
 		print_kptr(fp, (char *)NULL));
@@ -75,7 +81,6 @@ process_file(fp)
 	    return;
 	}
 	Lf->off = (SZOFFTYPE)f.f_offset;
-
 	if (f.f_count) {
 
 	/*
@@ -87,6 +92,29 @@ process_file(fp)
 		Lf->access = 'w';
 	    else if (flag == (FREAD | FWRITE))
 		Lf->access = 'u';
+
+#if	defined(HASFSTRUCT)
+	/*
+	 * Save file structure values.
+	 */
+	    if (Fsv & FSV_CT) {
+		Lf->fct = (long)f.f_count;
+		Lf->fsv |= FSV_CT;
+	    }
+	    if (Fsv & FSV_FA) {
+		Lf->fsa = fp;
+		Lf->fsv |= FSV_FA;
+	    }
+	    if (Fsv & FSV_FG) {
+		Lf->ffg = (long)f.f_flag;
+		Lf->fsv |= FSV_FG;
+	    }
+	    if (Fsv & FSV_NI) {
+		Lf->fna = (KA_T)f.f_data;
+		Lf->fsv |= FSV_NI;
+	    }
+#endif	/* defined(HASFSTRUCT) */
+
 	/*
 	 * Process structure by its type.
 	 */
