@@ -37,7 +37,7 @@
 # if	!defined(lint)
 static char copyright[] =
 "@(#) Copyright 1997 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: rmnt.c,v 1.6 98/07/17 08:48:28 abe Exp $";
+static char *rcsid = "$Id: rmnt.c,v 1.7 99/06/22 09:56:33 abe Exp $";
 # endif	/* !defined(lint) */
 
 #include "../lsof.h"
@@ -45,15 +45,22 @@ static char *rcsid = "$Id: rmnt.c,v 1.6 98/07/17 08:48:28 abe Exp $";
 
 
 /*
- * The caller may define an RMNT_EXPDEV macro to expand (ala EP/IX)
- * device numbers.
+ * The caller may define:
  *
- * EP/IX, for example, uses:
+ * 1.  An RMNT_EXPDEV macro to expand (ala EP/IX) device numbers;
+ *
+ *     EP/IX, for example, uses:
  *
  *	#define RMNT_EXPDEV(n) expdev(n)
  *
- * The caller may define a custom macro for making decisions to
- * skip entries -- e.g., ones whose mnt_type is MNTTYPE_IGNORE.
+ * 2.  A custom macro, MNTSKIP, for making decisions to skip entries
+ *     -- e.g., ones whose mnt_type is MNTTYPE_IGNORE.
+ *
+ * 3.  RMNT_FSTYPE to specify the element name of the character string of the
+ *     mntent structure containing the file system type, and MOUNTS_FSTYPE to
+ *     specify the element name of the character string pointer of the local
+ *     mounts structure where RMNT_FSTYPE is to be copied.
+ *     
  */
 
 #if	!defined(RMNT_EXPDEV)
@@ -143,13 +150,6 @@ readmnt()
 		    (void) zeromem(&sb, sizeof(sb));
 		    if ((opte = x2dev(opt + 4, (dev_t *)&sb.st_dev))) {
 			sb.st_mode = S_IFDIR | 0777;
-
-#if	defined(HASFSTYPE)
-			(void) strncpy(sb.st_fstype, mp->mnt_type,
-			    sizeof(sb.st_fstype));
-			sb.st_fstype[sizeof(sb.st_fstype) - 1 ] = '\0';
-#endif	/* defined(HASFSTYPE */
-
 			if (!Fwarn)
 			    (void) fprintf(stderr,
 				"      assuming \"%.*s\" from %s\n",
@@ -182,6 +182,21 @@ no_space_for_mount:
 	    mtp->rdev = RMNT_EXPDEV(sb.st_rdev);
 	    mtp->inode = sb.st_ino;
 	    mtp->mode = sb.st_mode;
+
+# if	defined(RMNT_FSTYPE) && defined(MOUNTS_FSTYPE)
+	/*
+	 * Make a copy of RMNT_FSTYPE in MOUNTS_FSTYPE.
+	 */
+	    if (!(mtp->MOUNTS_FSTYPE = mkstrcpy(mp->RMNT_FSTYPE,
+						(MALLOC_S *)NULL)))
+	    {
+		(void) fprintf(stderr, "%s: no space for fstype (%s): %s\n",
+		    Pn, mtp->dir, mp->RMNT_FSTYPE);
+		Exit(1);
+	    }
+	    (void) strcpy(mtp->MOUNTS_FSTYPE, mp->RMNT_FSTYPE);
+# endif	/* defined(RMNT_FSTYP) && defined(MOUNTS_FSTYP) */
+
 	/*
 	 * Interpolate a possible file system (mounted-on device) name link.
 	 */
