@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: misc.c,v 1.15 98/08/18 08:07:30 abe Exp $";
+static char *rcsid = "$Id: misc.c,v 1.17 99/07/06 13:05:13 abe Exp $";
 #endif
 
 
@@ -44,6 +44,7 @@ static char *rcsid = "$Id: misc.c,v 1.15 98/08/18 08:07:30 abe Exp $";
  */
 
 _PROTOTYPE(static void closePipes,(void));
+_PROTOTYPE(static int dolstat,(char *path, char *buf, int len));
 _PROTOTYPE(static int dostat,(char *path, char *buf, int len));
 _PROTOTYPE(static int doreadlink,(char *path, char *buf, int len));
 _PROTOTYPE(static int doinchild,(int (*fn)(), char *fp, char *rbuf, int rbln));
@@ -366,6 +367,23 @@ doinchild(fn, fp, rbuf, rbln)
 }
 
 
+/*
+ * dolstat() - do an lstat() function
+ */
+
+static int
+dolstat(path, rbuf, rbln)
+	char *path;			/* path */
+	char *rbuf;			/* response buffer */
+	int rbln;			/* response buffer length */
+
+/* ARGSUSED */
+
+{
+	return(lstat(path, (struct stat *)rbuf));
+}
+
+
 /*
  * doreadlink() -- do a readlink() function
  */
@@ -381,10 +399,7 @@ doreadlink(path, rbuf, rbln)
 
 
 /*
- * dostat() - do a *stat() function
- *
- * DOSTAT_FUNCTION defines the function -- e.g., stat() or lstat() --
- * default = lstat().
+ * dostat() - do a stat() function
  */
 
 static int
@@ -396,12 +411,7 @@ dostat(path, rbuf, rbln)
 /* ARGSUSED */
 
 {
-
-#if	!defined(DOSTAT_FUNCTION)
-#define	DOSTAT_FUNCTION	lstat
-#endif	/* !defined(DOSTAT_FUNCTION) */
-
-	return(DOSTAT_FUNCTION(path, (struct stat *)rbuf));
+	return(stat(path, (struct stat *)rbuf));
 }
 
 
@@ -543,6 +553,26 @@ handleint(sig)
 }
 
 
+/*
+ * hashbyname() - hash by name
+ */
+
+int
+hashbyname(nm, mod)
+	char *nm;			/* pointer to NUL-terminated name */
+	int mod;			/* hash modulus */
+{
+	int i, j;
+
+	for (i = j = 0; *nm; nm++) {
+	    i ^= (int)*nm << j;
+	    if (++j > 7)
+		j = 0;
+	}
+	return(((int)(i * 31415)) & (mod - 1));
+}
+
+
 /*
  * is_nw_addr() - is this network address selected?
  */
@@ -714,6 +744,27 @@ is_readable(path, msg)
 }
 
 
+/*
+ * lstatsafely() - lstat path safely (i. e., with timeout)
+ */
+
+int
+lstatsafely(path, buf)
+	char *path;			/* file path */
+	struct stat *buf;		/* stat buffer address */
+{
+	if (Fblock) {
+	    if (!Fwarn) 
+		(void) fprintf(stderr,
+		    "%s: avoiding stat(%s): -b was specified.\n",
+		    Pn, path);
+	    errno = EWOULDBLOCK;
+	    return(1);
+	}
+	return(doinchild(dolstat, path, (char *)buf, sizeof(struct stat)));
+}
+
+
 /*
  * Readlink() - read and interpret file system symbolic links
  */
@@ -1154,6 +1205,44 @@ statsafely(path, buf)
 }
 
 
+/*
+ * stkdir() - stack directory name
+ */
+
+void
+stkdir(p)
+	char *p;		/* directory path */
+{
+	MALLOC_S len;
+/*
+ * Provide adequate space for directory stack pointers.
+ */
+	if (Dstkx >= Dstkn) {
+	    Dstkn += 128;
+	    len = (MALLOC_S)(Dstkn * sizeof(char *));
+	    if (!Dstk)
+		Dstk = (char **)malloc(len);
+	    else
+		Dstk = (char **)realloc((MALLOC_P *)Dstk, len);
+	    if (!Dstk) {
+		(void) fprintf(stderr,
+		    "%s: no space for directory stack at: ", Pn);
+		safestrprt(p, stderr, 1);
+		Exit(1);
+	    }
+	}
+/*
+ * Allocate space for the name, copy it there and put its pointer on the stack.
+ */
+	if (!(Dstk[Dstkx] = mkstrcpy(p, (MALLOC_S *)NULL))) {
+	    (void) fprintf(stderr, "%s: no space for: ", Pn);
+	    safestrprt(p, stderr, 1);
+	    Exit(1);
+	}
+	Dstkx++;
+}
+
+
 /*
  * x2dev() - convert hexadecimal ASCII string to device number
  */
@@ -1177,8 +1266,11 @@ x2dev(s, d)
 	if  (strncasecmp(s, "0x", 2) == 0)
 		s += 2;
 	for (cp = s, n = 0; *cp; cp++, n++) {
-	    if (isdigit((unsigned char)*cp) || (isalpha((unsigned char)*cp)
-	    &&  (c = toupper((unsigned char)*cp)) >= 'A' && c <= 'F'))
+	    if (isdigit((unsigned char)*cp))
+		continue;
+	    if ((unsigned char)*cp >= 'a' && (unsigned char)*cp <= 'f')
+		continue;
+	    if ((unsigned char)*cp >= 'A' && (unsigned char)*cp <= 'F')
 		continue;
 	    if (*cp == ' ' || *cp == ',')
 		break;
@@ -1200,11 +1292,15 @@ x2dev(s, d)
  * in the string relevant to sizeof(dev_t).
  */
 	for (r = 0; s < cp; s++) {
-	    r = (r << 4)
-	      | ((isdigit((unsigned char)*s)
-		? (unsigned char)(*s - '0')
-		: (unsigned char)(toupper((unsigned char)*s) - 'A' + 10))
-		  & 0xf);
+	    r = r << 4;
+	    if (isdigit((unsigned char)*s))
+		r |= (unsigned char)(*s - '0') & 0xf;
+	    else {
+		if (isupper((unsigned char)*s))
+		    r |= ((unsigned char)(*s - 'A') + 10) & 0xf;
+		else
+		    r |= ((unsigned char)(*s - 'a') + 10) & 0xf;
+	    }
 	}
 	*d = r;
 	return(s);
