@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: node.c,v 1.3 98/03/06 07:27:00 abe Exp $";
+static char *rcsid = "$Id: node.c,v 1.4 98/12/28 10:15:51 abe Exp $";
 #endif
 
 
@@ -45,7 +45,7 @@ static char *rcsid = "$Id: node.c,v 1.3 98/03/06 07:27:00 abe Exp $";
 
 char *
 print_kptr(kp, buf)
-	KA_T kp;			/* node address */
+	KA_T kp;			/* kernel pointer address */
 	char *buf;			/* optional destination buffer */
 {
 	static char dbuf[32];
