@@ -32,22 +32,18 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: print.c,v 1.24 98/08/03 15:43:04 abe Exp $";
+static char *rcsid = "$Id: print.c,v 1.34 99/11/28 06:34:37 abe Exp $";
 #endif
 
 
 #include "lsof.h"
 
-#if	defined(EMPTY)
-#undef	EMPTY
-#endif	/* defined(EMPTY) */
 
-#include <utmp.h>
-
-struct utmp dutmp;			/* dummy entry to get login name size */
+/*
+ * Local definitions, structures and function prototypes
+ */
 
 #define HCINC		64		/* host cache size increase chunk */
-#define LNML sizeof(dutmp.ut_name)
 #define PORTHASHBUCKETS	128		/* port hash bucket count
 					 * !!MUST BE A POWER OF 2!! */
 #define	PORTTABTHRESH	10		/* threshold at which we will switch
@@ -152,28 +148,42 @@ fill_portmap()
 	char buf[128], *cp, *nm;
 	CLIENT *c;
 	int h, port, pr;
-	struct hostent *he;
-	struct sockaddr_in ia;
 	MALLOC_S nl;
 	struct pmaplist *p = (struct pmaplist *)NULL;
 	struct porttab *pt;
 	struct rpcent *r;
-	int s = RPC_ANYSOCK;
 	struct TIMEVAL_LSOF tm;
+
+#if	!defined(CAN_USE_CLNT_CREATE)
+	struct hostent *he;
+	struct sockaddr_in ia;
+	int s = RPC_ANYSOCK;
+#endif	/* !defined(CAN_USE_CLNT_CREATE) */
+
 /*
- * Construct a TCP socket address for communicating with the portmapper.
+ * Construct structures for communicating with the portmapper.
  */
+
+#if	!defined(CAN_USE_CLNT_CREATE)
 	zeromem(&ia, sizeof(ia));
 	ia.sin_family = AF_INET;
 	if ((he = gethostbyname("localhost")))
 	    MEMMOVE((caddr_t)&ia.sin_addr, he->h_addr, he->h_length);
+	ia.sin_port = htons(PMAPPORT);
+#endif	/* !defined(CAN_USE_CLNT_CREATE) */
+
 	tm.tv_sec = 60;
 	tm.tv_usec = 0;
-	ia.sin_port = htons(PMAPPORT);
 /*
  * Get an RPC client handle.  Then ask for a dump of the port map.
  */
+
+#if	defined(CAN_USE_CLNT_CREATE)
+	if (!(c = clnt_create("localhost", PMAPPROG, PMAPVERS, "tcp")))
+#else	/* !defined(CAN_USE_CLNT_CREATE) */
 	if (!(c = clnttcp_create(&ia, PMAPPROG, PMAPVERS, &s, 0, 0)))
+#endif	/* defined(CAN_USE_CLNT_CREATE) */
+
 	    return;
 	if (clnt_call(c, PMAPPROC_DUMP, XDR_VOID, NULL, XDR_PMAPLIST,
 		      (caddr_t)&p, tm)
@@ -216,7 +226,7 @@ fill_portmap()
 		    cp = r->r_name;
 	    }
 	    if (!cp) {
-		(void) sprintf(buf, "%lu", p->pml_map.pm_prog);
+		(void) sprintf(buf, "%lu", (unsigned long)p->pml_map.pm_prog);
 		cp = buf;
 	    }
 	    if (!strlen(cp))
@@ -624,7 +634,7 @@ print_file()
 	if (PrPass && !Hdr) {
 
 	/*
-	 * Print the header line of this is the second pass and the
+	 * Print the header line if this is the second pass and the
 	 * header hasn't already been printed.
 	 */
 	    (void) printf("%-*.*s %*s", CmdColW, CmdColW, CMDTTL,
@@ -637,17 +647,33 @@ print_file()
 
 	    if (Fpgrp)
 		(void) printf(" %*s", PgrpColW, PGRPTTL);
-	    (void) printf(" %*s %*s   %*s %*s",
+	    (void) printf(" %*s %*s   %*s",
 		UserColW, USERTTL,
 		FdColW - 2, FDTTL,
-		TypeColW, TYPETTL,
-		DevColW, DEVTTL);
+		TypeColW, TYPETTL);
+
+#if	defined(HASFSTRUCT)
+	    if (Fsv) {
+		if (Fsv & FSV_FA)
+		    (void) printf(" %*s", FsColW, FSTTL);
+		if (Fsv & FSV_CT)
+		    (void) printf(" %*s", FcColW, FCTTL);
+		if (Fsv & FSV_FG)
+		    (void) printf(" %*s", FgColW, FGTTL);
+		if (Fsv & FSV_NI)
+		    (void) printf(" %*s", NiColW, NiTtl);
+	    }
+#endif	/* defined(HASFSTRUCT) */
+
+	    (void) printf(" %*s", DevColW, DEVTTL);
 	    if (Foffset)
 		(void) printf(" %*s", SzOffColW, OFFTTL);
 	    else if (Fsize)
 		(void) printf(" %*s", SzOffColW, SZTTL);
 	    else
 		(void) printf(" %*s", SzOffColW, SZOFFTTL);
+	    if (Fnlink)
+		(void) printf(" %*s", NlColW, NLTTL);
 	    (void) printf(" %*s %s\n", NodeColW, NODETTL, NMTTL);
 	    Hdr++;
 	}
@@ -671,18 +697,6 @@ print_file()
 		PidColW = len;
 	} else
 	    (void) printf(" %*d", PidColW, Lp->pid);
-	if (Fpgrp) {
-
-	/*
-	 * Size or print the process group ID.
-	 */
-	    if (!PrPass) {
-		(void) sprintf(buf, "%d", Lp->pgrp);
-		if ((len = strlen(buf)) > PgrpColW)
-		    PgrpColW = len;
-	    } else
-		(void) printf(" %*d", PgrpColW, Lp->pgrp);
-	}
 
 #if	defined(HASPPID)
 	if (Fppid) {
@@ -699,6 +713,18 @@ print_file()
 	}
 #endif	/* defined(HASPPID) */
 
+	if (Fpgrp) {
+
+	/*
+	 * Size or print the process group ID.
+	 */
+	    if (!PrPass) {
+		(void) sprintf(buf, "%d", Lp->pgrp);
+		if ((len = strlen(buf)) > PgrpColW)
+		    PgrpColW = len;
+	    } else
+		(void) printf(" %*d", PgrpColW, Lp->pgrp);
+	}
 /*
  * Size or print the user ID or login name.
  */
@@ -734,6 +760,57 @@ print_file()
 		TypeColW = len;
 	} else
 	    (void) printf(" %*.*s", TypeColW, TypeColW, Lf->type);
+
+#if	defined(HASFSTRUCT)
+/*
+ * Size or print the file structure address, file usage count, and node
+ * ID (address).
+ */
+
+	if (Fsv) {
+	    if (Fsv & FSV_FA) {
+		cp =  (Lf->fsv & FSV_FA) ? print_kptr(Lf->fsa, buf) : "";
+		if (!PrPass) {
+		    if ((len = strlen(cp)) > FsColW)
+			FsColW = len;
+		} else
+		    (void) printf(" %*.*s", FsColW, FsColW, cp);
+		    
+	    }
+	    if (Fsv & FSV_CT) {
+		if (Lf->fsv & FSV_CT) {
+		    (void) sprintf(buf, "%ld", Lf->fct);
+		    cp = buf;
+		} else
+		    cp = "";
+		if (!PrPass) {
+		    if ((len = strlen(cp)) > FcColW)
+			FcColW = len;
+		} else
+		    (void) printf(" %*.*s", FcColW, FcColW, cp);
+	    }
+	    if (Fsv & FSV_FG) {
+		if ((Lf->fsv & FSV_FG) && (FsvFlagX || Lf->ffg || Lf->pof))
+		    cp = print_fflags(Lf->ffg, Lf->pof);
+		else
+		    cp = "";
+		if (!PrPass) {
+		    if ((len = strlen(cp)) > FgColW)
+			FgColW = len;
+		} else
+		    (void) printf(" %*.*s", FgColW, FgColW, cp);
+	    }
+	    if (Fsv & FSV_NI) {
+		cp = (Lf->fsv & FSV_NI) ? print_kptr(Lf->fna, buf) : "";
+		if (!PrPass) {
+		    if ((len = strlen(cp)) > NiColW)
+			NiColW = len;
+		} else
+		    (void) printf(" %*.*s", NiColW, NiColW, cp);
+	    }
+	}
+#endif	/* defined(HASFSTRUCT) */
+
 /*
  * Size or print the device information.
  */
@@ -840,6 +917,21 @@ print_file()
 	    } else
 		(void) printf("%*.*s", SzOffColW, SzOffColW, "");
 	}
+/*
+ * Size or print the link count.
+ */
+	if (Fnlink) {
+	    if (Lf->nlink_def) {
+		(void) sprintf(buf, " %ld", Lf->nlink);
+		cp = buf;
+	   } else
+		cp = "";
+	    if (!PrPass) {
+		if ((len = strlen(cp)) > NlColW)
+		    NlColW = len;
+	    } else
+		(void) printf(" %*s", NlColW, cp);
+	}
 /*
  * Size or print the inode information.
  */
@@ -1036,6 +1128,8 @@ print_init()
 	CmdColW = strlen(CMDTTL);
 	DevColW = strlen(DEVTTL);
 	FdColW = strlen(FDTTL);
+	if (Fnlink)
+	    NlColW = strlen(NLTTL);
 	NmColW = strlen(NMTTL);
 	NodeColW = strlen(NODETTL);
 	PgrpColW = strlen(PGRPTTL);
@@ -1049,6 +1143,14 @@ print_init()
 	    SzOffColW = strlen(SZOFFTTL);
 	TypeColW = strlen(TYPETTL);
 	UserColW = strlen(USERTTL);
+
+#if	defined(HASFSTRUCT)
+	FcColW = strlen(FCTTL);
+	FgColW = strlen(FGTTL);
+	FsColW = strlen(FSTTL);
+	NiColW = strlen(NiTtl);
+#endif	/* defined(HASFSTRUCT) */
+
 }
 
 
@@ -1903,6 +2005,58 @@ printrawaddr(sa)
 }
 
 
+/*
+ * printsockty() - print socket type
+ */
+
+char *
+printsockty(ty)
+	int ty;				/* socket type -- e.g., from so_type */
+{
+	static char buf[64];
+	char *cp;
+
+	switch (ty) {
+
+#if	defined(SOCK_STREAM)
+	case SOCK_STREAM:
+	    cp = "STREAM";
+	    break;
+#endif	/* defined(SOCK_STREAM) */
+
+#if	defined(SOCK_STREAM)
+	case SOCK_DGRAM:
+	    cp = "DGRAM";
+	    break;
+#endif	/* defined(SOCK_DGRAM) */
+
+#if	defined(SOCK_RAW)
+	case SOCK_RAW:
+	    cp = "RAW";
+	    break;
+#endif	/* defined(SOCK_RAW) */
+
+#if	defined(SOCK_RDM)
+	case SOCK_RDM:
+	    cp = "RDM";
+	    break;
+#endif	/* defined(SOCK_RDM) */
+
+#if	defined(SOCK_SEQPACKET)
+	case SOCK_SEQPACKET:
+	    cp = "SEQPACKET";
+	    break;
+#endif	/* defined(SOCK_SEQPACKET) */
+
+	default:
+	    (void) sprintf(buf, "SOCK_%#x", ty);
+	    return(buf);
+	}
+	(void) sprintf(buf, "SOCK_%s", cp);
+	return(buf);
+}
+
+
 /*
  * printuid() - print User ID or login name
  */
@@ -1921,7 +2075,7 @@ printuid(u, ty)
 	static struct stat sbs;
 	static struct uidcache {
 	    uid_t u;
-	    char nm[LNML+1];
+	    char nm[LOGINML+1];
 	    struct uidcache *next;
 	} **uc = (struct uidcache **)NULL;
 	struct uidcache *up, *upn;
@@ -1964,7 +2118,7 @@ printuid(u, ty)
 		if (sbs.st_mtime != sb.st_mtime || sbs.st_ctime != sb.st_ctime)
 		{
 		    for (i = 0; i < UIDCACHEL; i++) {
-			if (!(up = uc[i])) {
+			if ((up = uc[i])) {
 			    do {
 				upn = up->next;
 				(void) free((FREE_P *)up);
@@ -2009,8 +2163,8 @@ printuid(u, ty)
 			Pn, (unsigned long)u, pw->pw_name);
 		    Exit(1);
 		}
-		(void) strncpy(upn->nm, pw->pw_name, LNML);
-		upn->nm[LNML] = '\0';
+		(void) strncpy(upn->nm, pw->pw_name, LOGINML);
+		upn->nm[LOGINML] = '\0';
 		upn->u = (uid_t)u;
 		upn->next = uc[i];
 		uc[i] = upn;
@@ -2034,8 +2188,9 @@ printuid(u, ty)
  */
 
 void
-printunkaf(fam)
+printunkaf(fam, ty)
 	int fam;			/* unknown address family */
+	int ty;				/* output type: 0 = terse; 1 = full */
 {
 	char *p, *s;
 
@@ -2330,10 +2485,17 @@ printunkaf(fam)
 #endif	/* defined(AF_USER) */
 
 	default:
-	    (void) sprintf(Namech, "no further information on family %#x", fam);
+	    if (!ty)
+		(void) sprintf(Namech, "%#x", fam);
+	    else
+		(void) sprintf(Namech,
+		    "no further information on family %#x", fam);
 	    return;
 	}
-	(void) sprintf(Namech, "no further information on %sAF_%s", p, s);
+	if (!ty)
+	    (void) sprintf(Namech, "%sAF_%s", p, s);
+	else
+	    (void) sprintf(Namech, "no further information on %sAF_%s", p, s);
 	return;
 }
 
