@@ -32,30 +32,45 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: proc.c,v 1.16 98/08/18 08:07:33 abe Exp $";
+static char *rcsid = "$Id: proc.c,v 1.24 99/08/03 08:20:45 abe Exp $";
 #endif
 
 
 #include "lsof.h"
 
 
+#if	defined(HASFSTRUCT)
+_PROTOTYPE(static char *alloc_fflbuf,(char **bp, int *al, int lr));
+
+
 /*
- * comppid() - compare PIDs
+ * alloc_fflbuf() - allocate file flags print buffer
  */
 
-int
-comppid(a1, a2)
-	COMP_P *a1, *a2;
+static char *
+alloc_fflbuf(bp, al, lr)
+	char **bp;			/* current buffer pointer */
+	int *al;			/* current allocated length */
+	int lr;				/* length required */
 {
-	struct lproc **p1 = (struct lproc **)a1;
-	struct lproc **p2 = (struct lproc **)a2;
+	int sz;
 
-	if ((*p1)->pid < (*p2)->pid)
-	    return(-1);
-	if ((*p1)->pid > (*p2)->pid)
-	    return(1);
-	return(0);
+	sz = (int)(lr + 1);		/* allocate '\0' space */
+	if (*bp && (sz <= *al))
+	    return(*bp);
+	if (*bp)
+	    *bp = (char *)realloc((MALLOC_P *)*bp, (MALLOC_S)sz);
+	else
+	    *bp = (char *)malloc((MALLOC_S)sz);
+	if (!*bp) {
+	    (void) fprintf(stderr, "%s: no space (%d) for print flags\n",
+		Pn, sz);
+	    Exit(1);
+	}
+	*al = sz;
+	return(*bp);
 }
+#endif	/* defined(HASFSTRUCT) */
 
 
 /*
@@ -94,10 +109,11 @@ alloc_lfile(nm, num)
  */
 	Lf->access = Lf->lock = ' ';
 	Lf->dev_def = Lf->inp_ty = Lf->is_com = Lf->is_nfs = Lf->is_stream
-		    = Lf->lmi_srch = Lf->off_def = Lf->sz_def
+		    = Lf->lmi_srch = Lf->nlink_def = Lf->off_def = Lf->sz_def
 		    = (unsigned char)0;
 	Lf->li[0].af = Lf->li[1].af = 0;
 	Lf->lts.type = -1;
+	Lf->nlink = 0l;
 
 #if	defined(HASTCPTPIQ)
 	Lf->lts.rqs = Lf->lts.sqs = (unsigned char)0;
@@ -107,7 +123,6 @@ alloc_lfile(nm, num)
 	Lf->lts.rws = Lf->lts.wws = (unsigned char)0;
 #endif	/* defined(HASTCPTPIW) */
 
-
 #if	defined(HASFSINO)
 	Lf->fs_ino = 0;
 #endif	/* defined(HASFSINO) */
@@ -143,6 +158,12 @@ alloc_lfile(nm, num)
 	Lf->ntype = Ntype = N_REGLR;
 	Namech[0] = '\0';
 
+#if	defined(HASFSTRUCT)
+	Lf->fct = Lf->ffg = Lf->pof = (long)0;
+	Lf->fna = (KA_T)NULL;
+	Lf->fsv = (unsigned char)0;
+#endif	/* defined(HASFSTRUCT) */
+
 #if	defined(HASLFILEADD) && defined(SETLFILEADD)
 /*
  * Do local initializations.
@@ -232,6 +253,25 @@ alloc_lproc(pid, pgrp, ppid, uid, cmd, pss, sf)
 }
 
 
+/*
+ * comppid() - compare PIDs
+ */
+
+int
+comppid(a1, a2)
+	COMP_P *a1, *a2;
+{
+	struct lproc **p1 = (struct lproc **)a1;
+	struct lproc **p2 = (struct lproc **)a2;
+
+	if ((*p1)->pid < (*p2)->pid)
+	    return(-1);
+	if ((*p1)->pid > (*p2)->pid)
+	    return(1);
+	return(0);
+}
+
+
 /*
  * ent_inaddr() - enter Internet addresses
  */
@@ -450,7 +490,7 @@ is_proc_excl(pid, pgrp, uid, pss, sf)
  */
 	if (Myuid && Myuid != (uid_t)uid)
 	    return(1);
-#endif
+#endif	/* defined(HASSECURITY) */
 
 /*
  * If the excluding of process listing by UID has been specified,
@@ -592,6 +632,80 @@ link_lfile()
 }
 
 
+#if	defined(HASFSTRUCT)
+/*
+ * print_fflags() - print interpreted f_flag[s]
+ */
+
+char *
+print_fflags(ffg, pof)
+	long ffg;		/* file structure's flags value */
+	long pof;		/* process open files flags value */
+{
+	int al, ct, fx;
+	static char *bp = (char *)NULL;
+	static int bl = 0;
+	char *sep;
+	int sepl;
+	struct pff_tab *tp;
+	long wf;
+	char xbuf[64];
+/*
+ * Reduce the supplied flags according to the definitions in Pff_tab[] and
+ * Pof_tab[].
+ */
+	for (ct = fx = 0; fx < 2; fx++) {
+	    if (fx == 0) {
+		sep = "";
+		sepl = 0;
+		tp = Pff_tab;
+		wf = ffg;
+	    } else {
+		sep = ";";
+		sepl = 1;
+		tp = Pof_tab;
+		wf = pof;
+	    }
+	    for (; wf && !FsvFlagX; ct += al ) {
+		while (tp->nm) {
+		    if (wf & tp->val)
+			break;
+		    tp++;
+		}
+		if (!tp->nm)
+		    break;
+		al = (int)strlen(tp->nm) + sepl;
+		bp = alloc_fflbuf(&bp, &bl, al + ct);
+		(void) sprintf((bp + ct), "%s%s", sep, tp->nm);
+		sep = ",";
+		sepl = 1;
+		wf &= ~(tp->val);
+	    }
+	/*
+	 * If flag bits remain, print them in hex.  If hex output was
+	 * specified with +fG, print all flag values, including zero,
+	 * in hex.
+	 */
+	    if (wf || FsvFlagX) {
+		(void) sprintf(xbuf, "0x%lx", wf);
+		al = (int)strlen(xbuf) + sepl;
+		bp = alloc_fflbuf(&bp, &bl, al + ct);
+		(void) sprintf((bp + ct), "%s%s", sep, xbuf);
+		ct += al;
+	    }
+	}
+/*
+ * Make sure there is at least a NUL terminated reply.
+ */
+	if (!bp) {
+	    bp = alloc_fflbuf(&bp, &bl, 0);
+	    *bp = '\0';
+	}
+	return(bp);
+}
+#endif	/* defined(HASFSTRUCT) */
+
+
 /*
  * print_proc() - print process
  */
@@ -707,6 +821,33 @@ print_proc()
 		    lc++;
 		}
 	    }
+
+#if	defined(HASFSTRUCT)
+	    if (FieldSel[LSOF_FIX_FA].st && (Fsv & FSV_FA)
+	    &&  (Lf->fsv & FSV_FA)) {
+		(void) printf("%c%s%c", LSOF_FID_FA,
+		    print_kptr(Lf->fsa, (char *)NULL), Terminator);
+		lc++;
+	    }
+	    if (FieldSel[LSOF_FIX_CT].st && (Fsv & FSV_CT)
+	    &&  (Lf->fsv & FSV_CT)) {
+		(void) printf("%c%ld%c", LSOF_FID_CT, Lf->fct, Terminator);
+		lc++;
+	    }
+	    if (FieldSel[LSOF_FIX_FG].st && (Fsv & FSV_FG)
+	    &&  (Lf->fsv & FSV_FG) && (FsvFlagX || Lf->ffg || Lf->pof)) {
+		(void) printf("%c%s%c", LSOF_FID_FG,
+		    print_fflags(Lf->ffg, Lf->pof), Terminator);
+		lc++;
+	    }
+	    if (FieldSel[LSOF_FIX_NI].st && (Fsv & FSV_NI)
+	    &&  (Lf->fsv & FSV_NI)) {
+		(void) printf("%c%s%c", LSOF_FID_NI,
+		    print_kptr(Lf->fna, (char *)NULL), Terminator);
+		lc++;
+	    }
+#endif	/* defined(HASFSTRUCT) */
+
 	    if (FieldSel[LSOF_FIX_DEVCH].st && Lf->dev_ch && Lf->dev_ch[0]) {
 		for (cp = Lf->dev_ch; *cp == ' '; cp++)
 		    ;
@@ -736,6 +877,10 @@ print_proc()
 		(void) printf("%c%lu%c", LSOF_FID_INODE, Lf->inode, Terminator);
 		lc++;
 	    }
+	    if (FieldSel[LSOF_FIX_NLINK].st && Lf->nlink_def) {
+		(void) printf("%c%ld%c", LSOF_FID_NLINK, Lf->nlink, Terminator);
+		lc++;
+	    }
 	    if (FieldSel[LSOF_FIX_PROTO].st && Lf->inp_ty == 2) {
 		for (cp = Lf->iproto; *cp == ' '; cp++)
 		    ;
@@ -747,7 +892,7 @@ print_proc()
 	    if (FieldSel[LSOF_FIX_STREAM].st && Lf->nm && Lf->is_stream) {
 		if (strncmp(Lf->nm, "STR:", 4) == 0
 		||  strcmp(Lf->iproto, "STR") == 0) {
-		    putchar(FieldSel[LSOF_FIX_STREAM].id);
+		    putchar(LSOF_FID_STREAM);
 		    printname(0);
 		    putchar(Terminator);
 		    lc++;
@@ -755,7 +900,7 @@ print_proc()
 		}
 	    }
 	    if (st == 0 && FieldSel[LSOF_FIX_NAME].st) {
-		putchar(FieldSel[LSOF_FIX_NAME].id);
+		putchar(LSOF_FID_NAME);
 		printname(0);
 		putchar(Terminator);
 		lc++;
@@ -764,43 +909,6 @@ print_proc()
 		print_tcptpi(0);
 		lc++;
 	    }
-
-#if	defined(HASFIELDAP1)
-	    LISTLFILEAP1
-#endif	/* defined(HASFIELDAP1) */
-
-#if	defined(HASFIELDAP2)
-	    LISTLFILEAP2
-#endif	/* defined(HASFIELDAP2) */
-
-#if	defined(HASFIELDAP3)
-	    LISTLFILEAP3
-#endif	/* defined(HASFIELDAP3) */
-
-#if	defined(HASFIELDAP4)
-	    LISTLFILEAP4
-#endif	/* defined(HASFIELDAP4) */
-
-#if	defined(HASFIELDAP5)
-	    LISTLFILEAP5
-#endif	/* defined(HASFIELDAP5) */
-
-#if	defined(HASFIELDAP6)
-	    LISTLFILEAP6
-#endif	/* defined(HASFIELDAP6) */
-
-#if	defined(HASFIELDAP7)
-	    LISTLFILEAP7
-#endif	/* defined(HASFIELDAP7) */
-
-#if	defined(HASFIELDAP8)
-	    LISTLFILEAP8
-#endif	/* defined(HASFIELDAP8) */
-
-#if	defined(HASFIELDAP9)
-	    LISTLFILEAP9
-#endif	/* defined(HASFIELDAP9) */
-
 	    if (Terminator == '\0' && lc)
 		putchar('\n');
 	}
