@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: proto.h,v 1.14 98/06/25 07:00:34 abe Exp $
+ * $Id: proto.h,v 1.19 99/11/28 06:33:28 abe Exp $
  */
 
 
@@ -66,7 +66,7 @@
 _PROTOTYPE(extern void alloc_lfile,(char *nm, int num));
 _PROTOTYPE(extern void alloc_lproc,(int pid, int pgrp, int ppid, UID_ARG uid, char *cmd, int pss, int sf));
 _PROTOTYPE(extern void childx,(void));
-_PROTOTYPE(extern int ck_file_arg,(int i, int ac, char *av[]));
+_PROTOTYPE(extern int ck_file_arg,(int i, int ac, char *av[], int fv, int rs, struct stat *sbp));
 _PROTOTYPE(extern void ckkv,(char *d, char *er, char *ev, char *ea));
 _PROTOTYPE(extern void clr_devtab,(void));
 _PROTOTYPE(extern int compdev,(COMP_P *a1, COMP_P *a2));
@@ -78,6 +78,7 @@ _PROTOTYPE(extern void dropgid,(void));
 
 _PROTOTYPE(extern char *endnm,(void));
 _PROTOTYPE(extern void enter_dev_ch,(char *m));
+_PROTOTYPE(extern int enter_dir,(char *d, int descend));
 _PROTOTYPE(extern int enter_fd,(char *f));
 _PROTOTYPE(extern int enter_network_address,(char *na));
 _PROTOTYPE(extern int enter_id,(enum IDType ty, char *p));
@@ -91,6 +92,8 @@ _PROTOTYPE(extern void find_ch_ino,(void));
 _PROTOTYPE(extern void free_lproc,(struct lproc *lp));
 _PROTOTYPE(extern void gather_proc_info,(void));
 _PROTOTYPE(extern char *gethostnm,(unsigned char *ia, int af));
+_PROTOTYPE(extern int hashbyname,(char *nm, int mod));
+_PROTOTYPE(extern void hashSfile,(void));
 _PROTOTYPE(extern void initialize,(void));
 _PROTOTYPE(extern int is_cmd_excl,(char *cmd, short *pss, short *sf));
 _PROTOTYPE(extern int is_file_sel,(struct lfile *lf));
@@ -101,6 +104,7 @@ _PROTOTYPE(extern int kread,(KA_T addr, char *buf, READLEN_T len));
 _PROTOTYPE(extern void link_lfile,(void));
 _PROTOTYPE(extern struct l_dev *lkupdev,(dev_t *dev, int i, int r));
 _PROTOTYPE(extern int main,(int argc, char *argv[]));
+_PROTOTYPE(extern int lstatsafely,(char *path, struct stat *buf));
 _PROTOTYPE(extern char *mkstrcpy,(char *src, MALLOC_S *rlp));
 _PROTOTYPE(extern char *mkstrcat,(char *s1, int l1, char *s2, int l2, char *s3, int l3, MALLOC_S *clp));
 _PROTOTYPE(extern int printchdevname,(dev_t *rdev, int f));
@@ -113,7 +117,8 @@ _PROTOTYPE(extern int print_proc,(void));
 _PROTOTYPE(extern void printrawaddr,(struct sockaddr *sa));
 _PROTOTYPE(extern void print_tcptpi,(int nl));
 _PROTOTYPE(extern char *printuid,(UID_ARG u, int *ty));
-_PROTOTYPE(extern void printunkaf,(int fam));
+_PROTOTYPE(extern void printunkaf,(int fam, int ty));
+_PROTOTYPE(extern char *printsockty,(int ty));
 _PROTOTYPE(extern void process_file,(KA_T fp));
 _PROTOTYPE(extern void process_node,(KA_T f));
 _PROTOTYPE(extern char *Readlink,(char *arg));
@@ -124,6 +129,7 @@ _PROTOTYPE(extern int safestrlen,(char *sp, int flags));
 _PROTOTYPE(extern void safestrprtn,(char *sp, int len, FILE *fs, int flags));
 _PROTOTYPE(extern void safestrprt,(char *sp, FILE *fs, int flags));
 _PROTOTYPE(extern int statsafely,(char *path, struct stat *buf));
+_PROTOTYPE(extern void stkdir,(char *p));
 _PROTOTYPE(extern void usage,(int xv, int fh, int version));
 _PROTOTYPE(extern int vfy_dev,(struct l_dev *dp));
 _PROTOTYPE(extern char *x2dev,(char *s, dev_t *d));
@@ -154,6 +160,10 @@ _PROTOTYPE(extern void write_dcache,(void));
 _PROTOTYPE(extern int readfifonode,(KA_T fa, struct fifonode *f));
 #endif	/* defined(HASFIFONODE) */
 
+#if	defined(HASFSTRUCT)
+_PROTOTYPE(extern char *print_fflags,(long ffg, long pof));
+#endif	/* defined(HASFSTRUCT) */
+
 #if	defined(HASGNODE)
 _PROTOTYPE(extern int readgnode,(KA_T ga, struct gnode *g));
 #endif	/* defined(HASGNODE) */
