@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: store.c,v 1.14 98/08/18 08:07:35 abe Exp $";
+static char *rcsid = "$Id: store.c,v 1.22 10/.0/.1 .0:.5:.4 abe Exp $";
 #endif
 
 
@@ -53,6 +53,12 @@ struct l_dev **BSdev = (struct l_dev **)NULL;
 #endif	/* defined(HASBLKDEV) */
 
 int CkPasswd = 0;		/* time to check /etc/passwd for change */
+
+#if	defined(HAS_STD_CLONE)
+struct clone *Clone = (struct clone *)NULL;
+				/* clone device list */
+#endif	/* defined(HAS_STD_CLONE) */
+
 struct str_lst *Cmdl = (struct str_lst *)NULL;
 				/* command names selected with -c */
 int CmdColW;			/* COMMAND column width */
@@ -91,6 +97,17 @@ int DChelp = 0;			/* -D? status */
 int DevColW;			/* DEVICE column width */
 struct l_dev *Devtp = (struct l_dev *)NULL;
 				/* device table pointer */
+
+
+/*
+ * Externals for a stkdir(), dumbed-down for older AIX compilers.
+ */
+
+char **Dstk = (char **)NULL;	/* the directory stack */
+int Dstkx = 0;			/* Dstk[] index */
+int Dstkn = 0;			/* Dstk[] entries allocated */
+
+int ErrStat = 0;		/* path stat() error count */
 int Fand = 0;			/* -a option status */
 int Fblock = 0;			/* -b option status */
 int FdColW;			/* FD column width */
@@ -110,6 +127,7 @@ int Fnet = 0;			/* -i option status: 0==none, 1==find all,
 				 * 2==some found*/
 int Fnfs = 0;			/* -N option status: 0==none, 1==find all,
 				 * 2==some found*/
+int Fnlink = 0;			/* -L option status */
 int Foffset = 0;		/* -o option status */
 int Fovhd = 0;			/* -O option status */
 int Fport = 1;			/* -P option status */
@@ -121,12 +139,15 @@ int FportMap = 0;		/* +|-M option status */
 #endif	/* defined(HASPMAPENABLED) */
 
 int Fpgrp = 0;			/* -g option status */
-
-#if	defined(HASPPID)
 int Fppid = 0;			/* -R option status */
-#endif	/* defined(HASPPID) */
-
 int Fsize = 0;			/* -s option status */
+int FcColW;			/* FCT column width */
+int FgColW;			/* FILE-FLAG column width */
+int FsColW;			/* FSTR-ADDR column width */
+int Fsv = FSV_DEFAULT;		/* file struct value selections */
+int FsvFlagX = 0;		/* hex format status for FSV_FG */
+int NiColW;			/* NODE-ID column width */
+char *NiTtl = NITTL;		/* NODE-ID column title */
 int Ftcptpi = TCPTPI_STATE;	/* -T option status */
 int Fterse = 0;			/* -t option status */
 int Funix = 0;			/* -U option status */
@@ -147,65 +168,70 @@ struct fd_lst *Fdl = (struct fd_lst *)NULL;
 				/* file descriptors selected with -d */
 
 struct fieldsel FieldSel[] = {
-	{ LSOF_FID_ACCESS,	0,	LSOF_FNM_ACCESS	},	/*  0 */
-	{ LSOF_FID_CMD,		0,	LSOF_FNM_CMD	},	/*  1 */
-	{ LSOF_FID_DEVCH,	0,	LSOF_FNM_DEVCH	},	/*  2 */
-	{ LSOF_FID_DEVN,	0,	LSOF_FNM_DEVN	},	/*  3 */
-	{ LSOF_FID_FD,		0,	LSOF_FNM_FD	},	/*  4 */
-	{ LSOF_FID_INODE,	0,	LSOF_FNM_INODE	},	/*  5 */
-	{ LSOF_FID_LOCK,	0,	LSOF_FNM_LOCK	},	/*  6 */
-	{ LSOF_FID_LOGIN,	0,	LSOF_FNM_LOGIN	},	/*  7 */
-	{ LSOF_FID_MARK,	1,	LSOF_FNM_MARK	},	/*  8 */
-	{ LSOF_FID_NAME,	0,	LSOF_FNM_NAME	},	/*  9 */
-	{ LSOF_FID_OFFSET,	0,	LSOF_FNM_OFFSET	},	/* 10 */
-	{ LSOF_FID_PID,		1,	LSOF_FNM_PID	},	/* 11 */
-	{ LSOF_FID_PGRP,	0,	LSOF_FNM_PGRP	},	/* 12 */
-	{ LSOF_FID_PROTO,	0,	LSOF_FNM_PROTO	},	/* 13 */
-	{ LSOF_FID_PPID,	0,	LSOF_FNM_PPID	},	/* 14 */
-	{ LSOF_FID_SIZE,	0,	LSOF_FNM_SIZE	},	/* 15 */
-	{ LSOF_FID_STREAM,	0,	LSOF_FNM_STREAM	},	/* 16 */
-	{ LSOF_FID_TYPE,	0,	LSOF_FNM_TYPE	},	/* 17 */
-	{ LSOF_FID_TCPTPI,	0,	LSOF_FNM_TCPTPI	},	/* 18 */
-	{ LSOF_FID_UID,		0,	LSOF_FNM_UID	},	/* 19 */
-	{ LSOF_FID_TERM,	0,	LSOF_FNM_TERM	},	/* 20 */
+    { LSOF_FID_ACCESS, 0,  LSOF_FNM_ACCESS, NULL,     0		 }, /*  0 */
+    { LSOF_FID_CMD,    0,  LSOF_FNM_CMD,    NULL,     0		 }, /*  1 */
+    { LSOF_FID_CT,     0,  LSOF_FNM_CT,     &Fsv,     FSV_CT 	 }, /*  2 */
+    { LSOF_FID_DEVCH,  0,  LSOF_FNM_DEVCH,  NULL,     0		 }, /*  3 */
+    { LSOF_FID_DEVN,   0,  LSOF_FNM_DEVN,   NULL,     0		 }, /*  4 */
+    { LSOF_FID_FD,     0,  LSOF_FNM_FD,     NULL,     0		 }, /*  5 */
+    { LSOF_FID_FA,     0,  LSOF_FNM_FA,     &Fsv,     FSV_FA	 }, /*  6 */
+    { LSOF_FID_FG,     0,  LSOF_FNM_FG,     &Fsv,     FSV_FG	 }, /*  7 */
+    { LSOF_FID_INODE,  0,  LSOF_FNM_INODE,  NULL,     0		 }, /*  8 */
+    { LSOF_FID_NLINK,  0,  LSOF_FNM_NLINK,  &Fnlink,  1		 }, /*  9 */
+    { LSOF_FID_LOCK,   0,  LSOF_FNM_LOCK,   NULL,     0		 }, /* 10 */
+    { LSOF_FID_LOGIN,  0,  LSOF_FNM_LOGIN,  NULL,     0		 }, /* 11 */
+    { LSOF_FID_MARK,   1,  LSOF_FNM_MARK,   NULL,     0		 }, /* 12 */
+    { LSOF_FID_NAME,   0,  LSOF_FNM_NAME,   NULL,     0		 }, /* 13 */
+    { LSOF_FID_NI,     0,  LSOF_FNM_NI,     &Fsv,     FSV_NI	 }, /* 14 */
+    { LSOF_FID_OFFSET, 0,  LSOF_FNM_OFFSET, NULL,     0		 }, /* 15 */
+    { LSOF_FID_PID,    1,  LSOF_FNM_PID,    NULL,     0		 }, /* 16 */
+    { LSOF_FID_PGRP,   0,  LSOF_FNM_PGRP,   &Fpgrp,   1		 }, /* 17 */
+    { LSOF_FID_PROTO,  0,  LSOF_FNM_PROTO,  NULL,     0		 }, /* 18 */
+    { LSOF_FID_PPID,   0,  LSOF_FNM_PPID,   &Fppid,   1		 }, /* 19 */
+    { LSOF_FID_SIZE,   0,  LSOF_FNM_SIZE,   NULL,     0		 }, /* 20 */
+    { LSOF_FID_STREAM, 0,  LSOF_FNM_STREAM, NULL,     0		 }, /* 21 */
+    { LSOF_FID_TYPE,   0,  LSOF_FNM_TYPE,   NULL,     0		 }, /* 22 */
+    { LSOF_FID_TCPTPI, 0,  LSOF_FNM_TCPTPI, &Ftcptpi, TCPTPI_ALL }, /* 23 */
+    { LSOF_FID_UID,    0,  LSOF_FNM_UID,    NULL,     0		 }, /* 24 */
+    { LSOF_FID_TERM,   0,  LSOF_FNM_TERM,   NULL,     0		 }, /* 25 */
 
 #if	defined(HASFIELDAP1)
-	{ '1',			0,	HASFIELDAP1},		/* TERM + 1 */
+    { '1',	       0,  HASFIELDAP1,     NULL,     0		 }, /* TERM+1 */
 #endif	/* defined(HASFIELDAP1) */
 
 #if	defined(HASFIELDAP2)
-	{ '2',			0,	HASFIELDAP2},		/* TERM + 2 */
+    { '2',	       0,  HASFIELDAP2,     NULL,     0		 }, /* TERM+2 */
 #endif	/* defined(HASFIELDAP2) */
 
 #if	defined(HASFIELDAP3)
-	{ '3',			0,	HASFIELDAP3},		/* TERM + 3 */
+    { '3',	       0,  HASFIELDAP3,     NULL,     0		 }, /* TERM+3 */
 #endif	/* defined(HASFIELDAP3) */
 
 #if	defined(HASFIELDAP4)
-	{ '4',			0,	HASFIELDAP4},		/* TERM + 4 */
+    { '4',	       0,  HASFIELDAP4,     NULL,     0		 }, /* TERM+4 */
 #endif	/* defined(HASFIELDAP4) */
 
 #if	defined(HASFIELDAP5)
-	{ '5',			0,	HASFIELDAP5},		/* TERM + 5 */
+    { '5',	       0,  HASFIELDAP5,     NULL,     0		 }, /* TERM+5 */
 #endif	/* defined(HASFIELDAP5) */
 
 #if	defined(HASFIELDAP6)
-	{ '6',			0,	HASFIELDAP6},		/* TERM + 6 */
+    { '6',	       0,  HASFIELDAP6,     NULL,     0		 }, /* TERM+6 */
 #endif	/* defined(HASFIELDAP6) */
 
 #if	defined(HASFIELDAP7)
-	{ '7',			0,	HASFIELDAP7},		/* TERM + 7 */
+    { '7',	       0,  HASFIELDAP7,     NULL,     0		 }, /* TERM+7 */
 #endif	/* defined(HASFIELDAP7) */
 
 #if	defined(HASFIELDAP8)
-	{ '8',			0,	HASFIELDAP8},		/* TERM + 8 */
+    { '8',	       0,  HASFIELDAP8,     NULL,     0		 }, /* TERM+8 */
 #endif	/* defined(HASFIELDAP8) */
 
 #if	defined(HASFIELDAP9)
-	{ '9',			0,	HASFIELDAP9},		/* TERM + 9 */
+    { '9',	       0,  HASFIELDAP9,     NULL,     0		 }, /* TERM+9 */
 #endif	/* defined(HASFIELDAP9) */
 
-	{ ' ',			0,	NULL		}
+    { ' ',	       0,  NULL,	    NULL,     0		 }
 };
 
 int Hdr = 0;			/* header print status */
@@ -237,7 +263,10 @@ struct NLIST_TYPE *Nl = (struct NLIST_TYPE *)NULL;
 int Nll = 0;			/* Nl calloc'd length */
 #endif	/* defined(HASNLIST) */
 
+long Nlink = 0l;		/* report nlink values below this number
+				 * (0 = report all nlink values) */
 int Nlproc = 0;			/* number of entries in Lproc[] */
+int NlColW;			/* NLINK column width */
 int NmColW;			/* NAME column width */
 char *Nmlst = (char *)NULL;	/* namelist file path */
 int NodeColW;			/* NODE column width */
