@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1998 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: usage.c,v 1.4 98/07/17 08:25:11 abe Exp $";
+static char *rcsid = "$Id: usage.c,v 1.11 99/07/19 13:59:58 abe Exp $";
 #endif
 
 
@@ -114,24 +114,38 @@ usage(xv, fh, version)
 	    (void) fprintf(stderr, " [-A A]");
 #endif	/* defined(HAS_AFS) && defined(HASAOPT) */
 
-	    (void) fprintf(stderr, " [-c c] [-d s]");
+	    (void) fprintf(stderr, " [-c c] [+|-d s] [+%sD D]",
 
 #if	defined(HASDCACHE)
-	    (void) fprintf(stderr, " [-D D]");
+		"|-"
+#else	/* !defined(HASDCACHE) */
+		""
 #endif	/* defined(HASDCACHE) */
 
-	    (void) fprintf(stderr, " [+|-f] [-F [f]]\n [-g [s]] [-i [i]]");
+		);
+
+	    (void) fprintf(stderr, " [+|-f%s]\n [-F [f]] [-g [s]] [-i [i]]",
+
+#if	defined(HASFSTRUCT)
+		"[cfgGn]"
+#else	/* !defined(HASFSTRUCT) */
+		""
+#endif	/* defined(HASFSTRUCT) */
+
+		);
 
 #if	defined(HASKOPT)
 	    (void) fprintf(stderr, " [-k k]");
 #endif	/* defined(HASKOPT) */
 
+	    (void) fprintf(stderr, " [+|-L [l]]");
+
 #if	defined(HASMOPT)
 	    (void) fprintf(stderr, " [-m m]");
 #endif	/* defined(HASMOPT) */
 
 	    (void) fprintf(stderr,
-	    " [+|-M] [-o [o]] [-p s] [+|-r [t]] [-S [t]]\n [-T [t]]");
+	    " [+|-M] [-o [o]] [-p s]\n [+|-r [t]] [-S [t]] [-T [t]]");
 	    (void) fprintf(stderr,
 	    " [-u s] [+|-w] [--] [names]\n");
 	}
@@ -144,14 +158,10 @@ usage(xv, fh, version)
 	if (Fhelp) {
 	    (void) fprintf(stderr,
 		"Defaults in parentheses; comma-separate set (s) items; dash-separate ranges.\n");
-	    (void) fprintf(stderr, "  %-23.23s",
-		"-? list help");
-	    (void) fprintf(stderr, "  %-25.25s",
-		"-a AND selections (OR)");
-	    (void) fprintf(stderr, "  %s\n",
-		"-b avoid kernel blocks");
-	    (void) fprintf(stderr, "  %-23.23s",
-		"-c c  list command c");
+	    (void) fprintf(stderr, "  %-23.23s", "-?|-h list help");
+	    (void) fprintf(stderr, "  %-25.25s", "-a AND selections (OR)");
+	    (void) fprintf(stderr, "  %s\n", "-b avoid kernel blocks");
+	    (void) fprintf(stderr, "  %-23.23s", "-c c  list command c");
 	    (void) fprintf(stderr, "  %-25.23s",
 
 #if	defined(HASNCACHE)
@@ -160,7 +170,9 @@ usage(xv, fh, version)
 		" ");
 #endif	/* defined(HASNCACHE) */
 
-	    (void) fprintf(stderr, "  %s\n", "-d s  select by FD set");
+	    (void) fprintf(stderr, "  %s\n", "+d s  dir s files");
+	    (void) fprintf(stderr, "  %-23.23s", "-d s  select by FD set");
+	    (void) fprintf(stderr, "  %-25.25s", "+D D  dir D tree *SLOW?*");
 
 #if	defined(HASDCACHE)
 	    if (Setuidroot)
@@ -178,9 +190,7 @@ usage(xv, fh, version)
 	    (void) strcpy(buf, " ");
 #endif	/* defined(HASDCACHE) */
 
-	    (void) fprintf(stderr, "  %-23.23s", buf);
-	    (void) fprintf(stderr, "  %-25.25s", "+|-f -files +filesys");
-	    (void) fprintf(stderr, "  %s\n", "-h list help");
+	    (void) fprintf(stderr, "  %s\n", buf);
 	    (void) sprintf(buf, "-i select IPv%s files",
 
 #if	defined(HASIPv6)
@@ -200,9 +210,9 @@ usage(xv, fh, version)
 	    (void) fprintf(stderr, "  %-25.25s",
 
 #if	defined(HASPPID)
-		"-R list paRent PID"
+	 	"-R list paRent PID"
 #else	/* !defined(HASPPID) */
-		" "
+		""
 #endif	/* defined(HASPPID) */
 
 	    );
@@ -210,7 +220,7 @@ usage(xv, fh, version)
 	    (void) fprintf(stderr, "  %-23.23s", "-t terse listing");
 	    (void) fprintf(stderr, "  %-25.25s", "-T disable TCP/TPI info");
 	    (void) fprintf(stderr, "  %s\n", "-U select Unix socket");
-	    (void) fprintf(stderr, "  %-23.23s", "-v display version info");
+	    (void) fprintf(stderr, "  %-23.23s", "-v list version info");
 	    (void) fprintf(stderr, "  %-25.25s", "-V verbose search");
 	    (void) sprintf(buf, "+|-w  Warnings (%s)",
 
@@ -226,12 +236,31 @@ usage(xv, fh, version)
 #if	defined(HASXOPT)
 # if	defined(HASXOPT_ROOT)
 	    if (Myuid == 0)
-		(void) fprintf(stderr, "  -X %s\n", HASXOPT);
+		(void) sprintf(buf, "-X %s", HASXOPT);
 # else	/* !defined(HASXOPT_ROOT) */
-	    (void) fprintf(stderr, "  -X %s\n", HASXOPT);
+	    (void) sprintf(buf, "-X %s", HASXOPT);
 # endif	/* defined(HASXOPT_ROOT) */
+	    (void) fprintf(stderr, "  %-23.23s", buf);
 #endif	/* defined(HASXOPT) */
 
+	    (void) fprintf(stderr, "  %s\n", "-- end option scan");
+	    (void) fprintf(stderr, "  %-36.36s",
+		"+f|-f  +filesystem or -file names");
+
+#if	defined(HASFSTRUCT)
+	    (void) fprintf(stderr,
+		"  +|-f[cfgGn] Ct,Fstr,flaGs,Node %s%s%s%s%s%s%s\n",
+		Fsv ? "(" : "",
+		(Fsv & FSV_CT) ? "C" : "",
+		(Fsv & FSV_FA) ? "F" : "",
+		((Fsv & FSV_FG) && FsvFlagX)  ? "g" : "",
+		((Fsv & FSV_FG) && !FsvFlagX) ? "G" : "",
+		(Fsv & FSV_NI) ? "N" : "",
+		Fsv ? ")" : "");
+#else	/* !defined(HASFSTRUCT) */
+	    putc('\n', stderr);
+#endif	/* defined(HASFSTRUCT) */
+
 	    (void) fprintf(stderr, "  %-36.36s",
 		"-F [f] select fields; -F? for help");
 
@@ -251,6 +280,9 @@ usage(xv, fh, version)
 	    putc('\n', stderr);
 #endif	/* defined(HASKOPT) */
 
+	    (void) fprintf(stderr,
+		"  +|-L [l] list (+) suppress (-) link counts < l (0 = all; default = 0)\n");
+
 # if	defined(HASMOPT)
 	    (void) sprintf(buf, "-m m   kernel memory (%s)", KMEM);
 #else	/* !defined(HASMOPT) */
@@ -303,9 +335,6 @@ usage(xv, fh, version)
 
 	    );
 	    (void) fprintf(stderr, "  %s\n", buf);
-	    (void) fprintf(stderr, "  %-36.36s",
-		"-u s   exclude(^)/select login/UID set\n");
-	    (void) fprintf(stderr, "  --     end option scan\n");
 
 #if	defined(HAS_AFS) && defined(HASAOPT)
 	    (void) fprintf(stderr,
@@ -329,6 +358,8 @@ usage(xv, fh, version)
 	    (void) fprintf(stderr,
 		"  +|-r [t] repeat every t seconds (%d);", RPTTM);
 	    (void) fprintf(stderr, " + until no files, - forever\n");
+	    (void) fprintf(stderr,
+		"  -u s   exclude(^)|select login|UID set s\n");
 	    (void) fprintf(stderr,
 		"  names  select named files or files on named file systems\n");
 	    (void) fprintf(stderr, "%s can list all files;",
@@ -390,6 +421,14 @@ usage(xv, fh, version)
 		    continue;
 #endif	/* !defined(HASPPID) */
 
+#if	!defined(HASFSTRUCT)
+		if (FieldSel[i].id == LSOF_FID_FA
+		||  FieldSel[i].id == LSOF_FID_CT
+		||  FieldSel[i].id == LSOF_FID_FG
+		||  FieldSel[i].id == LSOF_FID_NI)
+		    continue;
+#endif	/* !defined(HASFSTRUCT) */
+
 		(void) fprintf(stderr, "\t %c    %s\n",
 		    FieldSel[i].id, FieldSel[i].nm);
 	    }
@@ -475,40 +514,46 @@ usage(xv, fh, version)
 	/*
 	 * Display version information in reponse to ``-v''.
 	 */
-		(void) fprintf(stderr, "%s version information:\n", Pn);
-		(void) fprintf(stderr,
-			"\trevision: %s -- find the latest revision at:\n\t",
-			LSOF_VERSION);
-		(void) fprintf(stderr, "\t%s\n", LSOF_URL);
-		if ((cp = isnullstr(LSOF_CCDATE)))
-			(void) fprintf(stderr, "\tconstructed: %s\n", cp);
-		cp = isnullstr(LSOF_HOST);
-		if (!(cp1 = isnullstr(LSOF_LOGNAME)))
-			cp1 = isnullstr(LSOF_USER);
-		if (cp || cp1) {
-			if (cp && cp1)
-				cp2 = "by and on";
-			else if (cp)
-				cp2 = "on";
-			else
-				cp2 = "by";
-			(void) fprintf(stderr, "\tconstructed %s: %s%s%s\n",
-				cp2,
-				cp1 ? cp1 : "",
-				cp1 ? "@" : "",
-				cp  ? cp  : ""
-			);
-		}
-		if ((cp = isnullstr(LSOF_CC)))
-			(void) fprintf(stderr, "\tcompiler: %s\n", cp);
-		if ((cp = isnullstr(LSOF_CCV)))
-			(void) fprintf(stderr, "\tcompiler version: %s\n", cp);
-		if ((cp = isnullstr(LSOF_CCFLAGS)))
-			(void) fprintf(stderr, "\tcompiler flags: %s\n", cp);
-		if ((cp = isnullstr(LSOF_LDFLAGS)))
-			(void) fprintf(stderr, "\tloader flags: %s\n", cp);
-		if ((cp = isnullstr(LSOF_SYSINFO)))
-			(void) fprintf(stderr, "\tsystem info: %s\n", cp);
+	    (void) fprintf(stderr, "%s version information:\n", Pn);
+	    (void) fprintf(stderr,
+		"    revision: %s -- find the latest revision at:\n    ",
+		LSOF_VERSION);
+	    (void) fprintf(stderr, "    %s\n", LSOF_URL);
+
+#if	defined(LSOF_CINFO)
+	    if ((cp = isnullstr(LSOF_CINFO)))
+		(void) fprintf(stderr, "    configuration info: %s\n", cp);
+#endif	/* defined(LSOF_CINFO) */
+
+	    if ((cp = isnullstr(LSOF_CCDATE)))
+		(void) fprintf(stderr, "    constructed: %s\n", cp);
+	    cp = isnullstr(LSOF_HOST);
+	    if (!(cp1 = isnullstr(LSOF_LOGNAME)))
+		cp1 = isnullstr(LSOF_USER);
+	    if (cp || cp1) {
+		if (cp && cp1)
+		    cp2 = "by and on";
+		else if (cp)
+		    cp2 = "on";
+		else
+		    cp2 = "by";
+		(void) fprintf(stderr, "    constructed %s: %s%s%s\n",
+		    cp2,
+		    cp1 ? cp1 : "",
+		    cp1 ? "@" : "",
+		    cp  ? cp  : ""
+		);
+	    }
+	    if ((cp = isnullstr(LSOF_CC)))
+		(void) fprintf(stderr, "    compiler: %s\n", cp);
+	    if ((cp = isnullstr(LSOF_CCV)))
+		(void) fprintf(stderr, "    compiler version: %s\n", cp);
+	    if ((cp = isnullstr(LSOF_CCFLAGS)))
+		(void) fprintf(stderr, "    compiler flags: %s\n", cp);
+	    if ((cp = isnullstr(LSOF_LDFLAGS)))
+		(void) fprintf(stderr, "    loader flags: %s\n", cp);
+	    if ((cp = isnullstr(LSOF_SYSINFO)))
+		(void) fprintf(stderr, "    system info: %s\n", cp);
 	}
 	Exit(xv);
 }
